SystemOrganization addCategory: #'SWHTTPClient-Core'!
SystemOrganization addCategory: #'SWHTTPClient-Tests'!
SystemOrganization addCategory: #'SWHTTPClient-Support'!
SystemOrganization addCategory: #'SWHTTPClient-Compatibility'!
SystemOrganization addCategory: #'SWHTTPClient-Streams'!

Error subclass: #SptHTTPNotAuthorized
	instanceVariableNames: 'target challenge site '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPNotAuthorized methodsFor: 'accessing' stamp: ''!
target
	^target! !

!SptHTTPNotAuthorized methodsFor: 'displaying' stamp: 'avi 9/29/2003 13:02'!
descriptionArguments
	"Answer the set of arguments to be substitued into the receiver's descriptionFormat
	String."

	^super descriptionArguments , (Array with: site displayString)! !

!SptHTTPNotAuthorized methodsFor: 'displaying' stamp: 'avi 9/29/2003 13:01'!
descriptionFormat
	"Answer the Win32 format String to be used to format the description for the receiver."

	^'HTTP Not Authorized for %3'! !

!SptHTTPNotAuthorized methodsFor: 'operations' stamp: ''!
username: aString password: aString2
	target notNil 
		ifTrue: [target credentials: (challenge credentialsForUsername: aString password: aString2)]! !

!SptHTTPNotAuthorized methodsFor: 'accessing' stamp: ''!
site
	^site! !

!SptHTTPNotAuthorized class methodsFor: 'instance creation' stamp: ''!
newForRealm: aString absoluteUri: aUrlString
	^(self new)
		challenge: (SptHTTPChallenge newBasicChallengeRealm: aString absoluteUri: aUrlString);
		site: aUrlString;
		yourself! !

!SptHTTPNotAuthorized methodsFor: 'accessing' stamp: ''!
realm
	^challenge realm! !

!SptHTTPNotAuthorized methodsFor: 'accessing' stamp: ''!
challenge
	^challenge! !

!SptHTTPNotAuthorized methodsFor: 'accessing' stamp: ''!
target: anObject
	target := anObject! !

!SptHTTPNotAuthorized methodsFor: 'accessing' stamp: ''!
challenge: aSptHTTPChallenge
	challenge := aSptHTTPChallenge! !

!SptHTTPNotAuthorized methodsFor: 'operations' stamp: ''!
removeCredentials
	target notNil ifTrue: [target credentials: nil]! !

!SptHTTPNotAuthorized methodsFor: 'accessing' stamp: ''!
site: aString
	site := aString! !

Error subclass: #SptHTTPError
	instanceVariableNames: 'httpStatus '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPError methodsFor: 'accessing' stamp: ''!
httpStatus
	^httpStatus! !

!SptHTTPError methodsFor: 'accessing' stamp: ''!
httpStatus: aSptHTTPStatus
	httpStatus := aSptHTTPStatus.
	messageText := httpStatus displayString! !

!SptHTTPError methodsFor: 'accessing' stamp: ''!
statusCode
	"Answer the status code of the response"

	^self httpStatus statusCode! !

!SptHTTPError methodsFor: 'displaying' stamp: ''!
descriptionArguments
	"Answer the set of arguments to be substitued into the receiver's descriptionFormat
	String."

	^super descriptionArguments , (Array with: httpStatus displayString)! !

!SptHTTPError methodsFor: 'accessing' stamp: ''!
statusText
	"Answer the Status text of the response"

	^self httpStatus statusText! !

!SptHTTPError methodsFor: 'displaying' stamp: 'avi 9/29/2003 13:01'!
descriptionFormat
	"Answer the Win32 format String to be used to format the description for the receiver."

	^'HTTP Error %3'! !

SptHTTPError subclass: #SptHTTPClientError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPClientError methodsFor: 'displaying' stamp: ''!
descriptionFormat
	"Answer the Win32 format String to be used to format the description for the receiver."

	^'Client Error %3'! !

SptHTTPError subclass: #SptHTTPServerError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPServerError methodsFor: 'displaying' stamp: 'avi 9/29/2003 13:02'!
descriptionFormat
	"Answer the Win32 format String to be used to format the description for the receiver."

	^'Server Error %3'! !

Object subclass: #SptHTTPProgressAbstract
	instanceVariableNames: 'request '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPProgressAbstract commentStamp: '<historical>' prior: 0!
Instance Variables
	request 	<SptHTTPRequest>!

!SptHTTPProgressAbstract methodsFor: 'accessing' stamp: ''!
request: anObject
	request := anObject! !

SptHTTPProgressAbstract subclass: #SptHTTPProgress
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPProgress commentStamp: '<historical>' prior: 0!
This is a stub class that implements all the messages a request's progress can receive.

	See: SptHTTPRequest(class)>>exampleProgress

Subclasses can implement the methods that they are interested in.

Two subclasses are provided:
	SptHTTPTriggerProgress ... for triggering notifications of the % of a download/upload complete
	SptHTTPValueProgress ... sets string descriptions of progress into a value model!

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onReadingHeaders
	"Note: In the case of chunked transfer encoding, this will be sent both before and after reading the body"! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onReadingBody! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onReadStatus
	"Note: This may be sent more than once if the server sends a informational CONTINUE response, "! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onSentHeaders! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onConnected! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onReadBody! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onCompleted! !

!SptHTTPProgress methodsFor: 'accessing' stamp: ''!
sent: cumulativeNumOfBytes of: totalNumOfBytes! !

SptHTTPProgress subclass: #SptHTTPTriggerProgress
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPTriggerProgress methodsFor: 'accessing' stamp: ''!
progressChanged: aNumber
	request trigger: #progressChanged: with: aNumber
	"valueModel value: aNumber"! !

!SptHTTPTriggerProgress methodsFor: 'accessing' stamp: ''!
sent: cumulativeNumOfBytes of: totalNumOfBytes
	self progressChanged: cumulativeNumOfBytes * 100 // totalNumOfBytes! !

!SptHTTPTriggerProgress methodsFor: 'accessing' stamp: 'avi 9/29/2003 13:02'!
read: cumulativeNumOfBytes of: totalNumOfBytes
	"totalNumOfBytesOrNil will be nil unless we received a content-length header.
		-For HTTP1.0 this can be common ... we are expected to read from the server until it closes its end of the socket
		-For HTTP1.1 this will occur if the server is using chunked transfer-encoding"

	| total |
	total := totalNumOfBytes ifNil: [50000] ifNotNil: [totalNumOfBytes].
	self progressChanged: cumulativeNumOfBytes * 100 // total! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onReadHeaders
	"Note: In the case of chunked transfer encoding, this will be sent both before and after reading the body"! !

!SptHTTPProgress methodsFor: 'accessing' stamp: ''!
read: cumulativeNumOfBytes of: totalNumOfBytesOrNil
	"totalNumOfBytesOrNil will be nil unless we received a content-length header.
		-For HTTP1.0 this can be common ... we are expected to read from the server until it closes its end of the socket
		-For HTTP1.1 this will occur if the server is using chunked transfer-encoding"! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onTimeout! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onSendingHeaders! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onSendingEntity! !

!SptHTTPProgress methodsFor: 'deprecated' stamp: ''!
progressChanged: aNumber
	Notification deprecated! !

SptHTTPProgress subclass: #SptHTTPValueProgress
	instanceVariableNames: 'percentageValue stateValue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPValueProgress methodsFor: 'accessing' stamp: ''!
stateValue
	^stateValue! !

!SptHTTPValueProgress methodsFor: 'accessing' stamp: 'avi 9/29/2003 13:02'!
read: cumulativeNumOfBytes of: totalNumOfBytes
	"totalNumOfBytesOrNil will be nil unless we received a content-length header.
		-For HTTP1.0 this can be common ... we are expected to read from the server until it closes its end of the socket
		-For HTTP1.1 this will occur if the server is using chunked transfer-encoding"

	| total |
	total := totalNumOfBytes ifNil: [50000] ifNotNil: [totalNumOfBytes].
	self progressChanged: cumulativeNumOfBytes * 100 // total! !

!SptHTTPValueProgress methodsFor: 'accessing' stamp: ''!
sent: cumulativeNumOfBytes of: totalNumOfBytes
	self progressChanged: cumulativeNumOfBytes * 100 // totalNumOfBytes! !

!SptHTTPValueProgress methodsFor: 'event handling' stamp: ''!
onReadingBody
	stateValue value: 'Reading'! !

!SptHTTPValueProgress methodsFor: 'event handling' stamp: ''!
onReadingHeaders
	stateValue value: 'Reading'! !

!SptHTTPValueProgress methodsFor: 'initializing' stamp: ''!
initialize
	super initialize.
	percentageValue := ValueHolder new.
	stateValue := ValueHolder new! !

!SptHTTPValueProgress methodsFor: 'event handling' stamp: ''!
onConnected
	stateValue value: 'Connected'! !

!SptHTTPValueProgress methodsFor: 'accessing' stamp: ''!
stateValue: anObject
	stateValue := anObject! !

!SptHTTPValueProgress methodsFor: 'event handling' stamp: ''!
onConnecting
	stateValue value: 'Connecting'! !

!SptHTTPValueProgress methodsFor: 'event handling' stamp: ''!
onSendingHeaders
	stateValue value: 'Sending'! !

!SptHTTPValueProgress methodsFor: 'accessing' stamp: ''!
percentageValue
	^percentageValue! !

!SptHTTPValueProgress methodsFor: 'event handling' stamp: ''!
onSendingEntity
	stateValue value: 'Sending'! !

!SptHTTPValueProgress methodsFor: 'event handling' stamp: ''!
onCompleted
	percentageValue value: 0.
	stateValue value: ''! !

!SptHTTPValueProgress methodsFor: 'accessing' stamp: ''!
progressChanged: aNumber
	percentageValue value: aNumber! !

!SptHTTPValueProgress methodsFor: 'accessing' stamp: ''!
percentageValue: anObject
	percentageValue := anObject! !

!SptHTTPProgress methodsFor: 'initializing' stamp: ''!
initialize
	super initialize! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onReadingStatus
	"Note: This may be sent more than once if the server sends a informational CONTINUE response, "! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onRedirecting! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onSentEntity! !

!SptHTTPProgress methodsFor: 'event handling' stamp: ''!
onConnecting! !

!SptHTTPProgressAbstract class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

SptHTTPProgressAbstract subclass: #SptHTTPProgressDev
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPProgressDev commentStamp: '<historical>' prior: 0!
These classes are used for development and testing purposes.

see SptHTTPRequest(class)>>exampleProgress!

SptHTTPProgressDev subclass: #SptHTTPTranscriptProgress
	instanceVariableNames: 'stream nextProgress '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPTranscriptProgress methodsFor: 'accessing' stamp: ''!
stream: anObject
	stream := anObject! !

!SptHTTPTranscriptProgress methodsFor: 'initializing' stamp: ''!
initialize
	super initialize.
	stream := Transcript! !

!SptHTTPTranscriptProgress methodsFor: 'accessing' stamp: ''!
nextProgress: anObject
	nextProgress := anObject! !

!SptHTTPTranscriptProgress methodsFor: 'exceptions' stamp: 'avi 9/29/2003 13:01'!
doesNotUnderstand: aMessage
	|conn|
	conn _ request connection.
	stream
		nextPutAll: 'SptHTTPTranscriptProgress: ' , request hash displayString , ' ' 
					, (conn
							ifNil: ['nil']
							ifNotNil: [conn socket displayString , conn socket hash displayString]) 
						, ' ' , aMessage selector printString 
					, ' ' , aMessage arguments printString;
		cr.
	nextProgress notNil ifTrue: [aMessage forwardTo: nextProgress]! !

SptHTTPProgressDev subclass: #SptHTTPTestProgress
	instanceVariableNames: 'sentMessages '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPTestProgress methodsFor: 'initializing' stamp: ''!
initialize
	super initialize.
	sentMessages := OrderedCollection new! !

!SptHTTPTestProgress methodsFor: 'accessing' stamp: ''!
sentMessages
	^sentMessages! !

!SptHTTPTestProgress methodsFor: 'exceptions' stamp: ''!
doesNotUnderstand: aMessage
	sentMessages add: aMessage! !

!SptHTTPProgressAbstract methodsFor: 'accessing' stamp: ''!
request
	^request! !

!SptHTTPProgressAbstract methodsFor: 'accessing' stamp: ''!
response
	^request lastResponse! !

Object subclass: #SptHTTPHostCache
	instanceVariableNames: 'cache ipFactory '
	classVariableNames: 'Current '
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPHostCache commentStamp: '<historical>' prior: 0!
SptHTTPHostCache

I am an optimization class; I cache the ip address of host to avoid a DNS lookup

Instance Variables
	cache		<SharedLookupTable>
	ipFactory		<SptHTTPHostCache>!

!SptHTTPHostCache methodsFor: 'entries' stamp: ''!
newEntryForIP: anIPAddress
	^Array with: Delay millisecondClockValue with: anIPAddress! !

!SptHTTPHostCache class methodsFor: 'accessing' stamp: ''!
current
	Current isNil ifTrue: [Current := self new].
	^Current! !

!SptHTTPHostCache methodsFor: 'caching' stamp: ''!
hasExpired: entry
	^Delay millisecondClockValue - entry first > self freshMilliseconds! !

!SptHTTPHostCache methodsFor: 'accessing' stamp: ''!
ipFactory
	^ipFactory! !

!SptHTTPHostCache methodsFor: 'caching' stamp: ''!
ipAddressFor: aString
	| entry |
	entry := cache at: aString ifAbsent: [].
	(entry isNil or: [self hasExpired: entry]) 
		ifTrue: [cache at: aString put: (entry := self newEntryFor: aString)].
	^entry second! !

!SptHTTPHostCache methodsFor: 'accessing' stamp: ''!
ipFactory: anObject
	ipFactory := anObject! !

!SptHTTPHostCache methodsFor: 'initializing' stamp: ''!
initialize
	super initialize.
	cache := SharedLookupTable new.
	ipFactory := self! !

!SptHTTPHostCache methodsFor: 'accessing' stamp: ''!
cache: anObject
	cache := anObject! !

!SptHTTPHostCache methodsFor: 'accessing' stamp: ''!
cache
	^cache! !

!SptHTTPHostCache methodsFor: 'helpers' stamp: ''!
ipAddressFromString: aString
	^(InternetAddress fromString: aString) ipAddress! !

!SptHTTPHostCache methodsFor: 'constants' stamp: 'avi 9/29/2003 14:49'!
freshMilliseconds
	^1000 * 60 * 2! !

!SptHTTPHostCache class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!SptHTTPHostCache class methodsFor: 'initializing' stamp: ''!
uninitialize
	Current := nil! !

!SptHTTPHostCache methodsFor: 'entries' stamp: ''!
newEntryFor: aString
	^self newEntryForIP: (ipFactory ipAddressFromString: aString)! !

Object subclass: #SptHTTPResponse
	instanceVariableNames: 'statusLine headers bodyStream request '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPResponse commentStamp: '<historical>' prior: 0!
I am an HTTP Response message.

I can read responses that either;
  1/ Have a content-Length
  2/ Use chunked transer encoding
  3/ Have a Content-Range
  Or (if HTTP 1.0)  I will read the socket until it is closed by the server

Notes:
 - I will lazily initialize bodyStream to be a <WriteStream> on a ByteArray, however when I read a 200 response, I will ask my <SptHTTPRequest> request for its okBodyStream and if not nil, I will stream to it.

 - If 
        1/ I receive a 'Content-Range' response (typically in response to a request with a 'Range' header) and If
        2/ My Request answers true to isRangeRequest
   Then I will set the position of the bodyStream to the start of the Content-Range



Instance Variables:
	statusLine	<String>
	headers		<PluggableLookupTable> (case-insensitive)
	cookies		<OrderedCollection> of <SptHTTPCookie>s
	bodyStream	<WriteStream>
	request		<SptHTTPRequest>!

!SptHTTPResponse methodsFor: 'accessing-headers' stamp: ''!
basicChallenge
	| header challenges |
	header := headers at: 'WWW-Authenticate' ifAbsent: [^nil].
	challenges := SptHTTPChallenge readAllFrom: header readStream.
	^challenges detect: [:each | each isBasic] ifNone: []! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isHTMLContentType
	^(self headerAt: 'Content-Type' ifAbsent: [^false]) beginsWith: 'text/html'! !

!SptHTTPResponse methodsFor: 'accessing' stamp: ''!
request
	^request! !

!SptHTTPResponse methodsFor: 'operations' stamp: ''!
setRequestBodyStream
	request isNil ifTrue: [^nil].
	"If the <SptHTTPRequest> has a nonNil bodyStream use it, otherwise set it"
	request okBodyStream notNil 
		ifTrue: [bodyStream := request okBodyStream]
		ifFalse: [request okBodyStream: bodyStream]! !

!SptHTTPResponse methodsFor: 'accessing' stamp: ''!
makingProgress
	"Answer the request's <SptHTTPProgress> progress.

	Inform the <SptHTTPRequest> request that we are 'making progress' ...
		(ie we have either read or written something to the socket.

	Note: DeafObject is used for UnitTesting purposes "

	^request isNil ifTrue: [DeafObject current] ifFalse: [request makingProgress]! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isConnectionClose
	^(headers at: 'Connection' ifAbsent: [^self defaultIsConnectionClose]) asLowercase 
		= 'close'! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
hasContentRange
	^headers includesKey: 'Content-Range'! !

!SptHTTPResponse methodsFor: 'accessing' stamp: ''!
defaultIsConnectionClose
	^[self isVersion11 not] on: Error do: [:e | true]! !

!SptHTTPResponse methodsFor: 'accessing' stamp: ''!
readEntityBufferSize
	request ifNil: 
			["For UnitTests"

			^1024].
	^request readEntityBufferSize! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isSuccessful
	^self httpStatus isSuccessful! !

!SptHTTPResponse methodsFor: 'operations' stamp: ''!
readChunkedEncodedBody: aConnection
	"The response is using chunked encoding
		... read the chunks"

	| chunkSize stream read |
	self makingProgress onReadingBody.
	stream := aConnection socketStream.
	read := 0.
	
	[chunkSize := self chunkSizeFrom: stream.
	chunkSize > 0] whileTrue: 
				[| crlf |
				bodyStream nextPutAll: (stream nextBytes: chunkSize).
				crlf := stream next: 2.
				self assert: [crlf = String lineDelimiter].
				self makingProgress read: (read := read + chunkSize) of: nil].
	self readMessageHeaders: aConnection.
	self makingProgress onReadBody! !

!SptHTTPResponse methodsFor: 'operations' stamp: ''!
readMessageStatus: aConnection
	| line stream |
	stream := aConnection socketStream.
	self makingProgress onReadingStatus.
	[(line := stream upToCRLF) isEmpty] whileTrue: [line := stream upToCRLF].
	statusLine := line.
	self makingProgress onReadStatus! !

!SptHTTPResponse methodsFor: 'accessing-status' stamp: ''!
statusLine: aString
	"For example 'HTTP/1.1 200 OK'"

	statusLine := aString! !

!SptHTTPResponse methodsFor: 'unit tests' stamp: ''!
setStatus: aString headers: aLookupTable body: aString2
	"Private - For Unit tests"

	statusLine := aString.
	headers := aLookupTable.
	bodyStream := ByteArray writeStream.
	bodyStream nextPutAll: aString2 asByteArray! !

!SptHTTPResponse methodsFor: 'deprecated' stamp: ''!
hasRange
	Notification deprecated.
	^headers includesKey: 'Range'! !

!SptHTTPResponse methodsFor: 'accessing-headers' stamp: ''!
headerAt: aString
	^headers at: aString! !

!SptHTTPResponse methodsFor: 'helpers' stamp: ''!
chunkSizeFrom: stream
	| chunkString |
	chunkString := (stream upToCRLF readStream upTo: $;) asUppercase.
	"Transcript
		nextPutAll: chunkString;
		cr."
	^Integer readFrom: chunkString readStream radix: 16! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
canRangeRetry
	"Answer whether we can retry the request, only asking for the range of bytes we dont have"

	#todo.	"handle ChunkedTransferEncoding responses?"
	^self hasStatus and: 
			[self isSuccessful and: 
					[self isAcceptRangesBytes 
						and: [self isChunkedTransferEncoding not and: [self hasContentLength]]]]! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
hasContentLength
	^headers includesKey: 'Content-Length'! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
hasStatus
	^statusLine notNil and: [statusLine trimBlanks notEmpty]! !

!SptHTTPResponse methodsFor: 'accessing-headers' stamp: ''!
contentLength
	^(headers at: 'Content-Length') asNumber! !

!SptHTTPResponse class methodsFor: 'instance creation' stamp: ''!
fromConnection: anSptHTTPConnection request: anObject
	#deprecated.
	^(self new)
		request: anObject;
		readResponse: anSptHTTPConnection;
		yourself! !

!SptHTTPResponse methodsFor: 'accessing-headers' stamp: 'bkv 10/30/2003 18:43'!
contentRangeInterval
	| header stream rangeStart rangeEnd instanceLength |

	self httpStatus isUnsatisfiableRangeRequest ifTrue: [ ^ 0 to: 0 ].

	header := (self headerAt: 'Content-Range') trimBlanks.
	stream := header readStream.
	(stream upTo: $ ) trimBlanks = 'bytes' 
		ifFalse: [self error: 'Unexpected Content-Range header: ' , header displayString].
	rangeStart := (stream upTo: $-) trimBlanks.
	rangeEnd := (stream upTo: $/) trimBlanks.
	instanceLength := stream upToEnd trimBlanks.
	"Ignore the instanceLength ... we only need the range"
	^rangeStart asNumber to: rangeEnd asNumber! !

!SptHTTPResponse methodsFor: 'accessing-content' stamp: ''!
body
	"Answer the body of the response as a <String>"

	^bodyStream contents asString! !

!SptHTTPResponse methodsFor: 'initializing' stamp: 'avi 9/29/2003 13:24'!
initialize
	headers := PluggableLookupTable new searchPolicy: SearchPolicy caseInsensitive.
	bodyStream := ByteArray writeStream! !

!SptHTTPResponse methodsFor: 'accessing-content' stamp: ''!
bodyStream
	^bodyStream! !

!SptHTTPResponse methodsFor: 'accessing' stamp: ''!
putHeaderReportOn: stream
	stream
		nextPutAll: '===In';
		cr;
		display: statusLine;
		cr.
	headers keysAndValuesDo: 
			[:k :v | 
			stream
				display: k;
				nextPut: $:;
				display: v;
				cr]! !

!SptHTTPResponse methodsFor: 'operations' stamp: ''!
readMessageBody: aConnection
	self isSuccessful ifTrue: [self setRequestBodyStream].
	self isChunkedTransferEncoding ifTrue: [^self readChunkedEncodedBody: aConnection].
	self hasContentRange ifTrue: [^self readContentRangeBody: aConnection].
	self hasContentLength ifTrue: [^self readContentLengthBody: aConnection].
	"Only read to socket close if HTTP1.0. It is valid to have a response with no body"
	(self isVersion11 not or: [self isConnectionClose]) 
		ifTrue: 
			[^self readBodyToEnd: aConnection
			"mmm ... no body?"]! !

!SptHTTPResponse methodsFor: 'accessing-status' stamp: ''!
httpStatus
	^SptHTTPStatus fromString: statusLine! !

!SptHTTPResponse methodsFor: 'deprecated' stamp: ''!
onReadHeaders
	Notification deprecated! !

!SptHTTPResponse methodsFor: 'operations' stamp: ''!
readContentRangeBody: aConnection
	"Read Content-Length bytes from the socket (ie HTTP1.1 semantics).
	Note: read readEntityChunkSize bytes at a time so that we can update any progress/status objects."

	| total range read bytes |
	#todo.	"Refactor with readContentLengthBody"
	self makingProgress onReadingBody.
	range := self contentRangeInterval.

	"If the request isRangeRequest, set the position of the okBodyStream"
	request isRangeRequest ifTrue: [bodyStream position: range start].
	total := range stop - range start + 1.
	read := 0.
	[read >= total] whileFalse: 
			[bodyStream nextPutAll: (aConnection socketStream 
						nextBytes: (bytes := self readEntityBufferSize min: total - read)).
			read := read + bytes.
			self makingProgress read: read of: total].
	self makingProgress onReadBody! !

!SptHTTPResponse methodsFor: 'operations' stamp: ''!
readMessageHeaders: aConnection
	| line stream |
	self makingProgress onReadingHeaders.
	stream := aConnection socketStream.
	[(line := stream upToCRLF) isEmpty] whileFalse: [self parseHeaderLine: line].
	self makingProgress onReadHeaders! !

!SptHTTPResponse methodsFor: 'accessing-content' stamp: ''!
bodyBytes
	"Answer the body of the response as a <ByteArray>"

	^bodyStream contents! !

!SptHTTPResponse methodsFor: 'accessing' stamp: ''!
progress
	"Note: DeafObject is used for UnitTesting purposes "

	^request isNil ifTrue: [DeafObject current] ifFalse: [request progress]! !

!SptHTTPResponse methodsFor: 'unit tests' stamp: ''!
setStatusCode: aString headers: aLookupTable body: aString2
	"Private - For Unit tests"

	self 
		setStatus: 'HTTP/1.1 ' , aString
		headers: aLookupTable
		body: aString2! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isChunkedTransferEncoding
	^(headers at: 'transfer-encoding' ifAbsent: [^false]) = 'chunked'! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isUnauthorized
	^self status = 401! !

!SptHTTPResponse methodsFor: 'operations' stamp: ''!
parseHeaderLine: aString
	| stream key value |
	stream := aString readStream.
	key := stream upTo: $:.
	value := stream upToEnd.
	"cookies are a special-case ... we need to parse them to make sure they are valid
	before blindly returning them in a new request"
	key asLowercase = 'set-cookie' ifTrue: [self parseCookie: value].
	(headers includesKey: key) ifTrue: [value := (headers at: key) , ',' , value].
	headers at: key put: value trimBlanks! !

!SptHTTPResponse class methodsFor: 'deprecated' stamp: ''!
readEntityBufferSize
	Notification deprecated.
	^1024! !

!SptHTTPResponse methodsFor: 'accessing' stamp: ''!
bodyStreamSize
	^bodyStream size! !

!SptHTTPResponse methodsFor: 'accessing-status' stamp: ''!
status
	"Answer the status code of the response"

	^self httpStatus status! !

!SptHTTPResponse methodsFor: 'operations' stamp: 'avi 9/30/2003 02:08'!
readBodyToEnd: aConnection
	"Read the socket until it is closed (ie HTTP1.0 semantics.
	Note: read readEntityChunkSize bytes at a time so that we can update any progress/status objects."

	| inStream read |
	self makingProgress onReadingBody.
	read := 0.
	inStream := aConnection socketStream.
	
	[inStream doBytes: 
			[:each | 
			bodyStream nextPut: each.
			read := read + 1.
			read % self readEntityBufferSize = 0 ifTrue: [self makingProgress read: read of: nil]]] 
			on: Error
			do: [:e | ].
	self makingProgress onReadBody! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isRedirect
	^self status >= 300 and: [self status < 400 and: [headers includesKey: 'Location']]! !

!SptHTTPResponse methodsFor: 'exceptions' stamp: ''!
httpException
	"Answer an exception instance that describes the error"

	"See SptHTTPStatus>>httpException"

	^self httpStatus httpException! !

!SptHTTPResponse methodsFor: 'operations' stamp: ''!
readResponse: aConnection
	| startT |
	#stayDown.
	"startT := Delay millisecondClockValue."
	
	[self readMessageStatus: aConnection.
	self readMessageHeaders: aConnection.
	self isInformational] 
			whileTrue.
	self readMessageBody: aConnection

	"Transcript
		display: (Delay millisecondClockValue - startT) displayString;
		cr."! !

!SptHTTPResponse methodsFor: 'accessing-headers' stamp: ''!
headerAt: aString ifAbsent: operation
	^headers at: aString ifAbsent: operation! !

!SptHTTPResponse methodsFor: 'accessing' stamp: ''!
request: aSptHTTPRequest
	request := aSptHTTPRequest! !

!SptHTTPResponse methodsFor: 'operations' stamp: ''!
readContentLengthBody: aConnection
	"Read Content-Length bytes from the socket (ie HTTP1.1 semantics).
	Note: read readEntityChunkSize bytes at a time so that we can update any progress/status objects."

	| total read bytes |
	self makingProgress onReadingBody.
	total := self contentLength.
	read := 0.
	[read >= total] whileFalse: 
			[bodyStream nextPutAll: (aConnection socketStream 
						nextBytes: (bytes := self readEntityBufferSize min: total - read)).
			read := read + bytes.
			self makingProgress read: read of: total].
	self makingProgress onReadBody! !

!SptHTTPResponse methodsFor: 'accessing-headers' stamp: ''!
headers
	"Answer a table of response headers"

	^headers! !

!SptHTTPResponse methodsFor: 'accessing-status' stamp: ''!
statusCodeString
	"Answer the status string of the response"

	^self httpStatus statusCodeString! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isClientOrServerError
	^self httpStatus isClientOrServerError! !

!SptHTTPResponse methodsFor: 'accessing-status' stamp: ''!
statusLine
	"Answer the response status line"

	^statusLine! !

!SptHTTPResponse methodsFor: 'accessing-headers' stamp: ''!
location
	^headers at: 'Location'! !

!SptHTTPResponse class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isAcceptRangesBytes
	^(self headerAt: 'Accept-Ranges' ifAbsent: [^false]) trimBlanks = 'bytes'! !

!SptHTTPResponse methodsFor: 'accessing-status' stamp: ''!
statusText
	"Answer the Status text of the response"

	^self httpStatus statusText! !

!SptHTTPResponse methodsFor: 'accessing-headers' stamp: ''!
rangeInterval
	| header stream rangeStart rangeEnd cl |
	cl := self contentLength.
	header := self headerAt: 'Range'.
	"We can only handle a single range"
	(header includes: $,) 
		ifTrue: [self error: 'Unexpected range header: ' , header displayString].
	stream := header readStream.
	(stream upTo: $=) trimBlanks = 'bytes' 
		ifFalse: [self error: 'Unexpected range header: ' , header displayString].
	rangeStart := (stream upTo: $-) trimBlanks.
	rangeEnd := stream upToEnd trimBlanks.
	"The final 500 bytes (byte offsets 9500-9999, inclusive): bytes=-500"
	rangeStart isEmpty ifTrue: [^cl - rangeEnd asNumber to: cl - 1].
	"The final 500 bytes ... Or bytes=9500- "
	rangeEnd isEmpty ifTrue: [^rangeStart asNumber to: cl - 1].
	^rangeStart asNumber to: rangeEnd asNumber! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isVersion11
	^self httpStatus isVersion11! !

!SptHTTPResponse methodsFor: 'operations' stamp: ''!
parseCookie: aString
	"Note: aString may contain multiple cookies separated by a $,"

	| allCookies |
	(request isNil or: [request isUseCookies not]) ifTrue: [^nil].
	"Note: an error parsing any of the cookies, means we reject all cookies"
	allCookies := [SptHTTPCookie allFromSetCookieString: aString url: request url] on: Error
				do: 
					[:e | 
					SptHTTPRequest logDebug: 'Cookie rejected: ' , e description.
					nil].
	allCookies notNil ifTrue: [allCookies do: [:cookie | request state addCookie: cookie]]! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isInformational
	^self httpStatus isInformational! !

!SptHTTPResponse methodsFor: 'accessing-headers' stamp: ''!
contentTypeOrNil
	^headers at: 'Content-Type' ifAbsent: []! !

!SptHTTPResponse methodsFor: 'accessing-headers' stamp: ''!
contentType
	^headers at: 'Content-Type' ifAbsent: ['*']! !

!SptHTTPResponse methodsFor: 'accessing-status' stamp: ''!
statusCode
	"Answer the status code of the response"

	^self httpStatus statusCode! !

!SptHTTPResponse methodsFor: 'testing' stamp: ''!
isImageContentType
	^(self headerAt: 'Content-Type' ifAbsent: [^false]) beginsWith: 'image/'! !

Object subclass: #SptHTTPConfiguration
	instanceVariableNames: 'httpConnectionClass httpsConnectionClass responseClass progressClass sendEntityBufferSize readEntityBufferSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPConfiguration commentStamp: '<historical>' prior: 0!
SptHTTPConfiguration

I am a configuration class that can be used to modify what classes an <SptHTTPRequest> uses.

Typical use would be to create your own subclass which answers your own classes, and then create requests like:

	
	request := SptHTTPRequest newFor: (MyHTTPConfiguration newDefault)



Instance Variables:
	httpConnectionClass	<Class>
	httpsConnectionClass	<Class>
	responseClass		<Class>
	progressClass		<Class>
	sendEntityBufferSize	<Integer> (1024 is good for frequent progress, 4096 is better for performance)
	readEntityBufferSize		<Integer>!

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
responseClass
	^responseClass! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
readEntityBufferSize
	^readEntityBufferSize! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
newResponseFor: aRequest
	^(self responseClass new)
		request: aRequest;
		yourself! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
httpsConnectionClass
	^httpsConnectionClass! !

!SptHTTPConfiguration class methodsFor: 'instance creation' stamp: ''!
newDefault
	^(self new)
		httpConnectionClass: SptHTTPConnection;
		httpsConnectionClass: SptHTTPSConnection;
		responseClass: SptHTTPResponse;
		progressClass: SptHTTPProgress;
		sendEntityBufferSize: 1024;
		readEntityBufferSize: 1024;
		yourself! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
sendEntityBufferSize: anInteger
	sendEntityBufferSize := anInteger! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
progressClass
	^progressClass! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
responseClass: anObject
	responseClass := anObject! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
progressClass: anObject
	progressClass := anObject! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
newProgressFor: aRequest
	^(self progressClass new)
		request: aRequest;
		yourself! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
httpConnectionClass: anObject
	httpConnectionClass := anObject! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
httpConnectionClass
	^httpConnectionClass! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
readEntityBufferSize: anObject
	readEntityBufferSize := anObject! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
sendEntityBufferSize
	^sendEntityBufferSize! !

!SptHTTPConfiguration methodsFor: 'accessing' stamp: ''!
httpsConnectionClass: anObject
	httpsConnectionClass := anObject! !

Object subclass: #SptHTTPProxySettings
	instanceVariableNames: 'useProxy proxyHost proxyPort '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPProxySettings commentStamp: '<historical>' prior: 0!
SptHTTPProxySettings

See SptHTTPRequest(class)>>exampleProxy

Instance Variables:
	useProxy		<Boolean>
	proxyHost		<String>
	proxyPort		<Integer>!

!SptHTTPProxySettings methodsFor: 'accessing' stamp: ''!
proxyPort
	proxyPort isNil ifTrue: [proxyPort := 8000].
	^proxyPort! !

!SptHTTPProxySettings class methodsFor: 'instance creation' stamp: ''!
useHost: aString port: aSmallInteger
	^self new useHost: aString port: aSmallInteger! !

!SptHTTPProxySettings methodsFor: 'accessing' stamp: ''!
useHost: aString port: aSmallInteger
	useProxy := true.
	proxyHost := aString.
	proxyPort := aSmallInteger! !

!SptHTTPProxySettings methodsFor: 'accessing' stamp: ''!
useProxy
	useProxy isNil ifTrue: [useProxy := false].
	^useProxy! !

!SptHTTPProxySettings methodsFor: 'accessing' stamp: ''!
useProxy: aBool
	useProxy := aBool! !

!SptHTTPProxySettings methodsFor: 'accessing' stamp: ''!
proxyPort: anInteger
	proxyPort := anInteger! !

!SptHTTPProxySettings methodsFor: 'accessing' stamp: ''!
proxyHost: aString
	proxyHost := aString! !

!SptHTTPProxySettings methodsFor: 'accessing' stamp: ''!
proxyHost
	proxyHost isNil ifTrue: [proxyHost := 'localhost'].
	^proxyHost! !

Object subclass: #SearchPolicy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Compatibility'!

!SearchPolicy class methodsFor: 'as yet unclassified' stamp: 'avi 9/29/2003 13:26'!
caseInsensitive! !

Object subclass: #SptHTTPUrl
	instanceVariableNames: 'urlString '
	classVariableNames: 'EncodeArray '
	poolDictionaries: ''
	category: 'SWHTTPClient-Support'!

!SptHTTPUrl commentStamp: '<historical>' prior: 0!
I am an absolute HTTP Url.

I can be constructed in the context of a 'previous' instance, and I will:
  - If not absolute, use the previous instance to make myself absolute

My class has a number of utility methods for testing/accessing url <String>s

Instance Variables:
	urlString		<String>!

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
localBasePath
	"
	(SptHTTPUrl fromString: 'http://www.foo.org/bar/index.html') localBasePath
		'/bar/'
	"

	| path |
	path := self path.
	^path last = $/ 
		ifTrue: [path]
		ifFalse: [path copyFrom: 1 to: (path findLast: [:each | each = $/])]! !

!SptHTTPUrl methodsFor: 'testing' stamp: ''!
isFileScheme
	^false! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
lastSegment
	"
	(SptHTTPUrl fromString: 'http://www.foo.org/index.html') lastSegment
		'index.html' 
	"

	^self class lastSegmentOfPath: self path! !

!SptHTTPUrl class methodsFor: 'utilities-testing' stamp: ''!
isAbsolutePath: aUrlString
	^aUrlString beginsWith: '/'! !

!SptHTTPUrl class methodsFor: 'instance creation' stamp: ''!
fromString: aUrlString
	"Answer an instance.
	aUrlString must be absolute (ie http://host:port/path)"

	^self fromString: aUrlString previous: nil! !

!SptHTTPUrl class methodsFor: 'canonicalizing' stamp: ''!
shouldEncode: aCharacter
	^(aCharacter isAlphaNumeric 
		or: [('-_.!!~*''()' includes: aCharacter) or: [';/?:@=&' includes: aCharacter]]) not! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
host
	^self hostPort readStream upTo: $:! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
authority
	^self authorityFrom: urlString readStream! !

!SptHTTPUrl methodsFor: 'testing' stamp: ''!
isSecure
	^self scheme = 'https'! !

!SptHTTPUrl class methodsFor: 'instance creation' stamp: ''!
fromString: aUrlString previous: aSptHTTPUrlOrNil
	"Answer a new instance.
	- If aUrlString is an absolutePath (ie '/foo/index.html') or local (ie 'index.html') 
		then make it absolute to aSptHTTPUrl"

	^self new urlString: (self makeAbsolute: aUrlString previous: aSptHTTPUrlOrNil)! !

!SptHTTPUrl class methodsFor: 'utilities-converting' stamp: ''!
makeRelAbsolute: aRelUrlString previous: aSptHTTPUrl
	"http://www.ietf.org/rfc/rfc2396.txt  5.2"

	"a) All but the last segment of the base URI's path component is copied to the buffer. "

	| urlString |
	urlString := aSptHTTPUrl localBasePath allButFirst.

	"b) The reference's path component is appended to the buffer string. "
	urlString := urlString , aRelUrlString.	"(SWSU split: aRelUrlString atLast: $/) second displayString."

	"c) All occurrences of './', where '.' is a complete path segment, are removed from the buffer string. "
	urlString := self removeDotSlashOf: urlString.

	"d) If the buffer string ends with '.' as a complete path segment, that '.' is removed. "
	(SWSU does: urlString endWith: '/.') ifTrue: [urlString := urlString allButLast].

	"e) All occurrences of '/../', where is a complete path segment not equal to '..', are removed from the buffer string. Removal of these path segments is performed iteratively, removing the leftmost matching pattern on each iteration, until no matching pattern remains. "
	urlString := self removeSegmentDotDotSlashOf: urlString.

	"f) If the buffer string ends with '/..', where is a complete path segment not equal to '..', that '/..' is removed. "
	(SWSU does: urlString endWith: '/..') ifTrue: [urlString := urlString allButLast: 3].

	"g) If the resulting buffer string still begins with one or more complete path segments of '..', then the reference is considered to be in error. Implementations may handle this error by retaining these components in the resolved path (i.e., treating them as part of the final URI), by removing them from the resolved path (i.e., discarding relative levels above the root), or by avoiding traversal of the reference. "
	(urlString startsWith: '../') ifTrue: [urlString := urlString allButFirst: 3].
	(urlString startsWith: './') ifTrue: [urlString := urlString allButFirst: 2].
	^aSptHTTPUrl rootBaseUrlString , '/' , urlString! !

!SptHTTPUrl methodsFor: 'accessing' stamp: 'avi 9/30/2003 20:18'!
displayString
	^urlString ifNil: ['']! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
localBaseUrlString
	"
	(SptHTTPUrl fromString: 'http://www.foo.org/bar/index.html') localBaseUrlString
		'http://www.foo.org/bar/'
	"

	^self rootBaseUrlString , self localBasePath! !

!SptHTTPUrl class methodsFor: 'utilities-accessing' stamp: ''!
lastSegmentOf: aUrlString
	"Answer the last segment <String> of aUrlString  
		self lastSegmentOf: 'http://www.foo.org/'  answers '/'.
		self lastSegmentOf: 'http://www.foo.org/foo'  answers 'foo'.
		self lastSegmentOf: 'http://www.foo.org/foo/'  answers 'foo'.
		self lastSegmentOf: 'http://www.foo.org/foo/bar'  answers 'bar'.

		self lastSegmentOf: 'http://www.foo.org/foo/bar?blab'  answers 'bar'."

	^self lastSegmentOfPath: ((self isAbsolute: aUrlString) 
				ifTrue: [(self fromString: aUrlString) path]
				ifFalse: [aUrlString])! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
urlString: aUrlString
	urlString := aUrlString! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
hostPort
	| authority authorityStream |
	authority := self authority.
	authorityStream := authority readStream.
	(authority includes: $@) ifTrue: [authorityStream upTo: $@].
	^authorityStream upToEnd! !

!SptHTTPUrl methodsFor: 'deprecated' stamp: ''!
url: aUrlString
	Notification deprecated.
	urlString := aUrlString! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
lastSegmentAndQuery
	"
	(SptHTTPUrl fromString: 'http://www.foo.org/index.html?foo=bar') lastSegmentAndQuery
		'index.html?foo=bar'
	"

	| query |
	^(query := self queryString) isEmpty 
		ifTrue: [self lastSegment]
		ifFalse: [self lastSegment , '?' , query]! !

!SptHTTPUrl class methodsFor: 'utilities-testing' stamp: ''!
is: aUrlString1 descendentOrSelfOf: aUrlString2
	"Assume both strings are the same type of url (ie absolute/relative), 
		answer if 1 is a descendent or the same as  2 in the url namespace"

	^aUrlString1 beginsWith: aUrlString2! !

!SptHTTPUrl class methodsFor: 'utilities-accessing' stamp: ''!
extensionOf: aUrlString
	"Answer the last segment <String> of aUrlString  
		self extensionOf: 'http://www.foo.org/'  answers nil.
		self extensionOf: 'http://www.foo.org/foo'  answers nil.
		self extensionOf: 'http://www.foo.org/foo.'  answers ''.
		self extensionOf: 'http://www.foo.org/foo.bar'  answers 'bar'."

	^(SWSU split: (self lastSegmentOf: aUrlString) atLast: $.) second! !

!SptHTTPUrl methodsFor: 'deprecated' stamp: ''!
localBaseUrl
	Notification deprecated.
	^self localBaseUrlString! !

!SptHTTPUrl methodsFor: 'testing' stamp: ''!
isHttpUrl
	^true! !

!SptHTTPUrl methodsFor: 'testing' stamp: ''!
isNullUrl
	^urlString isNil or: [urlString isEmpty]! !

!SptHTTPUrl methodsFor: 'displaying' stamp: ''!
displayOn: aStream
	aStream display: urlString! !

!SptHTTPUrl class methodsFor: 'utilities-testing' stamp: ''!
isCommonNonHTTPScheme: aUrlString
	| stream possibleScheme |
	stream := aUrlString readStream.
	possibleScheme := stream upTo: $:.
	^#('ftp' 'gopher' 'mailto' 'emailto' 'news' 'telnet' 'javascript') 
		includes: possibleScheme asLowercase! !

!SptHTTPUrl class methodsFor: 'utilities-accessing' stamp: ''!
specialDomainEndings
	"As defined by the Cookie specification"

	^#('com' 'edu' 'net' 'org' 'gov' 'mil' 'int')! !

!SptHTTPUrl class methodsFor: 'utilities-testing' stamp: ''!
doesHost: aString1 match: aString2
	"
		SptHTTPUrl doesHost: 'cache.foo.org' match: 'www.foo.org'
			true
	"

	| host1 host2 requiredMatches |
	host1 := (aString1 subStrings: $.) reverse.
	host2 := (aString2 subStrings: $.) reverse.
	host1 isEmpty ifTrue: [^false].
	requiredMatches := (self specialDomainEndings includes: host1 first asLowercase) 
				ifTrue: [2]
				ifFalse: [3].
	(host1 size < requiredMatches or: [host2 size < requiredMatches]) ifTrue: [^false].
	1 to: requiredMatches do: [:i | ((host1 at: i) sameAs: (host2 at: i)) ifFalse: [^false]].
	^true! !

!SptHTTPUrl methodsFor: 'comparing' stamp: ''!
= aSptHTTPUrl
	"Note: this is quick and dirty and will have false negatives"

	^(aSptHTTPUrl isKindOf: self class) and: [urlString = aSptHTTPUrl urlString]! !

!SptHTTPUrl class methodsFor: 'utilities-converting' stamp: ''!
removeDotSlashOf: aUrlString
	| urlString index |
	urlString := aUrlString.
	[(index := urlString indexOfSubCollection: '/./') = 0] whileFalse: 
			[urlString := (urlString copyFrom: 1 to: index) 
						, (urlString copyFrom: index + 3 to: urlString size)].
	^urlString! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
querySegments
	"
	(SptHTTPUrl fromString: 'http://www.foo.org/index.html?foo=bar') querySegments
		an OrderedCollection('foo' 'bar')
	"

	| segments |
	segments := OrderedCollection new.
	(self queryString subStrings: $&) do: 
			[:each | 
			| stream |
			stream := each trimBlanks readStream.
			segments add: (stream upTo: $=).
			stream atEnd ifFalse: [segments add: stream upToEnd]].
	^segments! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
asSegments
	| coll stream |
	coll := OrderedCollection new.
	coll add: self hostPort.
	stream := self path readStream.
	self assert: [stream next = $/].
	[stream atEnd] whileFalse: [coll add: (stream upTo: $/)].
	^coll asArray! !

!SptHTTPUrl class methodsFor: 'initializing' stamp: ''!
initialize
	"
	self initialize
	"

	EncodeArray := Array new: 255.
	1 to: 255 do: [:i | EncodeArray at: i put: (self shouldEncode: (Character codePoint: i))]! !

!SptHTTPUrl class methodsFor: 'utilities-accessing' stamp: ''!
schemeOf: aUrlString
	| stream scheme |
	stream := aUrlString readStream.
	scheme := stream upToAll: '://'.
	stream atEnd ifTrue: [^nil].
	^scheme! !

!SptHTTPUrl class methodsFor: 'canonicalizing' stamp: ''!
uncanonicalize: aString
	"Answer a <String>"

	^aString unescapePercents
	"| outStream inStream |
	outStream := String writeStream.
	inStream := aString readStream.
	[inStream atEnd] whileFalse: 
			[| char |
			outStream nextPut: ((char := inStream next) = $% 
						ifTrue: 
							[Character value: (Integer readFrom: (inStream next: 2) asUppercase readStream radix: 16)]
						ifFalse: [char])].
	^outStream contents"! !

!SptHTTPUrl class methodsFor: 'utilities-testing' stamp: ''!
isLocal: aUrlString
	"
	self isLocal:  'http://www.foo.org/index.html'. 
		false
	self isLocal:  '/index.html'. 
		false
	self isLocal:  'index.html'. 
		true

	"

	^(self isAbsolute: aUrlString) not and: [(self isAbsolutePath: aUrlString) not]! !

!SptHTTPUrl methodsFor: 'helpers' stamp: ''!
schemeFrom: aReadStream
	| scheme |
	scheme := aReadStream upTo: $:.
	self assert: [(aReadStream next: 2) = '//'].
	^scheme! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
queryString
	"
	(SptHTTPUrl fromString: 'http://www.foo.org/index.html?foo=bar') queryString
		'foo=bar'
	"

	| stream |
	stream := self absolutePath readStream.
	stream upTo: $?.
	^stream upToEnd! !

!SptHTTPUrl methodsFor: 'deprecated' stamp: ''!
rootBaseUrl
	Notification deprecated.
	^self rootBaseUrlString! !

!SptHTTPUrl methodsFor: 'testing' stamp: ''!
isInSpaceOf: aString
	^urlString beginsWith: (self class fromString: aString) localBaseUrlString! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
defaultPort
	self scheme = 'http' ifTrue: [^80].
	self scheme = 'https' ifTrue: [^443].
	self error: 'Scheme not knowen ' , self scheme displayString! !

!SptHTTPUrl class methodsFor: 'utilities-converting' stamp: ''!
makeAbsolute: aUrlString previous: aSptHTTPUrl
	| canUrlString |
	canUrlString := self recan: aUrlString.
	(self isAbsolute: canUrlString) 
		ifTrue: [^canUrlString]
		ifFalse: [aSptHTTPUrl isNil ifTrue: [self error: 'Can not make absolute']].
	^(self isAbsolutePath: canUrlString) 
		ifTrue: [aSptHTTPUrl rootBaseUrlString , canUrlString]
		ifFalse: [self makeRelAbsolute: canUrlString previous: aSptHTTPUrl]! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
rootBaseUrlString
	"
	(SptHTTPUrl fromString: 'http://www.foo.org/bar/index.html') rootBaseUrlString
		'http://www.foo.org'
	"

	| stream scheme |
	stream := urlString readStream.
	scheme := self schemeFrom: stream.
	^scheme , '://' , (stream upTo: $/)! !

!SptHTTPUrl class methodsFor: 'utilities-accessing' stamp: ''!
lastSegmentOfPath: aPathString
	| slashRemoved index |
	aPathString = '/' ifTrue: [^aPathString].
	index := (slashRemoved := (self isContainer: aPathString) 
						ifTrue: [aPathString allButLast]
						ifFalse: [aPathString]) findLast: [:c | c = $/].
	self assert: [index > 0].
	^slashRemoved copyFrom: index + 1 to: slashRemoved size! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
absolutePath
	| stream |
	stream := urlString readStream.
	self authorityFrom: stream.
	^'/' , (stream atEnd ifTrue: [''] ifFalse: [stream upToEnd])! !

!SptHTTPUrl class methodsFor: 'utilities-testing' stamp: ''!
is: aUrlString1 descendentOf: aUrlString2
	"Assume both strings are the same type of url (ie absolute/relative), 
		answer if 1 is a descendent of 2 in the url namespace"

	^(self is: aUrlString1 descendentOrSelfOf: aUrlString2) 
		and: [aUrlString1 size > aUrlString2 size]! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
queryStrings
	"
	(SptHTTPUrl fromString: 'http://www.foo.org/index.html?foo=bar') queryStrings
		#('foo' -> 'bar')
	"

	^(self queryString subStrings: $&) collect: 
			[:each | 
			| k v stream |
			stream := each trimBlanks readStream.
			k := stream upTo: $=.
			v := stream atEnd ifTrue: [] ifFalse: [stream upToEnd].
			k -> v]! !

!SptHTTPUrl class methodsFor: 'canonicalizing' stamp: ''!
recan: aString
	"Answer a <String>"

	^self canonicalize: (self uncanonicalize: aString)! !

!SptHTTPUrl class methodsFor: 'utilities-accessing' stamp: ''!
containerOf: aUrlString
	"Answer the container url <String> of aUrlString or nil if it is the root url 
		self containerOf: 'http://www.foo.org/'  answers nil.
		self containerOf: 'http://www.foo.org/foo'  answers 'http://www.foo.org/'.
		self containerOf: 'http://www.foo.org/foo/'  answers 'http://www.foo.org/'.
		self containerOf: 'http://www.foo.org/foo/bar'  answers 'http://www.foo.org/foo/'."

	| contRemoved lastSlash |
	((self isAbsolute: aUrlString) and: [(self fromString: aUrlString) absolutePath = '/']) 
		ifTrue: [^nil].
	aUrlString = '/' ifTrue: [^nil].
	contRemoved := (self isContainer: aUrlString) 
				ifTrue: [aUrlString allButLast]
				ifFalse: [aUrlString].
	lastSlash := contRemoved findLast: [:c | c = $/].
	^aUrlString copyFrom: 1 to: lastSlash! !

!SptHTTPUrl class methodsFor: 'utilities-converting' stamp: ''!
makeSchemeLowercase: aString
	| stream scheme |
	(scheme := self schemeOf: aString) ifNil: [^aString].
	stream := aString readStream.
	stream skipTo: $:.
	^scheme asLowercase , ':' , stream upToEnd! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
userInfo
	| authority authorityStream |
	((authority := self authority) includes: $@) ifFalse: [^nil].
	authorityStream := authority readStream.
	^authorityStream upTo: $@! !

!SptHTTPUrl class methodsFor: 'utilities-converting' stamp: 'avi 9/29/2003 14:37'!
makeUrlRefString: aString
	"This method attempts to make a urlString from aString"

	| text firstLine |
	text := aString trimBlanks.
	firstLine := text readStream nextLine.
	firstLine := self makeSchemeLowercase: firstLine.
	"If aString has an existing scheme, we cant make a http url"
	({'http'. 'https'. nil} includes: (self schemeOf: firstLine)) 
		ifFalse: [self error: 'Cant not create a url from a non-http scheme'].
	"Is it already an http url?"
	('http*' match: firstLine) ifTrue: [^firstLine].
	"Doesnt it start with 'www'?"
	('www.*' match: firstLine) ifTrue: [^'http://' , firstLine].
	"If all else fails ... use the same technique as IE"
	^'http://www.' , firstLine! !

!SptHTTPUrl class methodsFor: 'utilities-testing' stamp: ''!
isAbsolute: aUrlString
	"
	self isAbsolute:  'http://www.foo.org/index.html'. 
		true
	self isAbsolute:  '/index.html'. 
		false
	"
	|scheme|
	scheme _ self schemeOf: aUrlString.
	^ scheme
		ifNil: [false]
		ifNotNil: [#('http' 'https') includes: scheme asLowercase]! !

!SptHTTPUrl class methodsFor: 'utilities-converting' stamp: ''!
splitFragmentOf: aUrlReferenceString
	"http://www.ietf.org/rfc/rfc2396.txt
	A URI reference may be absolute or relative, and may have additional information attached in the form of a fragment identifier. However, 'the URI' that results from such a reference includes only the absolute URI after the fragment identifier (if any) is removed and after any relative URI is resolved to its absolute form."

	^SWSU split: aUrlReferenceString atLast: $#! !

!SptHTTPUrl class methodsFor: 'utilities-testing' stamp: ''!
isContainer: aUrlString
	^aUrlString last = $/! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
port
	| stream |
	stream := self hostPort readStream.
	stream upTo: $:.
	^stream atEnd ifTrue: [self defaultPort] ifFalse: [stream upToEnd asNumber]! !

!SptHTTPUrl class methodsFor: 'canonicalizing' stamp: ''!
canonicalize: aString
	"Answer a <String>"

	| stream encoding |
	stream := (String new: aString size) writeStream.
	aString do: 
			[:each | 
			(EncodeArray at: each asInteger) 
				ifTrue: 
					[encoding := each asInteger printStringRadix: 16 showRadix: false.
					stream nextPut: $%.
					encoding size = 1 ifTrue: [stream nextPut: $0].
					stream nextPutAll: encoding]
				ifFalse: [stream nextPut: each]].
	^stream contents! !

!SptHTTPUrl methodsFor: 'comparing' stamp: ''!
hash
	^urlString hash! !

!SptHTTPUrl class methodsFor: 'utilities-testing' stamp: ''!
is: aUrlString1 childOf: aUrlString2
	"Assume both strings are the same type of url (ie absolute/relative), 
		answer if 1 is a direct child of 2 in the url namespace"

	| relative separatorPosition |
	(self is: aUrlString1 descendentOf: aUrlString2) ifFalse: [^false].
	relative := aUrlString1 copyFrom: aUrlString2 size + 1 to: aUrlString1 size.
	separatorPosition := relative findFirst: [:each | each = $/].
	^separatorPosition = 0 or: [separatorPosition = relative size]! !

!SptHTTPUrl methodsFor: 'testing' stamp: ''!
hasUserInfo
	^self authority includes: $@! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
scheme
	"
	(SptHTTPUrl fromString: 'http://www.foo.org/bar/index.html') scheme
		'http'
	"

	^self schemeFrom: urlString readStream! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
path
	| absolutePath end |
	absolutePath := self absolutePath.
	end := absolutePath indexOfAnyOf: '?#' startingAt: 1.
	^end = 0 ifTrue: [absolutePath] ifFalse: [absolutePath copyFrom: 1 to: end - 1]! !

!SptHTTPUrl methodsFor: 'helpers' stamp: ''!
authorityFrom: stream
	self schemeFrom: stream.
	^stream upTo: $/! !

!SptHTTPUrl methodsFor: 'accessing' stamp: ''!
urlString
	^urlString! !

!SptHTTPUrl class methodsFor: 'utilities-converting' stamp: ''!
removeSegmentDotDotSlashOf: aUrlString
	| urlString index |
	urlString := aUrlString.
	[(index := urlString indexOfSubCollection: '/../') = 0] whileFalse: 
			[| segmentIndex |
			segmentIndex := urlString 
						prevIndexOf: $/
						from: index - 1
						to: 1.
			urlString := (urlString copyFrom: 1 to: segmentIndex) 
						, (urlString copyFrom: index + 4 to: urlString size)].
	^urlString! !

Object subclass: #Base64Encoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Compatibility'!

!Base64Encoder methodsFor: 'as yet unclassified' stamp: 'avi 9/30/2003 01:55'!
encode: inStream to: outStream
	outStream nextPutAll: (Base64MimeConverter mimeEncode: inStream) contents ! !

Object subclass: #SptHTTPCookieJar
	instanceVariableNames: 'cookies '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPCookieJar commentStamp: '<historical>' prior: 0!
SptHTTPCookieJar

I am used to store and access http cookies.

See SptHTTPRequest(class)>>exampleCookies

Instance Variables:
	cookies	<SharedIdentitySet>!

!SptHTTPCookieJar methodsFor: 'accessing' stamp: ''!
headerFor: aSptHTTPUrl
	| stream matching |
	matching := self cookiesFor: aSptHTTPUrl.
	stream := String writeStream.
	matching do: [:each | each printNameValueOn: stream] separatedBy: [stream nextPut: $;].
	^stream contents! !

!SptHTTPCookieJar methodsFor: 'accessing' stamp: ''!
numberOfCookies
	^cookies size! !

!SptHTTPCookieJar methodsFor: 'adding' stamp: ''!
addCookie: aSptHTTPCookie
	"SPEC: Instances of the same path and name will overwrite each other, with the latest instance taking precedence. Instances of the same path but different names will add additional mappings. "

	| removed |
	#todo.	"Parse the expires date so that;
		SPEC: If a CGI script wishes to delete a cookie, it can do so by returning a cookie 
			with the same name, and an expires time which is in the past"
	removed := self removeCookiesMatching: aSptHTTPCookie.
	self assert: [removed size <= 1].
	removed do: 
			[:eachCookie | 
			eachCookie nameValuesDo: 
					[:n :v | 
					"only add the NAME=VALUE pair if not present"

					aSptHTTPCookie atName: n ifAbsentPut: [v]]].
	cookies add: aSptHTTPCookie! !

!SptHTTPCookieJar methodsFor: 'accessing' stamp: ''!
cookies
	^cookies! !

!SptHTTPCookieJar methodsFor: 'initializing' stamp: ''!
initialize
	super initialize.
	cookies := SharedIdentitySet new! !

!SptHTTPCookieJar class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!SptHTTPCookieJar methodsFor: 'removing' stamp: ''!
removeCookiesMatching: aSptHTTPCookie
	| same |
	same := cookies select: [:each | each isSameAs: aSptHTTPCookie].
	same do: [:each | cookies remove: each ifAbsent: []].
	^same! !

!SptHTTPCookieJar methodsFor: 'operations' stamp: ''!
emptyJar
	cookies := SharedIdentitySet new! !

!SptHTTPCookieJar methodsFor: 'accessing' stamp: ''!
cookiesFor: aSptHTTPUrl
	| fromDomain host path |
	host := aSptHTTPUrl host.
	path := aSptHTTPUrl path.
	fromDomain := cookies select: [:each | each isDomainMatch: host].
	^fromDomain select: [:each | each isPathMatch: path]! !

!Object methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 13:55'!
initialize! !

Object subclass: #SWSU
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Support'!

!SWSU commentStamp: '<historical>' prior: 0!
Loose method pain avoidence class!

!SWSU class methodsFor: 'strings' stamp: ''!
truncateStart: aString to: anInteger
	"truncate to anInteger chars if needed ... anInteger could be 4, but really??"

	anInteger < 5 ifTrue: [self error: 'Invalid truncatation argument'].
	^aString size > anInteger 
		ifTrue: ['...' , (aString rightString: anInteger - 3)]
		ifFalse: [aString]! !

!SWSU class methodsFor: 'paths' stamp: ''!
rootPathOf: path
	^(File splitPath: path) first asUppercase copyWith: File pathDelimiter! !

!SWSU class methodsFor: 'strings' stamp: ''!
pad: aString with: aChar to: aNumber
	| newString |
	newString := (String new: aNumber) atAllPut: aChar.
	^aString 
		replaceBytesOf: newString
		from: 1
		to: (aString size min: aNumber - 2)
		startingAt: 1! !

!SWSU class methodsFor: 'strings' stamp: ''!
split: aString atLast: aCharacter
	| charIndex |
	charIndex := aString findLast: [:c | c = aCharacter].
	^charIndex = 0 
		ifTrue: [Array with: aString with: nil]
		ifFalse: 
			[Array with: (aString copyFrom: 1 to: charIndex - 1)
				with: (aString copyFrom: charIndex + 1 to: aString size)]! !

!SWSU class methodsFor: 'strings' stamp: ''!
copyReplacingLineDelimiters: aString
	^aString 
		collect: [:each | each isLinefeed | (each = Character cr) ifTrue: [Character space] ifFalse: [each]]! !

!SWSU class methodsFor: 'files' stamp: ''!
numFilesIn: aPath
	| count |
	count := 0.
	File 
		for: '*.*'
		in: aPath
		do: [:each | each isDirectory ifFalse: [count := count + 1]].
	^count! !

!SWSU class methodsFor: 'html' stamp: ''!
extractHTMLFormatFragment: anHTMLFormatString
	| stream |
	stream := anHTMLFormatString readStream.
	(stream skipToAll: '<!!--StartFragment-->') 
		ifFalse: [self error: 'Could not find the StartFragment comment'].
	^stream upToAll: '<!!--EndFragment-->'! !

!SWSU class methodsFor: 'strings' stamp: ''!
padLeft: aString with: aChar to: aNumber
	aString size >= aNumber ifTrue: [^aString].
	^(String new: aNumber - aString size withAll: aChar) , aString! !

!SWSU class methodsFor: 'strings' stamp: ''!
byteSizeStringFor: bytes
	bytes < 1024 ifTrue: [^bytes displayString , ' bytes'].
	bytes <= 1000000 ifTrue: [^(bytes / 1024 roundTo: 0.1) displayString , 'KB'].
	bytes <= (1024 * 1024 * 1023) 
		ifTrue: [^(bytes / 1024 / 1024 roundTo: 1.0e-002) displayString , 'MB'].
	^(bytes / 1024 / 1024 / 1024 roundTo: 1.0e-002) displayString , 'GB'! !

!SWSU class methodsFor: 'strings' stamp: ''!
copy: aString expanding: aTable
	"Answer a copy of the receiver with the chartacters in aTable expanded to their value strings.
	-aTable must be a table keying characters to strings"

	| keys stream |
	keys := aTable keys asArray.
	stream := WriteStream on: (String new: aString size).
	aString do: 
			[:each | 
			(keys includes: each) 
				ifTrue: [stream nextPutAll: (aTable at: each)]
				ifFalse: [stream nextPut: each]].
	^stream contents! !

!SWSU class methodsFor: 'html' stamp: ''!
extractHTMLFormatSourceURL: anHTMLFormatString
	| stream |
	stream := anHTMLFormatString readStream.
	(stream skipToAll: 'SourceURL:') 
		ifFalse: [self error: 'Could not find the SourceURL header'].
	^(stream swUpToWhile: [:c | (c = Character lf | (c = Character cr)) not]) trimBlanks! !

!SWSU class methodsFor: 'paths' stamp: ''!
makePathUnique: aPathAndFilename
	"Note: Windows functions for this only seem to be win2k+"

	| split newFilename fnSplit num |
	(File exists: aPathAndFilename) ifFalse: [^aPathAndFilename].
	split := File splitPath: aPathAndFilename.
	('*(*)' match: split third) 
		ifTrue: 
			[fnSplit := self split: split third atLast: $(.
			num := [Integer fromString: (fnSplit second copyFrom: 1 to: fnSplit second size - 1)] 
						on: Error
						do: [:e | 0].
			num := num + 1.
			newFilename := fnSplit first , '(' , num displayString , ')']
		ifFalse: [newFilename := split third , ' (1)'].
	^self makePathUnique: (File 
				composePath: split first , split second
				stem: newFilename
				extension: split fourth)! !

!SWSU class methodsFor: 'strings' stamp: ''!
truncate: aString to: anInteger
	"truncate to anInteger chars if needed.
	Note it puts a space between the truncation and elipses"

	^aString size > anInteger 
		ifTrue: [(aString leftString: anInteger - 4) , ' ...']
		ifFalse: [aString]! !

!SWSU class methodsFor: 'strings' stamp: ''!
copyCombiningSeparators: aString
	| inStream outStream isLastSeparator |
	inStream := aString readStream.
	outStream := (String new: aString size) writeStream.
	isLastSeparator := false.
	[inStream atEnd] whileFalse: 
			[| next |
			next := inStream next.
			next isSeparator 
				ifTrue: 
					[isLastSeparator 
						ifFalse: 
							[outStream space.
							isLastSeparator := true]]
				ifFalse: 
					[isLastSeparator := false.
					outStream nextPut: next]].
	^outStream contents! !

!SWSU class methodsFor: 'paths' stamp: ''!
addUrlSlash: aUrlString
	(aUrlString isNil or: [aUrlString isEmpty]) ifTrue: [^aUrlString].
	^aUrlString last = $/ ifTrue: [aUrlString] ifFalse: [aUrlString , '/']! !

!SWSU class methodsFor: 'strings' stamp: ''!
does: aString1 endWith: aString2
	aString2 size > aString1 size ifTrue: [^false].
	^(aString1 last: aString2 size) = aString2! !

!SWSU class methodsFor: 'paths' stamp: ''!
addPathSlash: aPath
	(aPath isNil or: [aPath isEmpty]) ifTrue: [^aPath].
	^aPath last = $\ ifTrue: [aPath] ifFalse: [aPath , '\']! !

Object subclass: #SZReadWriteSocketStream
	instanceVariableNames: 'readStream writeStream '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Streams'!

!SZReadWriteSocketStream commentStamp: '<historical>' prior: 0!
I am  a ReadWrite SocketStream. My stream contentSpecies is <String>, but I also provide methods to access the streams using ByteArrays/Integers.

Instance Variables:
	readStream	<SocketReadStream> 
	writeStream	<SocketWriteStream>

==========
Note:
-The readStream and writeStream can be streams (on ByteArrays) for testing purposes!

!SZReadWriteSocketStream methodsFor: 'accessing-string' stamp: 'avi 9/30/2003 23:04'!
nextPutAll: aByteObject
	writeStream nextPutAll: aByteObject.
	^aByteObject! !

!SZReadWriteSocketStream methodsFor: 'accessing' stamp: ''!
readStream: aReadStream writeStream: aWriteStream
	readStream := aReadStream.
	writeStream := aWriteStream.! !

!SZReadWriteSocketStream methodsFor: 'operations' stamp: ''!
flush
	writeStream flush! !

!SZReadWriteSocketStream methodsFor: 'operations' stamp: ''!
resetLogStream! !

!SZReadWriteSocketStream methodsFor: 'testing' stamp: ''!
hasInput 
	^readStream hasInput! !

!SZReadWriteSocketStream methodsFor: 'accessing-bytes' stamp: ''!
nextByte
	^(self nextBytes: 1) first! !

!SZReadWriteSocketStream methodsFor: 'accessing' stamp: ''!
writeStream
	^writeStream! !

!SZReadWriteSocketStream methodsFor: 'accessing-string' stamp: ''!
peek
	^self peekByte asCharacter! !

!SZReadWriteSocketStream methodsFor: 'accessing-string' stamp: 'avi 9/30/2003 23:05'!
next

	^ readStream next! !

!SZReadWriteSocketStream methodsFor: 'operations' stamp: ''!
close
	writeStream notNil ifTrue: [writeStream close].
	readStream notNil ifTrue: [readStream close].
	writeStream := readStream := nil! !

!SZReadWriteSocketStream methodsFor: 'accessing-bytes' stamp: ''!
peekByte
	^readStream peek! !

!SZReadWriteSocketStream class methodsFor: 'instance creation' stamp: 'avi 9/30/2003 23:27'!
testStreamReading: aByteObject
	"SW Test helper method.
	-aByteArray is set (and reset) into the readStream."

	^self readStream: aByteObject withInternetLineEndings readStream writeStream: String writeStream! !

!SZReadWriteSocketStream class methodsFor: 'instance creation' stamp: 'avi 9/30/2003 23:01'!
socket: aSocket
	| stream |
	stream _ SocketStream on: aSocket.
	^self readStream: stream writeStream: stream! !

!SZReadWriteSocketStream methodsFor: 'accessing-bytes' stamp: 'avi 9/30/2003 23:05'!
nextPutByte: anInteger
	^self nextPut: (Character value: anInteger)! !

!SZReadWriteSocketStream methodsFor: 'accessing-string' stamp: ''!
next: anInteger
	^(self nextBytes: anInteger) asString! !

!SZReadWriteSocketStream methodsFor: 'accessing' stamp: ''!
socket
	^readStream socket! !

!SZReadWriteSocketStream methodsFor: 'accessing' stamp: ''!
next: size putAll: aSequenceableCollection startingAt: start
	^writeStream 
		next: size
		putAll: aSequenceableCollection
		startingAt: start! !

!SZReadWriteSocketStream methodsFor: 'accessing' stamp: ''!
contentsSpecies
	^String! !

!SZReadWriteSocketStream methodsFor: 'testing' stamp: ''!
atEnd
	^readStream atEnd! !

!SZReadWriteSocketStream methodsFor: 'accessing' stamp: ''!
log
	^''! !

!SZReadWriteSocketStream methodsFor: 'accessing' stamp: ''!
readStream
	^readStream! !

!SZReadWriteSocketStream methodsFor: 'as yet unclassified' stamp: 'avi 9/30/2003 20:13'!
upTo: anObject 
	| newStream element |
	newStream _ String new writeStream.
	[self atEnd or: [(element _ self next) = anObject]]
		whileFalse: [newStream nextPut: element].
	^newStream contents! !

!SZReadWriteSocketStream methodsFor: 'accessing-string' stamp: 'avi 9/30/2003 23:04'!
nextPut: aCharacter
	^writeStream nextPut: aCharacter! !

!SZReadWriteSocketStream methodsFor: 'accessing-bytes' stamp: 'avi 9/30/2003 23:06'!
nextBytes: anInteger
	^(readStream next: anInteger) asByteArray! !

!SZReadWriteSocketStream methodsFor: 'accessing' stamp: ''!
next: count into: aSequenceableCollection startingAt: startAt
	^readStream 
		next: count
		into: aSequenceableCollection
		startingAt: startAt! !

!SZReadWriteSocketStream methodsFor: 'as yet unclassified' stamp: 'avi 9/30/2003 01:20'!
crlf
	self nextPutBytes: #(13 10)! !

!SZReadWriteSocketStream class methodsFor: 'instance creation' stamp: ''!
port: anIntegerPort host: aStringHostName
	^self socket: ((Socket port: anIntegerPort host: aStringHostName)
				connect;
				yourself)! !

!SZReadWriteSocketStream class methodsFor: 'instance creation' stamp: ''!
readStream: aReadStream writeStream: aWriteStream
	^self basicNew readStream: aReadStream writeStream: aWriteStream! !

SZReadWriteSocketStream subclass: #SZLoggingSocketStream
	instanceVariableNames: 'logStream '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Streams'!

!SZLoggingSocketStream commentStamp: '<historical>' prior: 0!
I can be used in place of a SwazooStream. I logs everything read and written to me.!

!SZLoggingSocketStream methodsFor: 'accessing-bytes' stamp: ''!
nextPutBytes: aByteArray
	| ans |
	ans := super nextPutBytes: aByteArray.
	self doLog: [logStream nextPutAll: aByteArray asString].
	^ans! !

!SZLoggingSocketStream methodsFor: 'accessing' stamp: ''!
log
	^logStream contents! !

!SZLoggingSocketStream methodsFor: 'operations' stamp: ''!
resetLogStream
	logStream := WriteStream on: String new.! !

!SZLoggingSocketStream methodsFor: 'accessing-string' stamp: ''!
next
	| ans |
	ans := super next.
	self doLog: [logStream nextPut: ans].
	^ans! !

!SZLoggingSocketStream methodsFor: 'accessing-bytes' stamp: ''!
nextBytes: aNumber
	| ans |
	ans := super nextBytes: aNumber.
	self doLog: [logStream nextPutAll: ans asString].
	^ans! !

!SZLoggingSocketStream methodsFor: 'accessing' stamp: ''!
next: size putAll: aSequenceableCollection startingAt: start
	| ans |
	ans := super 
				next: size
				putAll: aSequenceableCollection
				startingAt: start.
	self doLog: [logStream nextPutAll: aSequenceableCollection asString].
	^ans! !

!SZLoggingSocketStream methodsFor: 'accessing' stamp: ''!
readStream: aReadStream writeStream: aWriteStream
	super readStream: aReadStream writeStream: aWriteStream.
	self resetLogStream.! !

!SZLoggingSocketStream methodsFor: 'accessing' stamp: ''!
next: count into: aSequenceableCollection startingAt: startAt
	| ans |
	ans := super 
				next: count
				into: aSequenceableCollection
				startingAt: startAt.
	self doLog: [logStream nextPutAll: ans asString].
	^ans! !

!SZLoggingSocketStream methodsFor: 'operations' stamp: ''!
doLog: operation
	logStream notNil ifTrue: [operation value]! !

!SZLoggingSocketStream methodsFor: 'accessing-bytes' stamp: ''!
nextPutByte: anInteger
	| ans |
	ans := super nextPutByte: anInteger.
	self doLog: [logStream nextPut: (Character value: anInteger)].
	^ans! !

!SZReadWriteSocketStream methodsFor: 'accessing-bytes' stamp: 'avi 9/30/2003 23:05'!
nextPutBytes: aByteArray
	^aByteArray do: [:ea | self nextPutByte: ea]! !

!SZReadWriteSocketStream methodsFor: 'accessing-bytes' stamp: ''!
doBytes: operation
	[operation value: self nextByte] repeat! !

!SZReadWriteSocketStream methodsFor: 'as yet unclassified' stamp: 'avi 9/30/2003 20:13'!
space
	self nextPut: Character space! !

!SZReadWriteSocketStream methodsFor: 'accessing-string' stamp: 'avi 9/30/2003 20:08'!
upToCRLF
	| line |
	line := self upTo: Character cr.
	self next = Character lf ifFalse: [self error: 'No line'].
	^line! !

!SZReadWriteSocketStream methodsFor: 'accessing' stamp: ''!
writeStream: aWriteStream
	writeStream := aWriteStream! !

Object subclass: #SptHTTPRequest
	instanceVariableNames: 'connection url method headers responses entity deferredSend okBodyStream state flags progress conf timeoutLoop '
	classVariableNames: 'IsVersion11Mask IsRangeRequestMask IsUseCookiesMask IsFollowRedirectsMask IsAuthenticateMask '
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPRequest commentStamp: '<historical>' prior: 0!
I am a single request and response http message send in a background process.

I am a single request and response http message send in a background process.

==========
Features:
	Each feature of the HTTPClient is demonstrated in one of the class #example* methods

==========
Instance Variables:
	connection 	<SptHTTPConnection>
	url		<SptHTTPUrl> 
	method		<String>. (HTTP method)
	headers		<Dictionary> (HTTP request headers)
	entity 		<SptHTTPEntity> or nil (content to be sent with PUT/POST etc)
	responses 	<Collection> of <SptHTTPResponse>s
	okBodyStream 	<WriteStream> or nil 
	state 		<SptSessionState>
	flags		<Integer>
	progess		<SptHTTPProgress>
	conf		<SptHTTPConfiguration>

==========
Debugging

	See #send

==========
Flags
	IsAuthenticateMask 
		(true) controls whether an Authorization header can be added 
	IsFollowRedirectsMask 
		(true) controls whether 300 responses are automatically resent to the new Location 
	IsRangeRequestMask
		(false) controls whether the okBodyStream is set to the start position of a Range interval 
	IsUseCookiesMask
		(true) controls whether Cookies are added to a Request
	IsVersion11Mask
		(true) controls whether the request is sent as 1.1 and whether connections are resused

==========
How it works

	1) Set-Up
	 - A new instance is created for each request that you want to send to a server. You set
	the url/method/entity/flags and any additonal headers into the request. You can also set existing
	state <SptHTTPSessionState> into the request (which holds information usually needed
	across a series of requests)

	2) Sending
	 - The #send method starts the request in a background process and returns immediately.
	- In this background process the request is sent, and the response is read. Certain responses
	may create the need to send a new request (for example redirection), and therefore a single
	request may have multiple responses.

	3) Waiting
	 - You can use the #isReady message to check if the request has finished, or ...
	 - You can block using one of the #waitOnReady* methods. The #waitOnReady* methods
	can make use of a timeoutLoop to limit the time that the process blocks.
	
	4) Timeouts
	 - It was necessary to implement a timeoutLoop as WinSock1 does not have configurable
	OS layer timeouts.
	 - If a timeout is activated, a SocketWaitCancelled error will be signalled in the process
	blocked in the #waitOnReady* method
	
	5) Finishing
	- Access the #lastResponse, or use the methods in the accessing-response category.
	- If you are making a series of requests, you may want to store the #state


==========
Notes

	- The multiple-process design of this class (ie the deferredSend and the timeoutLoop)
	was driven by the lack of configurable WinSock1 OS-level timeouts. 
	- WinSock2 does have timeouts, and would allow much of the process synchronization code
	to be removed.
	 
	- I have used this class heavily over the past six months; I  have a queue class (not included) that
	processes 4-5 requests concurrently. 
	
	-What I have not used heavily (and may be a more likely source of bugs)
		- The authorization/authentication code
 		- Cookies
		- Range Requests are a new feature!

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
entity: aSptHTTPEntity
	entity := aSptHTTPEntity! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleState
	"
		A <SptHTTPSessionState> instance can be used to store information needed across multiple requests.
		including
			-Cookies
			-Timeouts
			-Authentication information
			-Proxy settings
	"

	| state request1 request2 |
	state := SptHTTPSessionState new.
	state timeout: nil.	"nil disables timeouts"
	state maxRedirects: 6.	"maximum number of redirects that will be automatically followed"
	state proxySettings: (SptHTTPProxySettings useHost: 'localhost' port: 8000).
	"Note: the following will cause an Authorization header only for urls in the http://www.foo.org/ namespace"
	state 
		username: 'steve'
		password: 'steve'
		for: 'http://www.foo.org/'.


	"This state can then be reused across requests"
	request1 := SptHTTPRequest new openGetTo: 'http://www.foo.org/1'.
	request1 state: state.
	"Do stuff"
	request2 := SptHTTPRequest new openGetTo: 'http://www.foo.org/2'.
	request2 state: state! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
putSortedHeadersOn: aStream
	self sortedHeadersAndValuesDo: 
			[:name :value | 
			self 
				putHeaderName: name
				value: value
				on: aStream]! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
sendEntityBufferSize
	^conf sendEntityBufferSize! !

!SptHTTPRequest methodsFor: 'process synchronisation' stamp: ''!
waitOnReady
	"Block the active process until the request has been processed. 
	-This will signal an error in the active process if one occurs in the background process.
	"

	#todo.	"Should timeout of 0 return immediately?"
	"Necessary if sent from the main process"
	Processor forkMainIfMain.
	^
	[(self timeout isNil or: [self timeout = 0]) 
		ifFalse: 
			["-Fork a second process that repeats until terminated. It sends the receiver the #checkTimeout message. 
				This method firstly checks whether the receiver is in a state that can timeout 
				(See #checkTimeout comment) , and if so closes the socket. (See #close comment)"

			self startTimeoutLoop].
	deferredSend value] 
			ensure: [self terminateTimeoutLoop]! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleAuthentication
	"
		The HTTPClient currently only supports basic authentication.

		If a request's state includes credentials for the request's url, an Authorization header
		will be automatically added to the request.
		 
	"

	| request |
	request := SptHTTPRequest new.
	request
		openGetTo: 'http://www.dolphinharbor.org/authtest/index.html';
		addHeader: 'Accept' value: '*/*'.
	"This is a helper method which associates the username/password with the request's url"
	request username: 'user1' password: 'pass1'.
	request
		send;
		waitOnReady.
	request lastResponse body.

	"Or ... Accessing the state directly"
	request := SptHTTPRequest new.
	request
		openGetTo: 'http://www.dolphinharbor.org/authtest/index.html';
		addHeader: 'Accept' value: '*/*'.
	request state 
		username: 'user1'
		password: 'pass1'
		for: 'http://www.dolphinharbor.org/'.
	request
		send;
		waitOnReady.
	request lastResponse body.

	"Incorrect, or missing credentials will cause a SptHTTPNotAuthorized to be signalled.
		-See #exampleAuthorizationDialog"
	request := SptHTTPRequest new.
	request
		openGetTo: 'http://www.dolphinharbor.org/authtest/index.html';
		addHeader: 'Accept' value: '*/*'.
	request username: 'user1' password: 'NotPass1'.
	request
		send;
		waitOnReady! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
state
	"Answer a <SptHTTPSessionState>"

	^state! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
sendMessageStart
	self putMessageStartOn: self socketStream! !

!SptHTTPRequest class methodsFor: 'helpers' stamp: 'avi 9/30/2003 01:11'!
fixTemps! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
setEntityOn: aString
	entity := SptHTTPEntity fromByteObject: aString! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
socketStream
	^connection socketStream! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
setProgress: aSptHTTPProgress
	progress := aSptHTTPProgress.
	progress request: self! !

!SptHTTPRequest methodsFor: 'testing' stamp: ''!
isUseCookies
	^flags anyMask: IsUseCookiesMask! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
headers
	"Answer a <PluggableLookupTable >"

	^headers! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
sendHeader: anAssoc
	(self socketStream)
		nextPutAll: anAssoc key;
		nextPutAll: ': ';
		nextPutAll: anAssoc value displayString;
		crlf! !

!SptHTTPRequest methodsFor: 'initializing' stamp: ''!
initializeFor: aConfiguration
	headers := PluggableLookupTable new searchPolicy: SearchPolicy caseInsensitive.
	responses := OrderedCollection new.
	conf := aConfiguration.
	url := SptHTTPUrl new.
	state := SptHTTPSessionState new.
	progress := aConfiguration newProgressFor: self.
	flags := IsVersion11Mask | IsFollowRedirectsMask | IsAuthenticateMask | IsUseCookiesMask! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
isFollowRedirects: aBoolean
	flags := flags mask: IsFollowRedirectsMask set: aBoolean! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
method
	"Private - Answer the <String> method of the request (ie 'GET', 'POST' etc)"

	^method! !

!SptHTTPRequest methodsFor: 'process synchronisation' stamp: ''!
waitTimingMilliseconds
	^(connection notNil and: [connection isTimeoutWaiting]) 
		ifTrue: [self timeout]
		ifFalse: [3000]! !

!SptHTTPRequest methodsFor: 'accessing-response' stamp: ''!
statusText
	"Answer the Status text of the response"

	^self lastResponse statusText! !

!SptHTTPRequest methodsFor: 'deprecated' stamp: ''!
open: aMethodString bstrUrl: aUrl varAsync: aBool bstrUser: s1 bstrPassword: s2
	"MSXML IXMLHTTPRequest compatibility method"

	#deprecated.
	self urlString: aUrl.
	method := aMethodString.
	((s1 isNil or: [s1 isEmpty]) and: [s2 isNil or: [s2 isEmpty]]) 
		ifFalse: [self username: s1 password: s2]! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
example
	"
		Using the HTTPClient - general principles
	"

	" 1) Create the instance"

	| request |
	request := SptHTTPRequest new.

	" 2) Set the instance with the url and HTTP method"
	request openGetTo: 'http://www.dolphinharbor.org/'.

	"3) Set optional information (like headers/cookies/authorization/timeouts)"
	request addHeader: 'Accept' value: '*/*'.

	"4) Send the request (in a background process)"
	request send.

	"5) Wait (blocking) on the request to finish (or poll it)"
	request waitOnReady.

	"6) Access the response"
	request responseBody.	"Inspect It"
	request responseHeaders.	"Inspect It "
	request status.	"Inspect It "
	request statusText	"Inspect It "! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
okBodyStream: aByteWriteStream
	"Optional, Set the <WriteStream> (on a ByteArray)  that a 200 response will stream to"

	okBodyStream := aByteWriteStream! !

!SptHTTPRequest methodsFor: 'deprecated' stamp: ''!
port
	Notification deprecated.
	self halt
	"^self useProxy ifTrue: [proxySettings proxyPort] ifFalse: [self url port]"! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
setUrl: aSptHTTPUrl
	"aSptHTTPUrl isDifferentAuthority ifTrue: [self removeAuthorizationHeader]."

	url := aSptHTTPUrl! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
openGetTo: aUrlString
	"Set the receiver to send a GET request to the url"

	self open: 'GET' to: aUrlString! !

!SptHTTPRequest methodsFor: 'modes' stamp: ''!
beVersion10
	"This effectively stops connections from being re-used"

	self isVersion11: false! !

!SptHTTPRequest methodsFor: 'deprecated' stamp: ''!
host
	Notification deprecated.
	self halt
	"^self useProxy ifTrue: [proxySettings proxyHost] ifFalse: [self url host]"! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
httpVersionString
	^self isVersion11 ifTrue: ['HTTP/1.1'] ifFalse: ['HTTP/1.0']! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleStreaming
	"
		The HTTPClient can be configured to either
			1/ Stream an entity to a server as part of request (typically a POST or PUT)
			2/ Stream an entity from a server (typically a GET)

		When streaming from a server, only a successful response (200) will use the stream
	"

	| request myStream |
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.dolphinharbor.org/';
		addHeader: 'Accept' value: '*/*'.

	"Create the stream and set it into the request"
	myStream := FileStream 
				write: (FileLocator default localFileSpecFor: 'spt_stream_test.html')
				text: false.
	request okBodyStream: myStream.
	
	[request
		send;
		waitOnReady] ensure: [myStream close].

	"Notice that the first response received (a 302 redirection) was not streamed to 'myStream'"
	request responses! !

!SptHTTPRequest methodsFor: 'test accessing' stamp: ''!
deferredSend: anObject
	"Unit Test helper method"

	deferredSend := anObject! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
requestURI
	^self useProxy ifTrue: [self urlString] ifFalse: [url absolutePath]! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
timeout
	^state timeout! !

!SptHTTPRequest class methodsFor: 'instance creation' stamp: ''!
newFor: aConfiguration
	^super new initializeFor: aConfiguration! !

!SptHTTPRequest class methodsFor: 'deprecated' stamp: ''!
sendEntityBufferSize
	Notification deprecated.
	^1024! !

!SptHTTPRequest methodsFor: 'operations-reading' stamp: ''!
readResponse
	"Create a new response object, add it to our responses collection, then ask it to read the response.
	Note: adding the response to responses before reading it, allows it to be accessed through me.
		-Progress classes can take advantage of this."

	| response |
	self responses add: (response := conf newResponseFor: self).
	response readResponse: connection! !

!SptHTTPRequest methodsFor: 'exceptions' stamp: ''!
errorRetryNonIdempotentMethod
	self error: 'Should not automatically retry non-Idempotent methods'! !

!SptHTTPRequest methodsFor: 'operations-resending' stamp: ''!
prepareForRetry
	"There was an error retry the request.
		-make sure we *can* send a Range request, and whether it makes sense"

	(self canRangeRetry and: [self lastResponse bodyStreamSize > 0]) 
		ifTrue: 
			["Setting the beRangeRequest  mode means that if a response is read with a 'Range' header,
				the SptHTTPResponse instance will set the position of the okBodyStream to
				the start of the range interval."

			self isRangeRequest: true.
			"eg The second 500 bytes (byte offsets 500-999, inclusive): bytes=500-999 "
			self addHeader: 'Range'
				value: 'bytes=' , self lastResponse bodyStreamSize displayString , '-' 
						, (self lastResponse contentLength - 1) displayString].
	"Reset the bodyStream regardless ... we cant count on a server returning a range "
	okBodyStream notNil ifTrue: [okBodyStream reset]! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
open: aMethodString to: aUrlString
	"Set the receiver to send a request using the HTTP method (typically GET or POST), to the url"

	self urlString: aUrlString.
	method := aMethodString! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleCookies
	"
		A <SptHTTPSessionState> instance can be used to store information needed across multiple requests.
		A <SptHTTPSessionState> includes a cookieJar which can be used to store cookies.

		- Cookies are parsed from responses and stored in the CookieJar.
		- Before a request is sent, it checks the CookieJar for any cookies
			that should be sent with the request.

		If you dont re-use a <SptHTTPSessionState> instance, cookies will not be sent"

	| request1 request2 |
	request1 := SptHTTPRequest new.
	request1 openGetTo: 'http://www.amazon.com/'.
	request1 send.
	"Will block until complete"
	request1 waitOnReady.

	"Inspect the following, it should contain a Set-Cookie header"
	request1 lastResponse headers.

	"Inspect the following, and look in the cookieJar, there should be one cookie
		Note: (multiple cookies can be set from a single Set-Cookie header)"
	request1 state.

	"state <SptHTTPSessionState> contains information that can be used over multiple requests."
	request2 := SptHTTPRequest new.
	"re-use the <SptHTTPSessionState> instance"
	request2 state: request1 state.
	request2 openGetTo: 'http://www.amazon.com/'.
	request2 send.
	"Will block until complete"
	request2 waitOnReady.

	"Inspect the following, it should contain the cookie header we sent to the server"
	request2 headers! !

!SptHTTPRequest class methodsFor: 'helpers' stamp: ''!
value: header1 value: header2
	"Private - The receiver acts as the default sort block for HTTP Headers"

	| rank1 rank2 |
	(rank1 := self headerRankOf: header1) < (rank2 := self headerRankOf: header2) 
		ifTrue: [^true].
	rank1 = rank2 ifTrue: [^header1 <= header2].
	^false! !

!SptHTTPRequest class methodsFor: 'constants' stamp: ''!
requestHeaders
	^#('accept' 'accept-charset' 'accept-encoding' 'accept-language' 'authorization' 'from' 'host' 'if-modified-since' 'if-match' 'if-none-match' 'if-range' 'if-unmodified-since' 'max-forwards' 'proxy-authorization' 'range' 'referer' 'user-agent')! !

!SptHTTPRequest methodsFor: 'operations-resending' stamp: ''!
prepareNextMessage: visited
	(self lastResponse isRedirect and: [self isFollowRedirects]) 
		ifTrue: 
			["I default to a low number of maxRedirects, but there is no standard maximum"

			visited size > self maxRedirects ifTrue: [self errorMaximumRedirections].
			self prepareForRedirect.
			"Allow the same url to be visted twice, but not three times.
			This allows sites that redirect back to the original url after setting a cookie."
			(visited select: [:each | each = self urlString]) size > 2 
				ifTrue: [self errorRedirectionLoop].
			^true].
	(self lastResponse isUnauthorized and: [self isAuthenticate]) 
		ifTrue: [^self prepareForAuthorize].
	^false! !

!SptHTTPRequest methodsFor: 'testing' stamp: ''!
useProxy
	^state useProxy! !

!SptHTTPRequest methodsFor: 'operations-resending' stamp: ''!
prepareForSpecRetry
	"Prepare for a simple retry based on 
		<http://www.w3.org/Protocols/rfc2616/rfc2616-sec8.html#sec8.2.4>

	Unlike #prepareForRetry, dont attempt a 'Range' request ... this is intended for
		situations where the server closes the connection before we see any request "

	"mmm, maybe we dont need to do anything in this method?"

	okBodyStream notNil ifTrue: [okBodyStream reset]! !

!SptHTTPRequest methodsFor: 'test accessing' stamp: ''!
connection: aSptHTTPConnection
	"Unit Test helper method"

	connection := aSptHTTPConnection! !

!SptHTTPRequest methodsFor: 'operations-resending' stamp: ''!
prepareForAuthorize
	| basicChallenge user |
	self assert: [self isAuthenticate].
	(basicChallenge := self lastResponse basicChallenge) isNil 
		ifTrue: [self errorAuthenticationFailed].
	"Info required for Digest authentication (but not yet implemented)"
	basicChallenge rootUri: self url rootBaseUrlString.
	basicChallenge digestUri: self requestURI.
	"Note: we must check if we have previously sent authorization. If so, only resend if we get new info"
	((headers includesKey: 'Authorization') 
		or: [(user := SptHTTPUser userFor: basicChallenge realm host: url host) isNil]) 
			ifTrue: 
				[((SptHTTPNotAuthorized new)
					challenge: basicChallenge;
					target: self) signal]
			ifFalse: [self credentials: (basicChallenge credentialsFor: user)].
	^true! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
urlString
	^url displayString! !

!SptHTTPRequest class methodsFor: 'constants' stamp: ''!
icon
	^Icon fromId: 179 in: ShellLibrary default! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleDebuging
	"
		The HTTPClient provides a number of tools to aid debuging.

		1) Wire Logging. Everything sent/received on all sockets can be logged to the Transcript.
			To enable this see the SptHTTPConnection class methed #beLoggingWireToTranscript.

		2) SptHTTPTranscriptProgress is a progress class that prints all progress events
			to the transcript. 

		For debuging the SptHTTPRequest itself see the method #send
	"

	| request |
	SptHTTPConnection beLoggingWireToTranscript.	"Note: this is a global setting that remains until turned off"
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.dolphinharbor.org/dh/index.html';
		addHeader: 'Accept' value: '*/*'.
	request send.
	request waitOnReady.
	SptHTTPConnection beNotLoggingWire.

	"
		Using SptHTTPTranscriptProgress 
	"
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.dolphinharbor.org/dh/index.html';
		addHeader: 'Accept' value: '*'.
	request setProgress: SptHTTPTranscriptProgress new.
	request send.
	request waitOnReady! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
doAuthorizing: operation
	^operation on: SptHTTPNotAuthorized
		do: 
			[:e | 
			self removeAuthorizationHeader.
			e target: state.
			e site: self urlString.
			^(SptHTTPAuthorizationDialog showOnException: e) 
				ifTrue: [self doAuthorizing: operation]
				ifFalse: [self error: 'Authorization canceled']]! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
removeCookieHeader
	headers removeKey: 'Cookie' ifAbsent: []! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
immediateHost
	"Answer the <String> host that the connection's socket should connect to."

	^state useProxy ifTrue: [self proxySettings proxyHost] ifFalse: [url host]! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
headerReportString
	| stream |
	stream := String writeStream.
	stream
		nextPutAll: '===Out';
		cr.
	self putMessageStartOn: stream.
	self putSortedHeadersOn: stream.
	stream cr.
	responses do: 
			[:each | 
			each putHeaderReportOn: stream.
			stream
				cr;
				cr].
	^stream contents! !

!SptHTTPRequest methodsFor: 'testing' stamp: ''!
isIdempotent
	^#('GET' 'HEAD' 'PUT' 'DELETE' 'OPTIONS' 'TRACE') includes: method! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
proxySettings
	^state proxySettings! !

!SptHTTPRequest methodsFor: 'accessing-response' stamp: ''!
responseHeaders
	"Answer a table of response headers"

	^self lastResponse headers! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
isRangeRequest: aBoolean
	"Set if this is a range request.
		-If so, on a successful response, the okBodyStream's position 
		will be set to the starting byte position"

	flags := flags mask: IsRangeRequestMask set: aBoolean! !

!SptHTTPRequest methodsFor: 'process synchronisation' stamp: ''!
terminateTimeoutLoop
	timeoutLoop isNil ifTrue: [^nil].
	timeoutLoop terminate.
	timeoutLoop := nil! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleTimeouts
	"
		The HTTPClient will process a request in a background process.

		The #waitOnReady message will block the active process until the background
			process completes. A timeout can be used to set a limit on how long the
			active process will block. (without any progress being made in the request)

		The #waitOnReadyCancelling message behaves the same as #waitOnReady except
			if the waiting process is terminated, the request will be cancelled.

		Timeouts are tied into the HTTPClients progress messages. Everytime a progress
			message is sent, the timeout is reset.

		A timeout value of 0 or nil will disable timeouts

		
	"

	#todo! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
addRequiredHeaders
	"Add any required headers, **only** if they have not been manually added"

	self addHeader: 'Host' value: self url hostPort.
	self addIfAbsentHeader: 'User-Agent' value: self class defaultUserAgent.
	(state shouldAuthenticate: self urlString) ifTrue: [self addAuthorizationHeader].
	self isUseCookies ifTrue: [self addCookieHeader].
	self hasEntity 
		ifTrue: [self addIfAbsentHeader: 'Content-Length' value: entity contentLength].
	(self hasEntity and: [self entity hasContentType]) 
		ifTrue: [self addIfAbsentHeader: 'Content-Type' value: entity contentType]! !

!SptHTTPRequest methodsFor: 'process synchronisation' stamp: ''!
isReady
	"Answer whether the response has been fully read."

	^deferredSend hasValue! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
progress
	"Answer a <SptHTTPProgress>"

	^progress! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
username: aUsernameString password: aPasswordString
	(url isNil or: [url isNullUrl]) 
		ifTrue: [self error: 'Error: no url for the authentication, please set the url first'].
	state 
		username: aUsernameString
		password: aPasswordString
		for: self urlString! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
addIfAbsentHeader: aName value: aValue
	headers at: aName ifAbsentPut: [aValue]! !

!SptHTTPRequest methodsFor: 'accessing-response' stamp: ''!
status
	"Answer the status code <Integer> of the last response"

	^self lastResponse status! !

!SptHTTPRequest methodsFor: 'testing' stamp: ''!
isAuthenticate
	^flags anyMask: IsAuthenticateMask! !

!SptHTTPRequest methodsFor: 'deprecated' stamp: ''!
bodyStream
	Notification deprecated.
	^okBodyStream! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
conf: aSptHTTPConfiguration
	conf := aSptHTTPConfiguration! !

!SptHTTPRequest methodsFor: 'testing' stamp: ''!
canRangeRetry
	"Answer whether we can retry the request, only asking for the range of bytes we dont have"

	^responses notEmpty and: [self lastResponse canRangeRetry]! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
addCookieHeader
	| header |
	header := state cookieHeaderFor: url.
	(header notNil and: [header notEmpty]) 
		ifTrue: [self addIfAbsentHeader: 'Cookie' value: header]! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleHeaders
	"
		The HTTPClient will automatically add required headers, but *only* if they have not
		been added by the user. This gives the user full control, but care must be taken for
		certain headers.

		Headers that are automatically added:
			Content-Length (if an entity is to be sent)
			Host 
			Content-Type (if an entity is to be sent, and it contains a Content-Type)
			User-Agent (defaults to the class constant #defaultUserAgent)
			Authorization (if credentials are supplied that match the url)
			Cookie (if a cookie-jar is supplied with cookies that match the url)
	"

	| request stream |
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.dolphinharbor.org/ns3/test';
		addHeader: 'Accept' value: '*/*'.
	"Note: the above url is for a page that echoes the headers we send"
	request send.
	request waitOnReady.	"Will block until response received"
	stream := request responseBody readStream.
	stream upToAll: 'HTTP_USER_AGENT'.
	stream next: 40.	"Should answer the default #defaultUserAgent"

	"
		This time manually add the User-Agent header		
	"
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.dolphinharbor.org/ns3/test';
		addHeader: 'Accept' value: '*/*';
		addHeader: 'User-Agent' value: 'A test Agent'.
	request send.
	request waitOnReady.	"Will block until response received"
	stream := request responseBody readStream.
	stream upToAll: 'HTTP_USER_AGENT'.
	stream next: 40! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: 'avi 9/30/2003 23:08'!
send
	"Send the request with no entity (typical for a GET Request).
	This method returns immediately.
	You can either:
		-test with #isReady (and when true access the #responseBody).
		-wait (block) on #waitOnReady or  #waitOnReadyCancelling (and then access #responseBody)"

	"Note: If you are debuging an exception that occurs in the deferred send, you can
	uncomment the 'self halt' to open a debugger on the problem"

	deferredSend := 
			["["self sendMessages"] on: Error
				do: 
					[:e | 
					
					e]"] 
					deferredValue! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
connect
	self progress onConnecting.
	connection := (url isSecure 
				ifTrue: [conf httpsConnectionClass]
				ifFalse: [conf httpConnectionClass]) 
					port: self immediatePort
					host: self immediateHost
					reuse: self isVersion11.
	self makingProgress onConnected! !

!SptHTTPRequest methodsFor: 'testing' stamp: ''!
isFollowRedirects
	^flags anyMask: IsFollowRedirectsMask! !

!SptHTTPRequest methodsFor: 'testing' stamp: ''!
isVersion11
	^flags anyMask: IsVersion11Mask! !

!SptHTTPRequest methodsFor: 'unclassified' stamp: ''!
userId: aStringOrNil
	Notification deprecated.
	self halt.
	url userId: aStringOrNil! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
entity
	"Answer a <SptHTTPEntity>"

	^entity! !

!SptHTTPRequest methodsFor: 'process synchronisation' stamp: ''!
waitOnReadySpecRetrying: aNumber
	"This will retry a request if the server closes the connection before we receive any status from the server
	It is a loose implementation of 
		<http://www.w3.org/Protocols/rfc2616/rfc2616-sec8.html#sec8.2.4>
		8.2.4 Client Behavior if Server Prematurely Closes Connection 

		-It doesnt do a time back-off.

	Note: this method is more robust than #waitOnReadyRetrying: which attempts to retry requests that
		have been partially received (and also attempts to use a 'Range' Request"

	| shouldRetry |
	shouldRetry := false.
	[self waitOnReady] on: SocketClosed
		do: 
			[:e | 
			"Can only retry if we dont receive any status from the server"

			(aNumber = 0 or: [self lastResponse hasStatus]) 
				ifTrue: [e pass]
				ifFalse: [shouldRetry := true]].
	shouldRetry ifFalse: [^aNumber].
	deferredSend hasValue ifFalse: [self errorInternalError].
	self prepareForSpecRetry.
	"The spec suggests using a 'binary exponential backoff' algorithm, but a constant looks fine to me.
	Note: This delay should occur after sending the headers, but before sending any entity"
	[Delay forSeconds: 5] wait.
	self send.
	^self waitOnReadyRetrying: aNumber - 1! !

!SptHTTPRequest class methodsFor: 'initializing' stamp: ''!
initialize
	"
	self initialize
	"

	IsVersion11Mask := 1.
	IsFollowRedirectsMask := 2.
	IsAuthenticateMask := 4.
	IsUseCookiesMask := 8.
	IsRangeRequestMask := 16! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
cookieJar: aSptHTTPCookieJar
	state cookieJar: aSptHTTPCookieJar! !

!SptHTTPRequest methodsFor: 'exceptions' stamp: ''!
errorMaximumRedirections
	self error: 'Maximum number of redirections reached'! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
entityString: aString
	entity := SptHTTPEntity fromString: aString! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
timeout: millisecondsOrNil
	"Set the timeout value. 
	Note: The actual time before timing out will be somewhere between milliseconds and milliseconds * 2.
	nil disables the receiver's timeouts and relies on win socket timeouts"

	state timeout: millisecondsOrNil! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
putMessageStartOn: aStream
	aStream
		nextPutAll: method;
		space;
		nextPutAll: self requestURI;
		space;
		nextPutAll: self httpVersionString;
		crlf! !

!SptHTTPRequest methodsFor: 'testing' stamp: ''!
isRangeRequest
	"Answer if this is a range request.
		-If so, on a successful response, the okBodyStream's position 
		will be set to the starting byte position"

	^flags anyMask: IsRangeRequestMask! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
maxRedirects: aNumber
	state maxRedirects: aNumber! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
conf
	"Answer a <SptHTTPConfiguration>"

	^conf! !

!SptHTTPRequest methodsFor: 'process synchronisation' stamp: ''!
waitOnReadyCancelling
	"Same as #waitOnReady except if the waiting process is terminated,
		cancel the connection (which will cause the deferredSends process to terminate)"

	[self waitOnReady] ifCurtailed: [self cancelConnection]! !

!SptHTTPRequest class methodsFor: 'helpers' stamp: ''!
headerRankOf: aString
	| aLowercaseString |
	aLowercaseString := aString asLowercase.
	(self generalHeaders includes: aLowercaseString) ifTrue: [^1].
	(self requestHeaders includes: aLowercaseString) ifTrue: [^2].
	^3! !

!SptHTTPRequest class methodsFor: 'constants' stamp: ''!
generalHeaders
	^#('cache-control' 'connection' 'date' 'pragma' 'transfer-encoding' 'upgrade' 'via')! !

!SptHTTPRequest methodsFor: 'exceptions' stamp: ''!
errorRedirectionLoop
	self error: 'Redirection loop detected'! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
addAuthorizationHeader
	self addIfAbsentHeader: 'Authorization' value: state authorizationHeader! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
removeAuthorizationHeader
	headers removeKey: 'Authorization' ifAbsent: []! !

!SptHTTPRequest methodsFor: 'exceptions' stamp: ''!
errorInternalError
	"Private - If this error is signalled, something has gone wrong with the 
	synchronisation  of the multiple processes the reciever uses"

	self error: 'HTTPClient internal synchronisation error'! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
sendMessage
	self connect.
	self sendMessageStart.
	self sendMessageHeaders.
	self hasEntity ifTrue: [self sendMessageBody].
	self socketStream flush! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
readEntityBufferSize
	^conf readEntityBufferSize! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleConfiguration
	"

		You can use a <SptHTTPConfiguration> instance to customize requests to:
			-Use your own connection classes (advanced)
			-Use your own progress class (more typical).
	"

	| conf request1 request2 |
	conf := SptHTTPConfiguration newDefault.
	conf progressClass: SptHTTPValueProgress.
	request1 := SptHTTPRequest newFor: conf.
	" ... use it etc"
	request2 := SptHTTPRequest newFor: conf
	" ... use it etc"! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
connection
	"Answer a <SptHTTPConnection>"

	^connection! !

!SptHTTPRequest class methodsFor: 'constants' stamp: ''!
defaultUserAgent
	"^'Test'"

	^'sw-dol/0.60'! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
state: aSptHTTPSessionState
	state := aSptHTTPSessionState! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
method: aString
	"Set the HTTP method of the request (ie GET POST etc)"

	method := aString! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
headerAt: aString
	"Answer the <String> value of the request header"

	^headers at: aString! !

!SptHTTPRequest methodsFor: 'process synchronisation' stamp: 'avi 9/30/2003 20:19'!
startTimeoutLoop
	"
	-Fork a second process that repeats until terminated. It sends the receiver the #checkTimeout message. 
	This method firstly checks whether the receiver is in a state that can timeout (See #checkTimeout comment) , 
		and if so closes the socket. (See #close comment)"

	timeoutLoop isNil ifFalse: [self errorInternalError].
	timeoutLoop := 
			[
			[(Delay forMilliseconds: self waitTimingMilliseconds) wait.
			connection notNil ifTrue: [connection checkTimeout ifTrue: [progress onTimeout]].
			true] 
					whileTrue] 
					fork.
	"timeoutLoop name: 'Timeout for: ' , self urlString"! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleAuthenticationDialog
	"
		A Dialog is included that can be used to ask the user for Auth information.
		 
	"

	| request |
	request := SptHTTPRequest new.
	request
		openGetTo: 'http://www.dolphinharbor.org/authtest/index.html';
		addHeader: 'Accept' value: '*/*'.

	"use: user1/pass1"
	request doAuthorizing: 
			[request
				send;
				waitOnReady].
	request lastResponse body! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
urlString: aString
	self setUrl: (SptHTTPUrl fromString: aString previous: url)! !

!SptHTTPRequest methodsFor: 'operations-reading' stamp: 'avi 9/29/2003 13:03'!
closeOrKeepAliveConnection
	"Use the block temporary so that another process (possibly trying to cancel the connection)
		doesnt interupt the close-down sequence"

	connection ifNotNil: 
			[|conn|
			conn _ connection.
			connection := nil.
			conn beTimeoutOff.
			self terminateTimeoutLoop.
			conn dumpWireLog.
			(self isVersion11 not or: 
					[responses isEmpty or: 
							[self lastResponse hasStatus not 
								or: [self lastResponse isConnectionClose or: [self isConnectionClose]]]]) 
				ifTrue: [conn closeSocket]
				ifFalse: [conn keepAliveIfOpen]]! !

!SptHTTPRequest methodsFor: 'process synchronisation' stamp: ''!
waitOnReadyRetrying: aNumber
	"This will retry a request if the server closes the connection.
	It is *not* an implementation of 
		<http://www.w3.org/Protocols/rfc2616/rfc2616-sec8.html#sec8.2.4>
		8.2.4 Client Behavior if Server Prematurely Closes Connection 

		-It doesnt do a time back-off
		-It will retry even if a partial response has been read.

	See #prepareForRetry comment"

	| shouldRetry |
	self isIdempotent ifFalse: [self errorRetryNonIdempotentMethod].
	shouldRetry := false.
	[self waitOnReady] on: SocketClosed
		do: [:e | aNumber = 0 ifTrue: [e pass] ifFalse: [shouldRetry := true]].
	shouldRetry ifFalse: [^aNumber].
	"self reportPreRetry: aNumber."
	deferredSend hasValue ifFalse: [self errorInternalError].
	"This will check to see if we can use a 'Range' request"
	self prepareForRetry.
	"self reportPostRetry."
	self send.
	^self waitOnReadyRetrying: aNumber - 1! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleProxy
	"
		The HTTPClient can be configured to use a proxy. 

		A Dialog is provided to get this information from the user

		The SptHTTPProxySettings instance is set into the request, but
			is stored in the request's 'state' <SptHTTPSessionState> instance variable.
			This state can be used across multiple requests. (see #exampleState)
	"

	| request proxySettings |
	proxySettings := SptProxySettingDialog showModal.
	SptHTTPConnection beLoggingWireToTranscript.	"Note: this is a global setting that remains until turned off"
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.dolphinharbor.org/dh/index.html';
		addHeader: 'Accept' value: '*/*';
		proxySettings: proxySettings.
	request send.
	request waitOnReady.

	"Clean up"
	SptHTTPConnection beNotLoggingWire! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
openPostTo: aUrlString
	"Set the receiver to send a POST request to the url"

	self open: 'POST' to: aUrlString! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
maxRedirects
	"Answer the maximum number of redirections that can automatically be followed"

	^state maxRedirects! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
proxySettings: aSptHTTPProxySettings
	state proxySettings: aSptHTTPProxySettings! !

!SptHTTPRequest methodsFor: 'test accessing' stamp: ''!
timeoutLoop: anObject
	"Unit Test helper method"

	timeoutLoop := anObject! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleProgressControl
	"
		A progress instance <SptHTTPProgress> can also  be used to control a request:		

		Please review  #exampleProgress before this example.

		1) It provides access to the request, and you can send the request the #cancelConnection
		message at any time (and from any process) to shutdown the request.

		2) (Advanced) The progress messages are sent within the background
		process. Therefore, with a customized progress class, you can alter
		how a request is processed

	"

	| progress request |
	progress := SptHTTPTranscriptProgress new.
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.chartexplorer.com/dh/projects/spray/docs/google/ssGoogleResults.jpg';
		addHeader: 'Accept' value: '*/*'.
	request setProgress: progress.
	request send.

	"Instead of blocking on the request, wait a couple of seconds and evaluate:"
	progress request cancelConnection.

	"A copy of the exception will be signalled in any waiting process.
		for example:"
	request waitOnReady! !

!SptHTTPRequest methodsFor: 'test accessing' stamp: ''!
timeoutLoop
	"Unit Test helper method"

	^timeoutLoop! !

!SptHTTPRequest methodsFor: 'accessing-response' stamp: ''!
responses
	"Answer a collection of <SptHTTPResponse>s"
	^responses! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
isVersion11: aBoolean
	flags := flags mask: IsVersion11Mask set: aBoolean! !

!SptHTTPRequest methodsFor: 'testing' stamp: ''!
hasEntity
	^entity notNil	"and: [entity contentLength ~= 0]"! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
progress: aSptHTTPProgress
	progress := aSptHTTPProgress! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleProcessSync
	"
		The HTTPClient sends the request and reads responses in a background process.

		There are various methods that you can use to block 
			another process until the request completes (or times out)
	
	"

	| request |
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.chartexplorer.com/dh/projects/spray/docs/google/ssGoogleResults.jpg';
		addHeader: 'Accept' value: '*/*'.

	"Send it"
	request send.	"returns immedaitely"

	"Then use either:"
	request isReady.	"returns immedaitely"

	"Or ... block"
	request waitOnReady.

	"Or ... block, if the blocked process is curtailed (ie terminated) ensure the request is closed"
	request waitOnReadyCancelling.

	"Or ... block, retrying the request if the server closes the connection
		before any response is received"
	request waitOnReadySpecRetrying: 2.

	"Or ... block, retrying the request if the server closes the connection
		If a partial response has been read, this method will attempt
		a partial range request"
	request waitOnReadyRetrying: 2! !

!SptHTTPRequest methodsFor: 'operations-resending' stamp: ''!
prepareForRedirect
	"Set the new url
		-remove any authentication or cookie headers.
		- add any new authentication or cookie headers for the new url."

	self urlString: self lastResponse location.
	self removeAuthorizationHeader.
	(state shouldAuthenticate: self urlString) ifTrue: [self addAuthorizationHeader].
	self removeCookieHeader.
	self isUseCookies ifTrue: [self addCookieHeader].
	self makingProgress onRedirecting! !

!SptHTTPRequest methodsFor: 'deprecated' stamp: ''!
close
	"With persistent connections, the meaning of this message can be ambiguous.
	
		If you want to cancel the background processing of the request, use the method #cancelConnection
		
	"

	Notification deprecated.
	self cancelConnection! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
cookieJar
	^state cookieJar! !

!SptHTTPRequest methodsFor: 'testing' stamp: ''!
isConnectionClose
	"Answer whether the user added a Connection: close header to the request.
	-The server should respond with the same header, but use the fact that the user added it to ensure the
	connection is not re-used"

	^(headers at: 'Connection' ifAbsent: [^false]) = 'close'! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
putHeaderName: nameString value: valueString on: aStream
	aStream
		nextPutAll: nameString;
		nextPutAll: ': ';
		nextPutAll: valueString displayString;
		crlf! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
sortedHeadersAndValuesDo: aTwoArgBlock
	| keys |
	keys := headers keys asSortedCollection: self class.	"collect: [:each | each asLowercase]"
	keys do: [:each | aTwoArgBlock value: each value: (headers at: each)]! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
examplePersistentConnections
	"
		The HTTPClient supports HTTP 1.1's persistent connections.

		To disable this for a request, evaluate:
			request isVersion11: false

		Connections are only reused within 5 seconds (see SptHTTPConnection(class)>>staleMilliseconds.
			This takes advantage of being able to send a series of requests, and minimizes the
			probability that the server will kill the connection 
	
	"

	"Enable wire logging in order to see the connection being reused"

	| request |
	SptHTTPConnection beLoggingWireToTranscript.
	2 timesRepeat: 
			[(request := SptHTTPRequest new)
				openGetTo: 'http://www.dolphinharbor.org/dh/index.html';
				addHeader: 'Accept' value: '*/*'.
			request send.
			request waitOnReady].

	"On the Transcript, you should see the same Socket (ie '  a Socket(xyz) '  being used for both requests.
	(Note: if this is not the case, ensure that the server did not include a Connection: close header)"
	SptHTTPConnection beNotLoggingWire! !

!SptHTTPRequest methodsFor: 'deprecated' stamp: ''!
url: aString
	"Must be of the form 'http://host/resource'"

	Notification deprecated.
	self urlString: aString! !

!SptHTTPRequest methodsFor: 'accessing-response' stamp: ''!
lastResponse
	"Answer the last <SptHTTPResponse> received (previous responses may be
	redirects/not authorized responses that are automatically resent"

	^responses isEmpty ifTrue: [nil] ifFalse: [responses last]! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
immediatePort
	"Answer the <Integer> port that the connection's socket should connect to."

	^state useProxy ifTrue: [self proxySettings proxyPort] ifFalse: [url port]! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleSSL
	#todo! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleProgress
	"
		The HTTPClient will send progress messages to a <SptHTTPProgress> instance.

		The default progress class has empty stubs for these messages. Create your own
			subclass and override these methods, for customized progress.

		*Important* The progress messages are sent from the background process.
	"

	| request progress presenter1 presenter2 transcriptProgress |
	progress := SptHTTPValueProgress new.
	presenter1 := TextPresenter showOn: progress stateValue.
	presenter2 := NumberPresenter showOn: progress percentageValue.
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.chartexplorer.com/dh/projects/spray/docs/google/ssGoogleResults.jpg';
		addHeader: 'Accept' value: '*/*';
		setProgress: progress.
	request
		send;
		waitOnReady.

	"
		For debuging purposes, the SptHTTPTranscriptProgress can be chained
		to a second progress.

		(Note: the following example needs to be evaluated with the presenters still open
			from the above example)

	"
	(transcriptProgress := SptHTTPTranscriptProgress new) nextProgress: progress.
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.chartexplorer.com/dh/projects/spray/docs/google/ssGoogleResults.jpg';
		addHeader: 'Accept' value: '*';
		setProgress: transcriptProgress.
	request
		send;
		waitOnReady.

	"clean up"
	presenter1 topShell exit.
	presenter2 topShell exit! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
sendMessages
	"sent in the deferred process(thread)"

	| visited |
	visited := OrderedCollection new.
	
	[visited add: self urlString.
	[self sendAndRead] ensure: [self closeOrKeepAliveConnection].
	self prepareNextMessage: visited] 
			whileTrue.
	self makingProgress onCompleted! !

!SptHTTPRequest methodsFor: 'accessing-request' stamp: ''!
addHeader: aHeaderString value: aHeaderValue
	"Add the HTTP header to the request. 
	Note: do *not* include the colon in aHeaderString, (it will be added)"

	headers at: aHeaderString put: aHeaderValue! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
sendAndRead
	"Private ... send the message and read the response.
	If this is curtailed, close the socket. This is not strictly necessary as. the socket
		would be closed anyway, however the 'contents' of the socket (consider 
		peristent connections) may now
		be in an inconsistent state. Closing the socket ensures an error is signalled
		if we try to read/write from it again."

	
	[self sendMessage.
	self readResponse] 
			ifCurtailed: [connection notNil ifTrue: [connection closeSocket]]! !

!SptHTTPRequest methodsFor: 'exceptions' stamp: ''!
errorAuthenticationFailed
	self 
		error: 'Authentication failed. No Basic Authentication challenge, Digest is not supported'! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
okBodyStream
	"Answer the <WriteStream> that any OK response (ie 2xx) will be streamed to"

	^okBodyStream! !

!SptHTTPRequest methodsFor: 'deprecated' stamp: ''!
setTimeouts: t1 connectTimeout: t2 sendTimeout: t3 receiveTimeout: t4
	"MSXML IXMLHTTPRequest compatibility method"

	"On the receive timeout is used. See class comments."

	#deprecated.
	self timeout: t4! !

!SptHTTPRequest methodsFor: 'accessing' stamp: ''!
url
	"Private - Answer the <SptHTTPUrl> url.
	If a redirect was followed, this will be the actual url of the resource."

	^url! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
isUseCookies: aBoolean
	flags := flags mask: IsUseCookiesMask set: aBoolean! !

!SptHTTPRequest methodsFor: 'accessing-response' stamp: ''!
responseBody
	"Answer a the response entity <String>"

	^self lastResponse body! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
credentials: aSptCredentials
	state credentials: aSptCredentials! !

!SptHTTPRequest methodsFor: 'accessing-settings' stamp: ''!
isAuthenticate: aBoolean
	flags := flags mask: IsAuthenticateMask set: aBoolean! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleCommonHeaders
	"
		The HTTPClient will automatically add some headers (if they have not already been added)
		including: Content-Length,Host, Content-Type, User-Agent , Authorization , Cookie 

		Some other headers that may be useful are:

		See <http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html> for more info
	"

	| request |
	(request := SptHTTPRequest new) openGetTo: 'http://www.dolphinharbor.org/ns3/test'.

	"Use the Accept header to limit what content-type(s) you will accept. 
		The value '*/*' indicates any response is acceptable."
	request addHeader: 'Accept' value: '*/*'.

	"Use the Accept-Encoding header to limit the content-codings you will accept. 
		To avoid receiving responses that are gzipped or compressed use the following"
	request addHeader: 'Accept-Encoding' value: ''.

	"To force caches to revalidate use both"
	request addHeader: 'Pragma' value: 'no-cache'.
	request addHeader: 'Cache-Control' value: 'no-cache'! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: 'avi 9/30/2003 23:13'!
sendMessageBody
	| sent total chunk |
	self makingProgress onSendingEntity.
	sent := 0.
	total := entity contentLength.
	[entity stream atEnd] whileFalse: 
			[self socketStream 
				nextPutAll: (chunk := entity stream nextAvailable: self sendEntityBufferSize) asString.
			self makingProgress sent: (sent := sent + chunk size) of: total].
	self makingProgress onSentEntity! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleHeadRequest
	"
		'HEAD' requests have no body
	"

	| request |
	request := SptHTTPRequest new.
	request
		open: 'HEAD' to: 'http://www.dolphinharbor.org/';
		addHeader: 'Accept' value: '*/*';
		isFollowRedirects: false.
	request
		send;
		waitOnReady.
	"Inspect It"
	request lastResponse! !

!SptHTTPRequest methodsFor: 'operations-closing' stamp: ''!
cancelConnection
	"This cancels the socket.
	-If  the DeferredValue send is blocked reading, an error will be signalled in that process (which will cause the socket to be closed)
	-An error will also be signalled in any client processes blocking on #waitOnReady"

	connection notNil ifTrue: [connection cancelSocket]! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleRedirection
	"
		The HTTPClient can automatically follow redirection responses.

		Following redirects can be fully disabled using the #isFollowRedirects setting.

		If this is enabled (the default):
			1/ It will check for redirection loops and signal an error if a url is visited more than twice.
			2/ The #maxRedirects: setting puts a limit on how many redirections will be followed.
				The default is SptHTTPSessionState(class)>>defaultMaxRedirects (5)
	"

	| request |
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.dolphinharbor.org/';
		addHeader: 'Accept' value: '*'.
	request send.
	request waitOnReady.	"Will block until response received"
	request status.	"Inspect It ... should be a 200"
	"Inspect the last response received"
	request lastResponse.
	"Inspect all responses received ... should have two response, the first being the redirection."
	request responses.

	"
		To disable redirection:
	"
	(request := SptHTTPRequest new)
		openGetTo: 'http://www.dolphinharbor.org/';
		addHeader: 'Accept' value: '*/*';
		isFollowRedirects: false.
	request send.
	request waitOnReady.	"Will block until response received"
	request status.	"Inspect It ... should be a 302"
	"Inspect the last response received"
	request lastResponse.
	"Inspect all responses received ... should ony have one response"
	request responses! !

!SptHTTPRequest class methodsFor: 'helpers' stamp: ''!
logDebug: aString
	Transcript
		nextPutAll: 'SptHTTPRequest warning: ' , aString;
		cr! !

!SptHTTPRequest class methodsFor: 'must strip' stamp: ''!
exampleBestPractice
	"
		Using the HTTPClient - best practice
	"

	| request |
	request := SptHTTPRequest new.
	"setup"
	request
		openGetTo: 'http://www.dolphinharbor.org/';
		addHeader: 'Accept' value: '*/*';
		addHeader: 'Accept-Encoding' value: ''.
	"sending"
	request
		send;
		waitOnReadySpecRetrying: 2.
	"accessing"
	^(request lastResponse hasStatus and: [request lastResponse isSuccessful]) 
		ifTrue: [request lastResponse body]
		ifFalse: ['']! !

!SptHTTPRequest methodsFor: 'deprecated' stamp: ''!
send: aString
	"Use #setEntityOn: "

	#deprecated.
	self setEntityOn: aString.
	self send! !

!SptHTTPRequest methodsFor: 'deprecated' stamp: ''!
setRequestHeader: aHeaderString bstrValue: aHeaderValue
	"MSXML IXMLHTTPRequest compatibility method"

	#deprecated.
	self addHeader: aHeaderString value: aHeaderValue! !

!SptHTTPRequest methodsFor: 'accessing' stamp: 'avi 9/29/2003 13:02'!
makingProgress
	"Answer the <SptHTTPProgress>.
	- Reset any timeouts"

	connection ifNotNil: [connection beTimeoutOn].
	^progress! !

!SptHTTPRequest methodsFor: 'operations-sending' stamp: ''!
sendMessageHeaders
	self makingProgress onSendingHeaders.
	self addRequiredHeaders.
	self putSortedHeadersOn: self socketStream.
	self socketStream crlf.
	self makingProgress onSentHeaders! !

!SptHTTPRequest class methodsFor: 'instance creation' stamp: ''!
new
	^self newFor: SptHTTPConfiguration newDefault! !

!Object methodsFor: '*swhttpclient' stamp: 'rw 8/24/2002 14:40'!
displayString

	^self printString! !

Object subclass: #SptHTTPSessionState
	instanceVariableNames: 'proxySettings credentials timeout cookieJar maxRedirects '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPSessionState commentStamp: '<historical>' prior: 0!
I am the state that (can) exist across a number of HTTP requests 

See SptHTTPRequest(class)>>exampleState

Instance Variables:
	proxySettings	<SptHTTPProxySettings>
	credentials	<SptCredentials>
	timeout		<Integer> or nil
	cookieJar		<SptHTTPCookieJar>
	maxRedirects	<Integer>!

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
authorizationHeader
	^credentials httpHeaderValue! !

!SptHTTPSessionState methodsFor: 'testing' stamp: ''!
useProxy
	proxySettings isNil ifTrue: [^false].
	^proxySettings useProxy! !

!SptHTTPSessionState methodsFor: 'initializing' stamp: 'avi 9/29/2003 13:55'!
initialize
	timeout := self class defaultTimeout.
	maxRedirects := self class defaultMaxRedirects.
	cookieJar := SptHTTPCookieJar new! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
proxySettings
	^proxySettings! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
addCookie: aSptHTTPCookie
	cookieJar addCookie: aSptHTTPCookie! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
maxRedirects: anInteger
	"Set the maximum number of redirects that will be automatically followed"

	maxRedirects := anInteger! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
proxySettings: aSptHTTPProxySettings
	proxySettings := aSptHTTPProxySettings! !

!SptHTTPSessionState class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!SptHTTPSessionState methodsFor: 'enquiries' stamp: ''!
username: aUsernameString password: aPasswordString for: aUrlString
	| user |
	user := SptHTTPUser new.
	user username: aUsernameString password: aPasswordString.
	credentials isNil ifTrue: [credentials := SptBasicCredentials new].
	credentials user: user.
	credentials challenge: SptHTTPChallenge newBasicChallenge.
	credentials challenge absoluteUrl: aUrlString! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
timeout
	^timeout! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
cookieJar
	^cookieJar! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
timeout: anInteger
	timeout := anInteger! !

!SptHTTPSessionState class methodsFor: 'constants' stamp: ''!
defaultTimeout
	^20000! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
cookieJar: aSptHTTPCookieJar
	cookieJar := aSptHTTPCookieJar! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
maxRedirects
	"Answer the maximum number of redirections that can automatically be followed"

	^maxRedirects! !

!SptHTTPSessionState methodsFor: 'testing' stamp: ''!
shouldAuthenticate: aUrlString
	^credentials notNil and: [credentials isInProtectionSpace: aUrlString]! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
credentials: aSptCredentials
	credentials := aSptCredentials! !

!SptHTTPSessionState methodsFor: 'enquiries' stamp: ''!
cookieHeaderFor: aSptHTTPUrl
	^cookieJar headerFor: aSptHTTPUrl! !

!SptHTTPSessionState methodsFor: 'accessing' stamp: ''!
credentials
	^credentials! !

!SptHTTPSessionState class methodsFor: 'constants' stamp: ''!
defaultMaxRedirects
	^5! !

Object subclass: #SptHTTPEntity
	instanceVariableNames: 'stream contentLength contentType '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPEntity commentStamp: '<historical>' prior: 0!
I am an HTTP Entity sent with an HTTP Request used with requests that send a body (ie POST, PUT)

Instance Variables:
	stream		<ReadStream>
	contentLength	<Integer>
	contentType	<String>!

!SptHTTPEntity methodsFor: 'content types' stamp: ''!
beTextPlain
	contentType := 'text/plain'! !

!SptHTTPEntity methodsFor: 'accessing' stamp: ''!
contentType: aString
	contentType := aString! !

!SptHTTPEntity methodsFor: 'testing' stamp: ''!
hasContentType
	^contentType notNil! !

!SptHTTPEntity class methodsFor: 'instance creation' stamp: ''!
onFileStream: aFileStream
	"The FileStream should be opened in the text: false mode"

	^(self new)
		stream: aFileStream contentLength: aFileStream size;
		yourself! !

!SptHTTPEntity methodsFor: 'accessing' stamp: ''!
contentType
	^contentType! !

!SptHTTPEntity methodsFor: 'content types' stamp: ''!
beTextHtml
	contentType := 'text/html'! !

!SptHTTPEntity class methodsFor: 'instance creation' stamp: ''!
fromByteObject: aByteObject
	^(self new)
		stream: aByteObject asByteArray readStream contentLength: aByteObject size;
		yourself! !

!SptHTTPEntity class methodsFor: 'instance creation' stamp: ''!
onStream: aByteReadStream contentLength: anInteger
	^(self new)
		stream: aByteReadStream contentLength: anInteger;
		yourself! !

!SptHTTPEntity methodsFor: 'content types' stamp: ''!
beTextXml
	contentType := 'text/xml'! !

!SptHTTPEntity methodsFor: 'accessing' stamp: ''!
stream: aReadStream contentLength: anInteger
	stream := aReadStream.
	contentLength := anInteger! !

!SptHTTPEntity class methodsFor: 'instance creation' stamp: ''!
fromString: aString
	^self fromByteObject: aString! !

!SptHTTPEntity methodsFor: 'accessing' stamp: ''!
contentLength
	^contentLength ifNil: [0]! !

!SptHTTPEntity methodsFor: 'content types' stamp: ''!
beAppOctetStream
	contentType := 'application/octet-stream'! !

!SptHTTPEntity methodsFor: 'accessing' stamp: ''!
stream
	^stream! !

Object subclass: #SptHTTPConnection
	instanceVariableNames: 'host port socket socketStream timeoutState keepAliveTime '
	classVariableNames: 'WireLogStream IsLoggingWire '
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

SptHTTPConnection class
	instanceVariableNames: 'keepAlives '!

!SptHTTPConnection commentStamp: '<historical>' prior: 0!
SptHTTPConnection

I am a connection to an HTTP server.

I can be reused, which is the default in HTTP 1.1. To avoid problems of the server closing a connection via a time-out, and having to resend the request, I will only be reused within 5 seconds of my last use. The keepAliveTime instanceVariable is used to store the millisecondClockValue of my last use.

The class mode; #beLoggingWire (#beNotLoggingWire) can be used to dump full logs of all characters received/sent over each connection to the Transcript. 

See #checkTimeout for info on timeoutState 


Instance Variables
	host		<String> (eg 'www.yahoo.com')
	port		<Integer> (eg 80)
	socket		<Socket>
	socketStream	<SZReadWriteSocketStream>
	timeoutState	<Symbol>
	keepAliveTime	<Integer>!

!SptHTTPConnection class methodsFor: 'instance creation' stamp: ''!
newOnPort: aSmallInteger host: aString
	^(self new)
		port: aSmallInteger host: aString;
		connect;
		yourself! !

!SptHTTPConnection methodsFor: 'operations' stamp: ''!
isTimeoutWaiting
	^timeoutState == #wait! !

!SptHTTPConnection class methodsFor: 'initializing' stamp: ''!
uninitialize
	"
	self uninitialize
	"

	keepAlives := nil.
	WireLogStream := nil! !

!SptHTTPConnection class methodsFor: 'instance creation' stamp: ''!
reuseOnPort: aSmallInteger host: aString
	"Note: Need to be careful that two requests (each in forked process) 
		dont think they can re-use the same connection"

	self removeStaleKeepAlives.
	^keepAlives removeKey: aString , ':' , aSmallInteger displayString
		ifAbsent: [self newOnPort: aSmallInteger host: aString]! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
host: aHostString
	host := aHostString! !

!SptHTTPConnection methodsFor: 'operations' stamp: ''!
cancelSocket
	"Cancel the socket.
	See SptHTTPRequest>>cancelConnection "

	socket notNil ifTrue: [socket cancel]! !

!SptHTTPConnection methodsFor: 'operations' stamp: ''!
dumpWireLog
	self class isLoggingWire 
		ifTrue: 
			[self logWire: socketStream log.
			socketStream resetLogStream]! !

!SptHTTPConnection class methodsFor: 'operations' stamp: ''!
removeStaleKeepAlives
	"
	self removeStaleKeepAlives.
	keepAlives
	"

	| removes |
	removes := OrderedCollection new.
	keepAlives keysAndValuesDo: [:key :value | value isStale ifTrue: [removes add: key]].
	removes 
		do: [:each | (keepAlives removeKey: each ifAbsent: [DeafObject current]) closeSilently]! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
socketStream: aSZReadWriteSocketStream
	socketStream := aSZReadWriteSocketStream! !

!SptHTTPConnection methodsFor: 'operations' stamp: ''!
closeSilently
	[self closeSocket] on: Error do: [:e | ]! !

!SptHTTPConnection methodsFor: 'operations' stamp: ''!
keepAliveIfOpen
	(socket notNil and: [socket isOpen]) ifTrue: [self class keepAlive: self]! !

!SptHTTPConnection methodsFor: 'testing' stamp: ''!
isStale
	keepAliveTime isNil ifTrue: [^true].
	^Delay millisecondClockValue - keepAliveTime > self class staleMilliseconds! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
logWire: aString
	| stream |
	stream := String writeStream.
	stream
		cr;
		nextPutAll: '=======';
		cr;
		print: socket;
		cr;
		nextPutAll: aString.
	
	[(self class wireLogStream ifNil: [Transcript])
		nextPutAll: stream contents;
		cr] 
			on: Error
			do: [:e | ]! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
setKeepAliveTime
	keepAliveTime := Delay millisecondClockValue! !

!SptHTTPConnection class methodsFor: 'modes' stamp: ''!
beNotLoggingWire
	"
	self beNotLoggingWire
	"

	IsLoggingWire := false.
	self removeKeepAlives! !

!SptHTTPConnection class methodsFor: 'modes' stamp: ''!
beLoggingWire
	"
	self beLoggingWire.
	self wireLogStream: Transcript.
	"

	IsLoggingWire := true.
	self removeKeepAlives! !

!SptHTTPConnection class methodsFor: 'accessing' stamp: ''!
wireLogStream
	^WireLogStream! !

!SptHTTPConnection class methodsFor: 'accessing' stamp: ''!
isLoggingWire: aBoolean
	IsLoggingWire := aBoolean! !

!SptHTTPConnection methodsFor: 'deprecated' stamp: ''!
close
	"Close the socket.
	-If  the DeferredValue send is blocked reading, an error will be signalled in that process (which will cause the socket to be closed)
	-An error will also be signalled in any client processes blocking on #waitOnReady"

	Notification deprecated.
	self cancelSocket! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
port
	^port! !

!SptHTTPConnection methodsFor: 'operations' stamp: ''!
closeSocket
	"Close the socket.
	
	This can safely be sent to a socket that is already closed"

	socket notNil ifTrue: [socket close]! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
hostPortString
	self assert: [port >= 0].
	^host , ':' , port displayString! !

!SptHTTPConnection class methodsFor: 'accessing' stamp: ''!
wireLogStream: aWriteStream
	WireLogStream := aWriteStream! !

!SptHTTPConnection class methodsFor: 'accessing' stamp: ''!
keepAlive: aConnection
	"Add the instance to our table of connection to (possibly) re-use"

	aConnection beTimeoutOff.
	aConnection setKeepAliveTime.
	keepAlives at: aConnection hostPortString put: aConnection! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
socketStream
	^socketStream! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
port: anObject
	port := anObject! !

!SptHTTPConnection class methodsFor: 'modes' stamp: ''!
beLoggingWireToTranscript
	"
	self beLoggingWireToTranscript.
	"

	self beLoggingWire.
	self wireLogStream: (self environment at: #Transcript)! !

!SptHTTPConnection methodsFor: 'deprecated' stamp: ''!
beTimeoutNotWaiting
	"See #checkTimeout"

	Notification deprecated.
	timeoutState == #wait ifTrue: [timeoutState := #on]! !

SptHTTPConnection subclass: #SptHTTPSConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPSConnection methodsFor: 'operations' stamp: ''!
connectSocket
	| address |
	address := InternetAddress fromString: self host.
	socket := (self class environment at: #OpenSSLSocket
				ifAbsent: [self error: 'The HTTPClient is not configured for HTTPS']) port: self port
					address: address.
	socket connect.
	^socket! !

!SptHTTPSConnection class methodsFor: 'initializing' stamp: ''!
uninitialize
	"
	self uninitialize
	"

	super uninitialize! !

!SptHTTPSConnection class methodsFor: 'initializing' stamp: ''!
initialize
	"
	self initialize
	"

	super initialize! !

!SptHTTPConnection class methodsFor: 'event handling' stamp: ''!
onImageSaveStarting
	"Avoid saving any instances into an image"

	| temp |
	temp := keepAlives.
	keepAlives := SharedLookupTable new.
	temp values do: [:each | each closeSilently]! !

!SptHTTPConnection methodsFor: 'operations' stamp: 'avi 9/30/2003 22:59'!
connectSocket
	socket _  Socket newTCP connectToHostNamed: self host port: self port! !

!SptHTTPConnection methodsFor: 'modes' stamp: ''!
beTimeoutOn
	"See #checkTimeout"

	timeoutState := #on! !

!SptHTTPConnection class methodsFor: 'operations' stamp: ''!
removeKeepAlives
	"
	self removeKeepAlives.
	keepAlives
	"

	| removes |
	removes := keepAlives values asOrderedCollection.
	removes 
		do: [:each | (keepAlives removeKey: each ifAbsent: [DeafObject current]) closeSilently]! !

!SptHTTPConnection class methodsFor: 'instance creation' stamp: ''!
port: aSmallInteger host: aString reuse: aBoolean
	"Answer an instance.
	Note: an exisiting instance will be answered if resuse is true, 
		and we have a connection that we are keeping alive for the host/port"

	^aBoolean 
		ifTrue: [self reuseOnPort: aSmallInteger host: aString]
		ifFalse: [self newOnPort: aSmallInteger host: aString]! !

!SptHTTPConnection methodsFor: 'test accessing' stamp: ''!
timeoutState
	"Unit Test helper method"

	^timeoutState! !

!SptHTTPConnection methodsFor: 'test accessing' stamp: ''!
socket: aSocket
	"Unit Test helper method"

	socket := aSocket! !

!SptHTTPConnection methodsFor: 'operations' stamp: ''!
checkTimeout
	"If the timeoutState is #wait, then we activate the timeout by cancelling the socket.
		(This will cause a timeout SocketError exception to be signaled in
		the process that is blocked trying to read (or write) from the socket)

	If the timeoutState is #on, we move it to #wait (so the next time we check, we may timeout).

	See SptHTTPRequest>>makingProgress ... which resets the timeoutState to #on each time
	we read or write to the socket"

	timeoutState == #off ifTrue: [^false].
	^timeoutState == #wait 
		ifTrue: 
			[timeoutState := #off.
			self cancelSocket.
			true]
		ifFalse: 
			[timeoutState := #wait.
			false]! !

!SptHTTPConnection class methodsFor: 'accessing' stamp: ''!
socketStreamClass
	^IsLoggingWire ifTrue: [SZLoggingSocketStream] ifFalse: [SZReadWriteSocketStream]! !

!SptHTTPConnection class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!SptHTTPConnection class methodsFor: 'event handling' stamp: 'avi 9/30/2003 22:54'!
shutDown: aBoolean
	aBoolean ifTrue: [self onImageSaveStarting]! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
socket
	^socket! !

!SptHTTPConnection methodsFor: 'initializing' stamp: ''!
initialize
	super initialize.
	timeoutState := #off! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
port: aSmallInteger host: aString
	port := aSmallInteger.
	host := aString! !

!SptHTTPConnection class methodsFor: 'testing' stamp: ''!
isLoggingWire
	^IsLoggingWire! !

!SptHTTPConnection methodsFor: 'modes' stamp: ''!
beTimeoutOff
	"See #checkTimeout"

	timeoutState := #off! !

!SptHTTPConnection class methodsFor: 'constants' stamp: ''!
staleMilliseconds
	"FWIW: I believe that Apache defaults to 20seconds, so this
	should be low enough to keep us under the radar screen while
	still being high enough to satisfy rapid fire multiple requests"

	^5000! !

!SptHTTPConnection methodsFor: 'operations' stamp: ''!
connect
	self connectSocket.
	socketStream := self class socketStreamClass socket: socket! !

!SptHTTPConnection class methodsFor: 'initializing' stamp: 'avi 9/30/2003 22:53'!
initialize
	"
	self initialize
	"

	keepAlives := SharedLookupTable new.
	IsLoggingWire := false.
	Smalltalk addToShutDownList: self.! !

!SptHTTPConnection methodsFor: 'test accessing' stamp: ''!
timeoutState: anObject
	"Unit Test helper method"

	timeoutState := anObject! !

!SptHTTPConnection methodsFor: 'accessing' stamp: ''!
host
	^host! !

Object subclass: #SptHTTPUser
	instanceVariableNames: 'username password '
	classVariableNames: 'Users '
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPUser class methodsFor: 'accessing' stamp: ''!
addRealm: aString host: aHostName username: aUsernameString password: aPasswordString
	self assert: [aString first ~= $" and: [aString last ~= $"]].
	self assert: [(aHostName beginsWith: 'http://') not].
	Users at: (Array with: aString with: aHostName)
		put: (self username: aUsernameString password: aPasswordString)! !

!SptHTTPUser class methodsFor: 'accessing' stamp: ''!
users
	^Users! !

!SptHTTPUser class methodsFor: 'initializing' stamp: ''!
uninitialize
	Users := nil! !

!SptHTTPUser methodsFor: 'accessing' stamp: ''!
username
	^username! !

!SptHTTPUser class methodsFor: 'initializing' stamp: ''!
initialize
	"
	self initialize
	"

	
	Users := LookupTable new! !

!SptHTTPUser class methodsFor: 'accessing' stamp: ''!
users: aLookupTable
	Users := aLookupTable! !

!SptHTTPUser class methodsFor: 'instance creation' stamp: ''!
username: aUsernameString password: aPasswordString
	^self new username: aUsernameString password: aPasswordString! !

!SptHTTPUser methodsFor: 'accessing' stamp: ''!
username: aUsernameString password: aPasswordString
	username := aUsernameString.
	password := aPasswordString! !

!SptHTTPUser class methodsFor: 'accessing' stamp: ''!
userFor: aString host: aHostName
	self assert: [(aHostName beginsWith: 'http://') not].
	^Users at: (Array with: aString with: aHostName) ifAbsent: []! !

!SptHTTPUser methodsFor: 'accessing' stamp: ''!
password
	^password! !

Object subclass: #SptHTTPCookie
	instanceVariableNames: 'domain path values expires isSecure '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPCookie commentStamp: '<historical>' prior: 0!
SptHTTPCookie

I am a cookie that is parsed from a Set-Cookie http header

See <http://wp.netscape.com/newsref/std/cookie_spec.html> for more information!

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
isSecure: aBoolean
	isSecure := aBoolean! !

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
isSecure
	^isSecure! !

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
domain: aString
	domain := aString! !

!SptHTTPCookie methodsFor: 'printing' stamp: ''!
printNameValueOn: aWriteStream
	values associationsDo: 
			[:a | 
			aWriteStream
				display: a key;
				nextPut: $=;
				display: a value]
		separatedBy: [aWriteStream nextPut: $;]! !

!SptHTTPCookie methodsFor: 'setting' stamp: ''!
setSecure: aString
	"Note: It is secure"

	isSecure := true! !

!SptHTTPCookie methodsFor: 'testing' stamp: ''!
isPathMatch: aString
	^aString startsWith: path! !

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
atName: aString
	^values at: aString! !

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
path: aString
	self assert: [aString size >= 1].
	self assert: [aString first = $/].
	path := aString! !

!SptHTTPCookie methodsFor: 'testing' stamp: ''!
isSameAs: aSptHTTPCookie
	^path = aSptHTTPCookie path and: [self isDomainMatch: aSptHTTPCookie domain]! !

!SptHTTPCookie methodsFor: 'setting' stamp: ''!
setDomain: aString
	"The cookie contains a 'domain' field.
	-If this value is not legal (ie check against the actual url's host ), signal an error"

	(domain sameAs: aString) ifTrue: [^self].
	(((self class isSpecialDomain: aString) or: [self class isValidDomain: aString]) 
		and: ['*' , aString match: domain ignoreCase: true]) 
			ifTrue: 
				[domain := aString.
				^self].
	self error: ('The cookies suggested domain %1 is not valid for %2' formatWith: aString
				with: domain)! !

!SptHTTPCookie methodsFor: 'setting' stamp: ''!
setExpires: aString
	expires := aString! !

!SptHTTPCookie class methodsFor: 'instance creation' stamp: ''!
allFromSetCookieString: aString url: aSptHTTPUrl
	"Answer a collection of Cookies"

	| cookieStrings pos |
	cookieStrings := OrderedCollection new.
	pos := 1.
	"This is disgusting ... multiple cookies can be separated by commas ... NAME=VALUE strings can not contain a comma, 
		but, expires=DATE does ... arrrg"
	[pos = 0] whileFalse: 
			[pos := self 
						addNextCookieString: aString
						to: cookieStrings
						startingAt: pos].
	^cookieStrings select: [:each | each trimBlanks notEmpty]
		thenCollect: [:each | self fromCookieString: aString url: aSptHTTPUrl]! !

!SptHTTPCookie methodsFor: 'initializing' stamp: 'avi 9/29/2003 13:22'!
initialize
	values := LookupTable new.
	isSecure := false! !

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
path
	^path! !

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
expires
	^expires! !

!SptHTTPCookie class methodsFor: 'helpers' stamp: ''!
isValidDomain: aString
	"SPEC
		and domains must have at least .... three (3) periods "

	^(aString select: [:each | each = $.]) size >= 3! !

!SptHTTPCookie class methodsFor: 'instance creation' stamp: ''!
fromCookieString: aString url: aSptHTTPUrl
	"Answer a single cookie"

	| fields instance |
	fields := LookupTable new.
	(aString subStrings: $;) do: 
			[:each | 
			each trimBlanks notEmpty 
				ifTrue: 
					[| stream key value |
					stream := each readStream.
					key := (stream upTo: $=) trimBlanks.
					value := stream upToEnd trimBlanks.
					(value notEmpty or: [key = 'secure']) ifTrue: [fields at: key put: value]]].
	instance := self newForUrl: aSptHTTPUrl.
	instance setFromFields: fields.
	^instance! !

!SptHTTPCookie class methodsFor: 'instance creation' stamp: ''!
newForUrl: aSptHTTPUrl
	^(self new)
		domain: aSptHTTPUrl host;
		path: aSptHTTPUrl path;
		yourself! !

!SptHTTPCookie methodsFor: 'setting' stamp: ''!
setFromFields: aLookupTable
	(aLookupTable includesKey: 'domain') 
		ifTrue: [self setDomain: (aLookupTable removeKey: 'domain')].
	(aLookupTable includesKey: 'path') 
		ifTrue: [self setPath: (aLookupTable removeKey: 'path')].
	(aLookupTable includesKey: 'expires') 
		ifTrue: [self setExpires: (aLookupTable removeKey: 'expires')].
	(aLookupTable includesKey: 'secure') 
		ifTrue: [self setSecure: (aLookupTable removeKey: 'secure')].
	values := aLookupTable! !

!SptHTTPCookie methodsFor: 'printing' stamp: ''!
nameValuesDo: aTwoArgBlock
	values keysAndValuesDo: aTwoArgBlock! !

!SptHTTPCookie methodsFor: 'testing' stamp: ''!
isDomainMatch: aString
	^aString size > domain size 
		ifTrue: [self isDomainS: domain matchDomainL: aString]
		ifFalse: [self isDomainS: aString matchDomainL: domain]! !

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
domain
	^domain! !

!SptHTTPCookie class methodsFor: 'helpers' stamp: ''!
isSpecialDomain: aString
	aString size < 5 ifTrue: [^false].
	"SPEC: Any domain that fails within one of the seven special top level domains listed below only require two periods"
	(aString select: [:each | each = $.]) size >= 2 ifFalse: [^false].
	^SptHTTPUrl specialDomainEndings 
		includes: (aString copyFrom: aString size - 2 to: aString size) asLowercase! !

!SptHTTPCookie class methodsFor: 'instance creation' stamp: ''!
addNextCookieString: cookieString to: aCollection startingAt: anIndex
	"Private - Answer 0 if no more"

	| expiresPos commaPos |
	expiresPos := cookieString indexOfSubCollection: 'expires' startingAt: anIndex.
	commaPos := cookieString indexOfSubCollection: ',' startingAt: anIndex.
	commaPos > expiresPos 
		ifTrue: 
			["The commaPos is in 'DATE', find the next"

			commaPos := cookieString indexOfSubCollection: ',' startingAt: commaPos + 1].
	"Is it the last cookie?"
	^commaPos = 0 
		ifTrue: 
			[aCollection add: (cookieString copyFrom: anIndex to: cookieString size).
			0]
		ifFalse: 
			[aCollection add: (cookieString copyFrom: anIndex to: commaPos - 1).
			commaPos + 1]! !

!SptHTTPCookie methodsFor: 'testing' stamp: ''!
isDomainS: aSmallerString matchDomainL: aLargerString
	| compString |
	compString := aLargerString copyFrom: aLargerString size - aSmallerString size + 1
				to: aLargerString size.
	^compString sameAs: aSmallerString! !

!SptHTTPCookie methodsFor: 'setting' stamp: ''!
setPath: aString
	self assert: [aString size >= 1].
	self assert: [aString first = $/].
	path := aString! !

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
values
	^values! !

!SptHTTPCookie class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
expires: aString
	expires := aString! !

!SptHTTPCookie methodsFor: 'accessing' stamp: ''!
atName: aString ifAbsentPut: operation
	^values at: aString ifAbsentPut: operation! !

Object subclass: #SptCredentials
	instanceVariableNames: 'user challenge '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptCredentials commentStamp: '<historical>' prior: 0!
SptCredentials

I am the credentials that supply the value of an 'Authorization' http header.

Typically an instance is created by sending a <SptHTTPChallenge> the #credentialsFor: message.

Note: While 'Basic' Authorization could be implemented more simply, creating the <SptCredentials> from the <SptHTTPChallenge> is necessary for 'Digest' Authorization.

See <http://www.ietf.org/rfc/rfc2617.txt> for details on HTTP Authentication.

Instance Variables

	user		SptHTTPUser
	challenge		SptHTTPChallenge!

!SptCredentials methodsFor: 'accessing' stamp: ''!
password
	^user password! !

!SptCredentials methodsFor: 'accessing' stamp: ''!
challenge
	^challenge! !

!SptCredentials methodsFor: 'subclass responsibility' stamp: ''!
isInProtectionSpace: aUrlString
	"Answer whether it is valid to use (or reuse) these credential for aUrlString"

	^self subclassResponsibility! !

SptCredentials subclass: #SptBasicCredentials
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptBasicCredentials methodsFor: 'testing' stamp: ''!
isInProtectionSpace: aUrlString
	"Answer whether it is valid to use (or reuse) these credential for aUrlString"

	^challenge isInProtectionSpace: aUrlString! !

!SptBasicCredentials methodsFor: 'accessing' stamp: ''!
httpHeaderValue
	"Answer the <String> header that authorizes the user in response to the server's challenge"

	^'Basic ' , (self class makeUserIdFor: self username password: self password)! !

!SptBasicCredentials class methodsFor: 'helpers' stamp: ''!
makeUserIdFor: aUsernameString password: aPasswordString
	| outStream |
	outStream := String writeStream.
	Base64Encoder new 
		encode: (aUsernameString , ':' , aPasswordString) asByteArray readStream
		to: outStream.
	^outStream contents! !

!SptCredentials methodsFor: 'accessing' stamp: ''!
user
	^user! !

!SptCredentials methodsFor: 'subclass responsibility' stamp: ''!
httpHeaderValue
	"Answer the <String> header that authorizes the user in response to the server's challenge"

	^self subclassResponsibility! !

!SptCredentials class methodsFor: 'constants' stamp: ''!
icon
	^Icon fromId: 45 in: ShellLibrary default! !

!SptCredentials methodsFor: 'accessing' stamp: ''!
realm
	^challenge realm! !

!SptCredentials class methodsFor: 'instance creation' stamp: ''!
forUser: aSptHTTPUser
	^self new user: aSptHTTPUser! !

!SptCredentials methodsFor: 'accessing' stamp: ''!
username
	^user username! !

!SptCredentials methodsFor: 'accessing' stamp: ''!
user: aSptHTTPUser
	user := aSptHTTPUser! !

!SptCredentials methodsFor: 'accessing' stamp: ''!
challenge: aSptDigestCredentials
	challenge := aSptDigestCredentials! !

Object subclass: #Base64Decoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Compatibility'!

Object subclass: #SptHTTPChallenge
	instanceVariableNames: 'scheme realm params rootUri digestUri method '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPChallenge commentStamp: '<historical>' prior: 0!
SptHTTPChallenge

I am a HTTP Authentication challenge. I contain the information that is parsed from a 'WWW-Authenticate' header included in a 401 (Unauthorized) response.

By supplying user infromation, I can be used to construct the <SptCredentials> required to authorize a request.

See <http://www.ietf.org/rfc/rfc2617.txt> for details on HTTP Authentication.

Instance Variables:
	scheme	<String> ... should be either Basic or Digest
	realm	<String>
	params	<LookupTable>
	rootUri	<String>
	digestUri	<String>
	method	<String>!

!SptHTTPChallenge class methodsFor: 'testing' stamp: ''!
isChallengeString: aString
	| stream |
	stream := aString readStream.
	stream upTo: Character space.
	^(stream nextAvailable: 5) = 'realm'! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
digestUri
	^digestUri! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
opaque
	^params at: 'opaque' ifAbsent: []! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
nonce
	^params at: 'nonce' ifAbsent: []! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
method
	^method! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
credentialsFor: aSptUser
	self isBasic ifFalse: [self error: 'Basic authentication only is supported.'].
	^(SptBasicCredentials new)
		challenge: self;
		user: aSptUser;
		yourself! !

!SptHTTPChallenge class methodsFor: 'instance creation' stamp: ''!
fromChallengeString: aString
	| scheme realm stream |
	stream := aString readStream.
	scheme := stream upTo: Character space.
	stream upTo: $=.
	realm := stream upToEnd.
	^(self new)
		scheme: scheme;
		realm: (realm copyFrom: 2 to: realm size - 1);
		yourself! !

!SptHTTPChallenge class methodsFor: 'instance creation' stamp: ''!
readAllFrom: aReadStream
	| subStrings instances |
	subStrings := self csStringsFrom: aReadStream.
	instances := OrderedCollection new.
	subStrings keysAndValuesDo: 
			[:i :each | 
			(self isChallengeString: each) 
				ifTrue: [instances add: (self fromChallengeString: each)]
				ifFalse: 
					[i = 1 ifTrue: [self error: 'Error parsing WWW-Authenticate header'].
					instances last parseParameter: each]].
	^instances! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
realm
	^realm! !

!SptHTTPChallenge methodsFor: 'testing' stamp: ''!
isBasic
	^scheme asLowercase = 'basic'! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
method: aString
	method := aString! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
digestUri: aString
	digestUri := aString! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
rootUri
	^rootUri! !

!SptHTTPChallenge class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
params: aCollection
	params := aCollection! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
qop
	^params at: 'qop' ifAbsent: []! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
domain: aString
	params at: 'domain' put: aString! !

!SptHTTPChallenge methodsFor: 'operations' stamp: ''!
parseParameter: aString
	| stream token value |
	stream := aString readStream.
	token := stream upTo: $=.
	value := stream upToEnd.
	params at: token put: (value copyFrom: 2 to: value size - 1)! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
absoluteUrl: aString
	| url |
	url := SptHTTPUrl fromString: aString.
	rootUri := url rootBaseUrlString.
	self domain: url path! !

!SptHTTPChallenge class methodsFor: 'instance creation' stamp: ''!
newBasicChallenge
	^(self new)
		scheme: 'Basic';
		yourself! !

!SptHTTPChallenge class methodsFor: 'helpers' stamp: ''!
makeUserIdFor: aUsernameString password: aPasswordString
	| outStream |
	outStream := String writeStream.
	Base64Encoder new 
		encode: (aUsernameString , ':' , aPasswordString) asByteArray readStream
		to: outStream.
	^outStream contents! !

!SptHTTPChallenge class methodsFor: 'helpers' stamp: ''!
csStringsFrom: stream
	"Answer a collection of comma-separated strings ... ignoring commas embedded in quoted strings"

	| coll string start isQuoted |
	coll := OrderedCollection new.
	string := stream contents.
	[stream atEnd] whileFalse: 
			[start := stream position + 1.
			isQuoted := false.
			stream skipWhile: 
					[:c | 
					c = $" ifTrue: [isQuoted := isQuoted not].
					isQuoted or: [c ~= $,]].
			coll add: (string copyFrom: start to: stream position) trimBlanks.
			"skip ,"
			stream atEnd ifFalse: [stream next].
			stream skipSeparators].
	^coll! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
rootUri: aString
	rootUri := aString! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
domain
	^params at: 'domain' ifAbsent: ['/']! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
realm: aString
	realm := aString! !

!SptHTTPChallenge methodsFor: 'initializing' stamp: 'avi 9/29/2003 13:18'!
initialize
	params := LookupTable new! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
credentialsForUsername: aString password: aString2
	^self credentialsFor: (SptHTTPUser username: aString password: aString2)! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
protectionSpaces
	| url |
	url := SptHTTPUrl fromString: rootUri.
	^(self domain subStrings: $ ) 
		collect: [:each | (SptHTTPUrl fromString: each previous: url) urlString]! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
scheme: aString
	scheme := aString! !

!SptHTTPChallenge methodsFor: 'testing' stamp: ''!
isInProtectionSpace: aString
	"SPEC:
The protection space determines the domain over which credentials can be automatically applied. If a prior request has been authorized, the same credentials MAY be reused for all other requests within that protection space for a period of time determined by the authentication scheme, parameters, and/or user preference. "

	| url |
	url := SptHTTPUrl fromString: aString.
	^self protectionSpaces anySatisfy: [:each | url isInSpaceOf: each]! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
params
	^params! !

!SptHTTPChallenge class methodsFor: 'instance creation' stamp: ''!
newBasicChallengeRealm: aString absoluteUri: aUrlString
	^(self newBasicChallenge)
		realm: aString;
		absoluteUrl: aUrlString;
		yourself! !

!SptHTTPChallenge methodsFor: 'accessing' stamp: ''!
scheme
	^scheme! !

Object subclass: #SptHTTPStatus
	instanceVariableNames: 'statusLine '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Core'!

!SptHTTPStatus commentStamp: '<historical>' prior: 0!
I am the status of an HTTP Response message

Instance Variables:
	statusLine	<String>!

!SptHTTPStatus class methodsFor: 'instance creation' stamp: ''!
fromString: aString
	^self new statusLine: aString! !

!SptHTTPStatus methodsFor: 'testing' stamp: ''!
isVersion11
	| stream |
	stream := statusLine readStream.
	self assert: [(stream next: 5) = 'HTTP/'].
	^(stream upTo: Character space) = '1.1'! !

!SptHTTPStatus methodsFor: 'accessing' stamp: ''!
statusText
	"Answer the Status text of the response"

	"
	(self fromString: 'HTTP/1.1 404 Not Found') statusText
		'Not Found'
	"

	| stream |
	stream := statusLine readStream.
	stream upTo: Character space.
	stream upTo: Character space.
	^stream upToEnd! !

!SptHTTPStatus methodsFor: 'exceptions' stamp: ''!
httpExceptionClass
	^self isClientError 
		ifTrue: [SptHTTPClientError]
		ifFalse: 
			[self isServerError 
				ifTrue: [SptHTTPServerError]
				ifFalse: 
					["? If you say so"

					SptHTTPError]]! !

!SptHTTPStatus methodsFor: 'accessing' stamp: ''!
statusCodeString
	"Answer the status code of the response"

	"
	(self fromString: 'HTTP/1.1 404 Not Found') statusCodeString
		'404'
	"

	| stream |
	stream := statusLine readStream.
	stream upTo: Character space.
	^stream upTo: Character space! !

!SptHTTPStatus methodsFor: 'test accessing' stamp: ''!
statusLine: aString
	statusLine := aString! !

!SptHTTPStatus methodsFor: 'accessing' stamp: ''!
statusLine
	"Answer the response status line"

	^statusLine! !

!SptHTTPStatus methodsFor: 'testing' stamp: ''!
isClientError
	^self statusCodeString first = $4! !

!SptHTTPStatus methodsFor: 'accessing' stamp: ''!
status
	"Answer the status code of the response"

	"
	(self fromString: 'HTTP/1.1 404 Not Found') status
		404
	"

	^self statusCodeString asNumber! !

!SptHTTPStatus methodsFor: 'testing' stamp: ''!
isInformational
	^self statusCodeString first = $1! !

!SptHTTPStatus methodsFor: 'range requests' stamp: 'bkv 10/30/2003 18:36'!
isUnsatisfiableRangeRequest

^ self status = 416
! !

!SptHTTPStatus methodsFor: 'testing' stamp: ''!
isSuccessful
	^self statusCodeString first = $2! !

!SptHTTPStatus methodsFor: 'testing' stamp: ''!
isClientOrServerError
	^self statusCode >= 400! !

!SptHTTPStatus methodsFor: 'exceptions' stamp: ''!
httpException
	"Answer an exception instance that describes the error"

	"
	(self fromString: 'HTTP/1.1 404 Not Found') httpException signal
	"

	| exClass |
	exClass := self httpExceptionClass.
	^(exClass new)
		httpStatus: self;
		yourself! !

!SptHTTPStatus methodsFor: 'displaying' stamp: ''!
displayOn: stream
	statusLine displayOn: stream! !

!SptHTTPStatus methodsFor: 'range requests' stamp: 'bkv 10/30/2003 18:35'!
isPartialContent

^ self status = 206
! !

!SptHTTPStatus methodsFor: 'accessing' stamp: ''!
statusCode
	"Answer the status code of the response"

	"
	(self fromString: 'HTTP/1.1 404 Not Found') statusCode
		404
	"

	^self status! !

!SptHTTPStatus methodsFor: 'testing' stamp: ''!
isServerError
	^self statusCodeString first = $5! !

!SequenceableCollection class methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:24'!
readStream
	^ self new readStream! !

!SequenceableCollection methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:41'!
prevIndexOf: anObject from: startIndex to: endIndex
	startIndex to: endIndex by: -1 do:
		[:i |
		(self at: i) = anObject ifTrue: [^ i]].
	^ 0! !

!SequenceableCollection class methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:24'!
writeStream
	^ self new writeStream! !

!Delay class methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:47'!
millisecondClockValue
	^ Time millisecondClockValue! !

TestCase subclass: #SptHTTPTests
	instanceVariableNames: 'usersSingleton '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

SptHTTPTests subclass: #SptHTTPUserTest
	instanceVariableNames: 'user '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPUserTest methodsFor: 'unit tests' stamp: ''!
setUp
	super setUp.
	user := SptHTTPUser username: 'steve' password: 'dolphin'! !

!SptHTTPUserTest methodsFor: 'unit tests' stamp: ''!
testUsers
	SptHTTPUser 
		addRealm: 'foo'
		host: 'www.bar.org'
		username: 'steve'
		password: 'dolphin'.
	user := SptHTTPUser userFor: 'foo' host: 'www.bar.org'.
	self assert: user username = 'steve'.
	self assert: user password = 'dolphin'.
	self assert: (SptHTTPUser userFor: 'bar' host: 'www.bar.org') isNil.
	self assert: (SptHTTPUser userFor: 'foo' host: 'www.foo.org') isNil! !

!SptHTTPUserTest methodsFor: 'unit tests' stamp: ''!
testConstruction
	self assert: user username = 'steve'.
	self assert: user password = 'dolphin'! !

!SptHTTPTests methodsFor: 'unclassified' stamp: ''!
tearDown
	super tearDown.
	SptHTTPUser users: usersSingleton! !

!SptHTTPTests methodsFor: 'unclassified' stamp: ''!
setUp
	super setUp.
	usersSingleton := SptHTTPUser users.
	SptHTTPUser users: LookupTable new! !

SptHTTPTests subclass: #SptBasicCredentialsTest
	instanceVariableNames: 'credentials '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptBasicCredentialsTest methodsFor: 'unit tests' stamp: ''!
testIsInProtectionSpace
	self deny: (credentials isInProtectionSpace: 'http://www.foo.org/').
	self assert: (credentials isInProtectionSpace: 'http://www.foo.org/bar/baz')! !

!SptBasicCredentialsTest methodsFor: 'unit tests' stamp: ''!
testHttpHeaderValue
	self assert: credentials httpHeaderValue = 'Basic d2FsbHk6d29ybGQ='! !

!SptBasicCredentialsTest methodsFor: 'set up' stamp: ''!
setUp
	| challenge |
	super setUp.
	challenge := (SptHTTPChallenge readAllFrom: 'Basic realm="WallyWorld"' readStream) first.
	challenge absoluteUrl: 'http://www.foo.org/bar/index.html'.
	credentials := challenge 
				credentialsFor: (SptHTTPUser username: 'wally' password: 'world')! !

!SptHTTPTests methodsFor: 'unclassified' stamp: ''!
crlf
	^String lineDelimiter! !

SptHTTPTests subclass: #SptHTTPProxySettingsTest
	instanceVariableNames: 'settings '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPProxySettingsTest methodsFor: 'unit tests' stamp: ''!
testUseProxy
	settings useProxy: true.
	self assert: settings useProxy.
	settings useProxy: false.
	self deny: settings useProxy.! !

!SptHTTPProxySettingsTest methodsFor: 'set up' stamp: ''!
setUp
	super setUp.
	settings := SptHTTPProxySettings new! !

!SptHTTPProxySettingsTest methodsFor: 'unit tests' stamp: ''!
testEmpty
	self deny: SptHTTPProxySettings new useProxy! !

!SptHTTPProxySettingsTest methodsFor: 'unit tests' stamp: ''!
testSetting
	settings proxyHost: 'foo'.
	settings proxyPort: 1234.
	self assert: settings proxyHost = 'foo'.
	self assert: settings proxyPort = 1234.! !

!SptHTTPProxySettingsTest methodsFor: 'unit tests' stamp: ''!
testLazy
	self assert: settings proxyHost = 'localhost'.
	self assert: settings proxyPort = 8000.! !

!SptHTTPTests class methodsFor: 'testing' stamp: ''!
isAbstract
	"Override to true if a TestCase subclass is Abstract and should not have
	TestCase instances built from it"

	^self name asString last = $s! !

!SptHTTPTests methodsFor: 'accessing' stamp: ''!
getTestConfDeaf
	^(SptHTTPConfiguration newDefault)
		httpConnectionClass: DeafObject current;
		httpsConnectionClass: DeafObject current;
		progressClass: SptHTTPTestProgress;
		yourself! !

!SptHTTPTests methodsFor: 'accessing' stamp: ''!
getConnectionReading: aString
	^SptHTTPConnection new socketStream: (SZReadWriteSocketStream testStreamReading: aString)! !

SptHTTPTests subclass: #SptHTTPHostCacheTest
	instanceVariableNames: 'cache '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPHostCacheTest methodsFor: 'accessing' stamp: 'avi 9/29/2003 13:04'!
addEntries
	cache cache at: 'www.foo.org' put: (cache newEntryForIP: #(1 2 3 4)).
	cache cache at: 'www.foo.com' put: (cache newEntryForIP: #(5 6 7 8))! !

!SptHTTPHostCacheTest methodsFor: 'unit tests' stamp: 'avi 9/29/2003 13:04'!
testExpired
	self addEntries.
	(cache cache at: 'www.foo.org') at: 1
		put: Delay millisecondClockValue - (cache freshMilliseconds * 2).
	self assert: (cache ipAddressFor: 'www.foo.org') = #(1 1 1 1)! !

!SptHTTPHostCacheTest methodsFor: 'accessing' stamp: 'avi 9/29/2003 13:04'!
ipAddressFromString: aString
	^#(1 1 1 1)! !

!SptHTTPHostCacheTest methodsFor: 'unit tests' stamp: 'avi 9/29/2003 13:04'!
testIPAddrressFor
	self addEntries.
	self assert: (cache ipAddressFor: 'www.foo.org') = #(1 2 3 4).
	self assert: (cache ipAddressFor: 'www.foo.com') = #(5 6 7 8).
	self assert: (cache ipAddressFor: 'www.Notfoo.com') = #(1 1 1 1)! !

!SptHTTPHostCacheTest methodsFor: 'set up' stamp: ''!
setUp
	super setUp.
	cache := SptHTTPHostCache new.
	cache ipFactory: self! !

SptHTTPTests subclass: #SptHTTPSessionStateTest
	instanceVariableNames: 'state '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPSessionStateTest methodsFor: 'unit tests' stamp: ''!
testInitializationDefaults
	self assert: state cookieJar class == SptHTTPCookieJar.
	self assert: state proxySettings isNil.
	self assert: state credentials isNil.
	self assert: state timeout = SptHTTPSessionState defaultTimeout.
	self assert: state maxRedirects = SptHTTPSessionState defaultMaxRedirects! !

!SptHTTPSessionStateTest methodsFor: 'unit tests' stamp: ''!
testUseProxy
	self deny: state useProxy.
	self setProxy.
	self assert: state useProxy.
	state proxySettings useProxy: false.
	self deny: state useProxy! !

!SptHTTPSessionStateTest methodsFor: 'accessing' stamp: ''!
setProxy
	state proxySettings: ((SptHTTPProxySettings new)
				useProxy: true;
				proxyHost: 'aHost';
				proxyPort: 4321;
				yourself)! !

!SptHTTPSessionStateTest methodsFor: 'unit tests' stamp: ''!
testAddCookie
	state addCookie: self getFooBarCookie.
	self assert: state cookieJar numberOfCookies = 1! !

!SptHTTPSessionStateTest methodsFor: 'set up' stamp: ''!
setUp
	super setUp.
	state := SptHTTPSessionState new! !

!SptHTTPSessionStateTest methodsFor: 'unit tests' stamp: ''!
testCookieHeaderFor
	| url |
	url := SptHTTPUrl fromString: 'http://www.foo.org/bar'.
	self assert: (state cookieHeaderFor: url) = ''.
	state addCookie: self getFooBarCookie.
	self assert: (state cookieHeaderFor: url) = 'n2=v2'! !

!SptHTTPSessionStateTest methodsFor: 'unit tests' stamp: ''!
testUsernamePasswordFor
	state 
		username: 'Aladdin'
		password: 'open sesame'
		for: 'http://www.foo.org/bar'.
	self assert: state credentials username = 'Aladdin'.
	self assert: state credentials password = 'open sesame'.
	self assert: state authorizationHeader = 'Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ=='.
	state 
		username: 'Aladdin1'
		password: 'open sesame1'
		for: 'http://www.foo.org/bar'.
	self assert: state credentials username = 'Aladdin1'.
	self assert: state credentials password = 'open sesame1'! !

!SptHTTPSessionStateTest methodsFor: 'unit tests' stamp: ''!
testShouldAuthenticate
	state 
		username: 'Aladdin'
		password: 'open sesame'
		for: 'http://www.foo.org/bar/index.html'.
	self assert: (state shouldAuthenticate: 'http://www.foo.org/bar/index.html').
	self assert: (state shouldAuthenticate: 'http://www.foo.org/bar/index2.html').
	self assert: (state shouldAuthenticate: 'http://www.foo.org/bar/').
	self deny: (state shouldAuthenticate: 'http://www.foo.org/bar').
	self deny: (state shouldAuthenticate: 'http://www.foo.org/foo/index.html').
	self deny: (state shouldAuthenticate: 'http://www.NOTfoo.org/bar/index.html')! !

SptHTTPTests subclass: #SptHTTPCookieTest
	instanceVariableNames: 'cookie '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testIsSpecialDomain
	#('www.foo.com' '.foo.com' '.foo.COM' '.foo.edu' '.foo.net' '.foo.org' '.foo.gov' '.foo.mil' '.foo.int') 
		do: [:each | self assert: (SptHTTPCookie isSpecialDomain: each)].
	#('foo.com' '.com' 'com') do: [:each | self deny: (SptHTTPCookie isSpecialDomain: each)]! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testMultipleNames
	cookie := SptHTTPCookie 
				fromCookieString: 'CUSTOMER=WILE_E_COYOTE; PART_NUMBER=ROCKET_LAUNCHER_0001; path=/; expires=Wednesday, 09-Nov-99 23:12:40 GMT'
				url: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: cookie domain = 'www.foo.org'.
	self assert: cookie path = '/'.
	self assert: (cookie atName: 'CUSTOMER') = 'WILE_E_COYOTE'.
	self assert: (cookie atName: 'PART_NUMBER') = 'ROCKET_LAUNCHER_0001'! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testIsDomainMatch
	cookie := SptHTTPCookie newForUrl: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	#('wwww.foo.org' 'www.www.foo.org' 'www.foo.org' 'WWW.foo.org' 'foo.org') 
		do: [:each | self assert: (cookie isDomainMatch: each)].
	#('www.foo.or' 'foo.org.bar' 'foo') do: [:each | self deny: (cookie isDomainMatch: each)]! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testDomainSpecialValid
	cookie := SptHTTPCookie 
				fromCookieString: 'CUSTOMER=WILE_E_COYOTE; path=/; domain=.foo.org'
				url: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: cookie domain = '.foo.org'! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testIsValidDomain
	#('www.abc.foo.us' '.abc.foo.us') 
		do: [:each | self assert: (SptHTTPCookie isValidDomain: each)].
	#('abc.foo.us' 'foo.us') do: [:each | self deny: (SptHTTPCookie isValidDomain: each)]! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testAllFromMultiple
	| cookies |
	cookies := SptHTTPCookie 
				allFromSetCookieString: 'session-id=002-7620309-1728015; path=/; domain=.amazon.com; expires=Wednesday, 08-Jan-2003 08:00:00 GMT, session-id-time=1042012800; path=/; domain=.amazon.com; expires=Wednesday, 08-Jan-2003 08:00:00 GMT, obidos_path_continue-shopping=continue-shopping-url=/subst/home/home.html/002-7620309-1728015&continue-shopping-post-data=&continue-shopping-description=generic.gateway.default; path=/; domain=.amazon.com'
				url: (SptHTTPUrl fromString: 'http://www.amazon.com').
	self assert: cookies size = 3! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testDomainNotSpecialInvalid1
	self should: 
			[SptHTTPCookie fromCookieString: 'CUSTOMER=WILE_E_COYOTE; path=/; domain=foo.org.au'
				url: (SptHTTPUrl fromString: 'http://www.foo.org.au/bar')]
		raise: Error.
	self should: 
			[SptHTTPCookie fromCookieString: 'CUSTOMER=WILE_E_COYOTE; path=/; domain=org.au'
				url: (SptHTTPUrl fromString: 'http://www.foo.org.au/bar')]
		raise: Error! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testIsPathMatch
	cookie := SptHTTPCookie newForUrl: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	#('/bar' '/barNoKidding' '/bar/' '/bar/foo') 
		do: [:each | self assert: (cookie isPathMatch: each)].
	#('/' '/foo' '/foo/bar' '/foobar') do: [:each | self deny: (cookie isPathMatch: each)]! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testNoPath
	cookie := SptHTTPCookie fromCookieString: 'CUSTOMER=WILE_E_COYOTE'
				url: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: cookie domain = 'www.foo.org'.
	self assert: cookie path = '/bar'.
	self assert: (cookie atName: 'CUSTOMER') = 'WILE_E_COYOTE'! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testDomainNotSpecialValid
	cookie := SptHTTPCookie 
				fromCookieString: 'CUSTOMER=WILE_E_COYOTE; path=/; domain=.foo.org.au'
				url: (SptHTTPUrl fromString: 'http://www.foo.org.au/bar').
	self assert: cookie domain = '.foo.org.au'! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testExample
	cookie := SptHTTPCookie 
				fromCookieString: 'CUSTOMER=WILE_E_COYOTE; path=/; expires=Wednesday, 09-Nov-99 23:12:40 GMT'
				url: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: cookie domain = 'www.foo.org'.
	self assert: cookie path = '/'.
	self assert: (cookie atName: 'CUSTOMER') = 'WILE_E_COYOTE'! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testDomainSame
	cookie := SptHTTPCookie 
				fromCookieString: 'CUSTOMER=WILE_E_COYOTE; path=/; domain=www.foo.org'
				url: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: cookie domain = 'www.foo.org'! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testDomainSpecialInvalid1
	self should: 
			[SptHTTPCookie fromCookieString: 'CUSTOMER=WILE_E_COYOTE; path=/; domain=foo.org'
				url: (SptHTTPUrl fromString: 'http://www.foo.org/bar')]
		raise: Error! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testIgnoreNoEquals
	cookie := SptHTTPCookie fromCookieString: 'CUSTOMER=WILE_E_COYOTE; path=/; junk'
				url: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: cookie domain = 'www.foo.org'.
	self assert: cookie path = '/'.
	self assert: (cookie atName: 'CUSTOMER') = 'WILE_E_COYOTE'.
	self should: [cookie atName: 'junk'] raise: Error! !

!SptHTTPCookieTest methodsFor: 'unit tests' stamp: ''!
testAllFromSingle
	| cookies |
	cookies := SptHTTPCookie 
				allFromSetCookieString: 'CUSTOMER=WILE_E_COYOTE; path=/; domain=.foo.org.au'
				url: (SptHTTPUrl fromString: 'http://www.foo.org.au/bar').
	self assert: cookies size = 1! !

SptHTTPTests subclass: #SptHTTPStatusTest
	instanceVariableNames: 'status '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPStatusTest methodsFor: 'set up' stamp: ''!
setUp
	super setUp.
	status := SptHTTPStatus fromString: 'HTTP/1.1 200 OK'! !

!SptHTTPStatusTest methodsFor: 'unit tests' stamp: ''!
testIsSuccess
	self setStatusOn: 'HTTP/1.1 200 OK'.
	self assert: status isSuccessful.
	self setStatusOn: 'HTTP/1.1 207'.
	self assert: status isSuccessful.
	self setStatusOn: 'HTTP/1.1 404'.
	self deny: status isSuccessful! !

!SptHTTPStatusTest methodsFor: 'unit tests' stamp: ''!
statusCodeString
	self setStatusOn: 'HTTP/1.1 200 OK'.
	self assert: status statusCodeString = '200'! !

!SptHTTPStatusTest methodsFor: 'unit tests' stamp: ''!
testIsClientOrServerError
	self setStatusOn: 'HTTP/1.1 200 OK'.
	self deny: status isClientOrServerError.
	self deny: status isClientError.
	self deny: status isServerError.
	self setStatusOn: 'HTTP/1.1 404'.
	self assert: status isClientOrServerError.
	self assert: status isClientError.
	self deny: status isServerError.
	self setStatusOn: 'HTTP/1.1 500'.
	self assert: status isClientOrServerError.
	self deny: status isClientError.
	self assert: status isServerError! !

!SptHTTPStatusTest methodsFor: 'unit tests' stamp: ''!
testIsInformational
	self setStatusOn: 'HTTP/1.1 200 OK'.
	self deny: status isInformational.
	self setStatusOn: 'HTTP/1.1 100'.
	self assert: status isInformational.
	self setStatusOn: 'HTTP/1.1 102'.
	self assert: status isInformational! !

!SptHTTPStatusTest methodsFor: 'accessing' stamp: ''!
setStatusOn: aString
	status := SptHTTPStatus fromString: aString! !

!SptHTTPStatusTest methodsFor: 'unit tests' stamp: ''!
testHttpClientException
	self setStatusOn: 'HTTP/1.1 404'.
	self 
		should: [status httpException signal]
		raise: SptHTTPClientError
		description: 'Client Error  HTTP/1.1 404'! !

!SptHTTPStatusTest methodsFor: 'unit tests' stamp: ''!
testIsVersion11
	self setStatusOn: 'HTTP/1.1 200 OK'.
	self assert: status isVersion11.
	self setStatusOn: 'HTTP/1.0 200 OK'.
	self deny: status isVersion11! !

!SptHTTPStatusTest methodsFor: 'unit tests' stamp: ''!
testHttpServerException
	self setStatusOn: 'HTTP/1.1 500'.
	self 
		should: [status httpException signal]
		raise: SptHTTPServerError
		description: 'Server Error  HTTP/1.1 500'! !

!SptHTTPStatusTest methodsFor: 'unit tests' stamp: ''!
testStatusCode
	self setStatusOn: 'HTTP/1.1 200 OK'.
	self assert: (status statusCode isKindOf: Integer)! !

SptHTTPTests subclass: #SptHTTPUrlTest
	instanceVariableNames: 'url '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteRelError
	self setUrlString: '../x' previousString: 'http://www.ics.uci.edu/Test'.
	self assert: url urlString = 'http://www.ics.uci.edu/x'.
	self setUrlString: '../../x' previousString: 'http://www.ics.uci.edu/Test/a'.
	self assert: url urlString = 'http://www.ics.uci.edu/x'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testIsDescendent
	#(#('http://foo/bar/baz' 'http://foo/bar/') #('http://foo/bar/baz/foo' 'http://foo/bar/') #('/foo' '/') #('/foo/bar' '/') #('/foo/bar/' '/')) 
		do: [:each | self assert: (SptHTTPUrl is: each first descendentOf: each second)].
	#(#('http://foo/bar/' 'http://foo/bar/') #('/' '/') #('http://foo/bar/' 'http://foo/baz/')) 
		do: [:each | self deny: (SptHTTPUrl is: each first descendentOf: each second)]! !

!SptHTTPUrlTest methodsFor: 'asserting' stamp: ''!
assertIsFoo80
	self assert: url host = 'www.foo.org'.
	self assert: url port = 80! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteAbsolute
	self setUrlString: 'http://www.foo.org/bar' previousString: 'http://www.foo.org/bar'.
	self assertIsFoo80.
	self assert: url absolutePath = '/bar'.
	self setUrlString: 'http://www.foo.org/bar?q=v' previousString: 'http://www.foo.org/bar'.
	self assertIsFoo80.
	self assert: url absolutePath = '/bar?q=v'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeUrlRefString
	#(#('http://foo/bar/baz' 'http://foo/bar/baz') #('https://foo/bar/baz' 'https://foo/bar/baz') #('HTTP://foo/bar/baz' 'http://foo/bar/baz') #('HTTPS://foo/bar/baz' 'https://foo/bar/baz')) 
		do: [:each | self assert: (SptHTTPUrl makeUrlRefString: each first) = each second].
	#(#('www.foo/bar/baz' 'http://www.foo/bar/baz') #('WWW.foo/bar/baz' 'http://WWW.foo/bar/baz') #('foo/bar/baz' 'http://www.foo/bar/baz')) 
		do: [:each | self assert: (SptHTTPUrl makeUrlRefString: each first) = each second]! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testQueryStringsOne
	| ans |
	url := SptHTTPUrl fromString: 'http://foo/bar/baz?a'.
	ans := url queryStrings.
	self assert: ans size = 1.
	self assert: ans first = ('a' -> nil).
	""
	url := SptHTTPUrl fromString: 'http://foo/bar/baz?a=b'.
	ans := url queryStrings.
	self assert: ans size = 1.
	self assert: ans first = ('a' -> 'b')! !

!SptHTTPUrlTest methodsFor: 'accessing' stamp: ''!
assert: aCString isCanonicalOf: aString
	| aStringCanonical aCStringUncanonical |
	aStringCanonical := SptHTTPUrl canonicalize: aString.
	self assert: aStringCanonical = aCString.
	aCStringUncanonical := SptHTTPUrl uncanonicalize: aCString.
	self assert: aCStringUncanonical = aString! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: 'avi 9/30/2003 20:26'!
testSplitExtension
	#(#('http://foo/bar' nil) #('http://foo/bar.' '') #('http://foo/bar.html' 'html') #('http://foo/bar/' nil) #('http://foo/bar/baz.html' 'html')) 
		do: [:each | self assert: (SptHTTPUrl extensionOf: each first) asString = each second]! !

!SptHTTPUrlTest methodsFor: 'accessing' stamp: ''!
setUrlString: aString previousString: aString2
	| prev |
	prev := SptHTTPUrl fromString: aString2.
	url := SptHTTPUrl fromString: aString previous: prev! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testIsAbsolute
	#('http://foo/bar/baz' 'https://foo/bar/baz') 
		do: [:each | self assert: (SptHTTPUrl isAbsolute: each)].
	#('/foo/bar/' '/f' '/' '') do: [:each | self deny: (SptHTTPUrl isAbsolute: each)]! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testIsCommonNonHTTPScheme
	#('http://foo/bar/baz' 'https://foo/bar/' '/q=:no' '/foo' '/' '') 
		do: [:each | self deny: (SptHTTPUrl isCommonNonHTTPScheme: each)].
	#('ftp://foo' 'FTP://foo' 'emailto:you' 'mailto:you' 'telnet://you' 'gopher://you/' 'news:you' 'NEWS:you' 'javascript:eeck') 
		do: [:each | self assert: (SptHTTPUrl isCommonNonHTTPScheme: each)]! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testIsChild
	#(#('http://foo/bar/baz' 'http://foo/bar/') #('http://foo/bar/baz/' 'http://foo/bar/') #('/foo' '/')) 
		do: [:each | self assert: (SptHTTPUrl is: each first childOf: each second)].
	#(#('http://foo/bar/baz/foo' 'http://foo/bar/') #('http://foo/bar/' 'http://foo/bar/') #('/' '/') #('http://foo/bar/' 'http://foo/baz/')) 
		do: [:each | self deny: (SptHTTPUrl is: each first childOf: each second)]! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: 'avi 9/30/2003 20:26'!
testSplitFragment
	self assert: (SptHTTPUrl splitFragmentOf: 'http://foo/bar') = {'http://foo/bar'. nil}.
	self assert: (SptHTTPUrl splitFragmentOf: 'http://foo/bar#') = #('http://foo/bar' '').
	self 
		assert: (SptHTTPUrl splitFragmentOf: 'http://foo/bar#baz') = #('http://foo/bar' 'baz').
	self assert: (SptHTTPUrl splitFragmentOf: '#baz') = #('' 'baz')! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteLocalQuery
	self setUrlString: '/bar2' previousString: 'http://www.foo.org/baz/bar?q=v'.
	self assertIsFoo80.
	self assert: url absolutePath = '/bar2'! !

!SptHTTPUrlTest methodsFor: 'accessing' stamp: ''!
setFoo1234Bar
	url := SptHTTPUrl fromString: 'http://www.foo.org:1234/bar'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteAbsolutePath
	self setUrlString: 'bar2' previousString: 'http://www.foo.org/baz/bar'.
	self assertIsFoo80.
	self assert: url absolutePath = '/baz/bar2'.
	self setUrlString: 'bar2?q1=v1' previousString: 'http://www.foo.org/baz/bar?q=v'.
	self assertIsFoo80.
	self assert: url absolutePath = '/baz/bar2?q1=v1'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testLastSegment
	#(#('http://host/' '/') #('http://host/foo' 'foo') #('http://host/foo/' 'foo') #('http://host/foo/bar' 'bar') #('http://host/foo/bar/' 'bar')) 
		do: [:each | self assert: (SptHTTPUrl fromString: each first) lastSegment = each second].
	#(#('http://host/foo/bar' 'bar') #('http://host/foo/bar?boo' 'bar') #('http://host/foo/bar?boo/' 'bar') #('http://host/foo/bar?boo/boo' 'bar')) 
		do: [:each | self assert: (SptHTTPUrl fromString: each first) lastSegment = each second]! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteRelUp
	self setUrlString: '../x' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/a/x'.
	self setUrlString: '../../x' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/x'.
	self setUrlString: '../../../x' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/x'.
	self setUrlString: '../x.' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/a/x.'.
	self setUrlString: '../x/.' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/a/x/'.
	self setUrlString: '../x/..' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/a/x'.
	self setUrlString: 'x/..' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/a/b/x'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteAbsolutePathSlash
	self setUrlString: 'bar2' previousString: 'http://www.foo.org/baz/bar/'.
	self assertIsFoo80.
	self assert: url absolutePath = '/baz/bar/bar2'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteAbsolutePathQuery
	self setUrlString: 'bar2' previousString: 'http://www.foo.org/baz/bar?q=s'.
	self assertIsFoo80.
	self assert: url path = '/baz/bar2'.
	self assert: url absolutePath = '/baz/bar2'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: 'avi 9/30/2003 20:25'!
testContainerOf
	#(#('http://www.foo.org/' nil) #('http://www.foo.org/foo' 'http://www.foo.org/') #('http://www.foo.org/foo/' 'http://www.foo.org/') #('http://www.foo.org/foo/bar' 'http://www.foo.org/foo/') #('http://www.foo.org/foo/bar/' 'http://www.foo.org/foo/')) 
		do: [:each | self assert: (SptHTTPUrl containerOf: each first) asString = each second].
	#(#('/' nil) #('/foo' '/') #('/foo/' '/') #('/foo/bar' '/foo/') #('/foo/bar/' '/foo/')) 
		do: [:each | self assert: (SptHTTPUrl containerOf: each first) asString = each second]! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testPort
	self setUrlString: 'http://www.foo.org:1234'.
	self assertIsFoo1234.
	self assert: url path = '/'.
	self setUrlString: 'http://www.foo.org:1234/'.
	self assertIsFoo1234.
	self assert: url path = '/'.
	self setUrlString: 'http://www.foo.org:1234/bar'.
	self assertIsFoo1234.
	self assert: url path = '/bar'.
	self setUrlString: 'http://www.foo.org:1234/bar?baz'.
	self assertIsFoo1234.
	self assert: url absolutePath = '/bar?baz'.
	self assert: url path = '/bar'.
	self should: [self setUrlString: 'www.foo.org:1234/bar'] raise: Error! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testLastSegmentAndQuery
	#(#('http://host/' '/') #('http://host/foo' 'foo') #('http://host/foo/' 'foo') #('http://host/foo/bar' 'bar') #('http://host/foo/bar/' 'bar')) 
		do: [:each | self assert: (SptHTTPUrl fromString: each first) lastSegmentAndQuery = each second].
	#(#('http://host/foo/bar' 'bar') #('http://host/foo/bar?boo' 'bar?boo') #('http://host/foo/bar?boo/' 'bar?boo/') #('http://host/foo/bar?boo/boo' 'bar?boo/boo')) 
		do: [:each | self assert: (SptHTTPUrl fromString: each first) lastSegmentAndQuery = each second]! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: 'avi 9/30/2003 20:26'!
testExtensionOf
	#(#('http://www.foo.org/' nil) #('http://www.foo.org/foo' nil) #('http://www.foo.org/foo.' '') #('http://www.foo.org/foo.bar' 'bar')) 
		do: [:each | self assert: (SptHTTPUrl extensionOf: each first) asString = each second]! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testHostWithUserInfo
	self setUrlString: 'http://u:p@www.foo.org'.
	self assertIsFoo80.
	self assert: url path = '/'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteAbsolutePathSeg
	self setUrlString: 'bar2' previousString: 'http://www.foo.org/baz/bar#s'.
	self assertIsFoo80.
	self assert: url absolutePath = '/baz/bar2'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteRelRoot
	self setUrlString: 'x' previousString: 'http://www.ics.uci.edu/'.
	self assert: url urlString = 'http://www.ics.uci.edu/x'.
	self setUrlString: '../x' previousString: 'http://www.ics.uci.edu/'.
	self assert: url urlString = 'http://www.ics.uci.edu/x'.
	self setUrlString: './x' previousString: 'http://www.ics.uci.edu/'.
	self assert: url urlString = 'http://www.ics.uci.edu/x'.
	self setUrlString: '././x' previousString: 'http://www.ics.uci.edu/'.
	self assert: url urlString = 'http://www.ics.uci.edu/x'.
	self setUrlString: 'x.' previousString: 'http://www.ics.uci.edu/'.
	self assert: url urlString = 'http://www.ics.uci.edu/x.'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testLastSegmentOf
	#(#('/' '/') #('/foo' 'foo') #('/foo/' 'foo') #('/foo/bar' 'bar') #('/foo/bar/' 'bar') #('http://host/' '/') #('http://host/foo' 'foo') #('http://host/foo/' 'foo') #('http://host/foo/bar' 'bar') #('http://host/foo/bar/' 'bar')) 
		do: [:each | self assert: (SptHTTPUrl lastSegmentOf: each first) = each second].
	#(#('http://host/foo/bar' 'bar') #('http://host/foo/bar?boo' 'bar') #('http://host/foo/bar?boo/' 'bar') #('http://host/foo/bar?boo/boo' 'bar')) 
		do: [:each | self assert: (SptHTTPUrl lastSegmentOf: each first) = each second]
	"#(#('http://host/foo/bar' 'bar') #('http://host/foo/bar#boo' 'bar') #('http://host/foo/bar#boo/' 'bar') #('http://host/foo/bar#boo/boo' 'bar')) 
		do: [:each | self assert: (SptHTTPUrl lastSegmentOf: each first) = each second]"! !

!SptHTTPUrlTest methodsFor: 'accessing' stamp: ''!
setUrlString: aString
	url := SptHTTPUrl fromString: aString! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testPortHTTPS
	self setUrlString: 'https://www.foo.org:1234'.
	self assert: url port = 1234.
	self setUrlString: 'https://www.foo.org/'.
	self assert: url port = 443! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testDoesHostMatch
	#(#('www.foo.org' 'cache.foo.org') #('www.foo.org.au' 'cache.foo.org.au') #('www.FOO.org' 'cache.foo.org')) 
		do: 
			[:each | 
			self assert: (SptHTTPUrl doesHost: each first match: each second).
			self assert: (SptHTTPUrl doesHost: each second match: each first)].
	#(#('' 'cache.foo.org') #('foo' 'cache.foo.org') #('org' 'cache.foo.org') #('www.foo.au' 'cache.foo.au')) 
		do: 
			[:each | 
			self deny: (SptHTTPUrl doesHost: each first match: each second).
			self deny: (SptHTTPUrl doesHost: each second match: each first)]! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testQueryStringsNone
	url := SptHTTPUrl fromString: 'http://foo/bar/baz'.
	self assert: url queryStrings isEmpty.
	url := SptHTTPUrl fromString: 'http://foo/bar/baz?'.
	self assert: url queryStrings isEmpty! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testHasUserInfo
	self setUrlString: 'http://u:p@www.foo.org'.
	self assert: url hasUserInfo.
	self assert: url userInfo = 'u:p'.
	self setUrlString: 'http://@www.foo.org'.
	self assert: url hasUserInfo.
	self assert: url userInfo = ''.
	self setUrlString: 'http://www.foo.org'.
	self deny: url hasUserInfo.
	self assert: url userInfo isNil! !

!SptHTTPUrlTest methodsFor: 'accessing' stamp: ''!
testComparison
	self setFooBar.
	self assert: url = (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self deny: url = (SptHTTPUrl fromString: 'http://www.Notfoo.org/bar').
	self deny: url = (SptHTTPUrl fromString: 'http://www.foo.org/not').
	self deny: url = (SptHTTPUrl fromString: 'http://www.foo.org/bar/not').
	self deny: url = (SptHTTPUrl fromString: 'http://www.foo.org:900/bar').
	url hash! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testQueryStringsWithQ
	| ans |
	url := SptHTTPUrl fromString: 'http://foo/bar/baz?a=b?c'.
	ans := url queryStrings.
	self assert: ans size = 1.
	self assert: ans first = ('a' -> 'b?c')! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testQueryStringsMultiple
	| ans |
	url := SptHTTPUrl fromString: 'http://foo/bar/baz?a=b&c'.
	ans := url queryStrings.
	self assert: ans size = 2.
	self assert: ans first = ('a' -> 'b').
	self assert: ans second = ('c' -> nil).
	""
	url := SptHTTPUrl fromString: 'http://foo/bar/baz?a=b&c=d'.
	ans := url queryStrings.
	self assert: ans size = 2.
	self assert: ans first = ('a' -> 'b').
	self assert: ans second = ('c' -> 'd')! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteRel
	self setUrlString: '../x' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/a/x'.
	self setUrlString: '../../x' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/x'.
	self setUrlString: '../../../x' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/x'.
	self setUrlString: '../foo' previousString: 'http://www.foo.org/bar/a1'.
	self assert: url urlString = 'http://www.foo.org/foo'! !

!SptHTTPUrlTest methodsFor: 'accessing' stamp: ''!
setFooBar
	url := SptHTTPUrl fromString: 'http://www.foo.org/bar'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testCanonicalize
	self assert: 'http://www.foo.org/bar' isCanonicalOf: 'http://www.foo.org/bar'.
	self assert: 'http://www.foo.org/b%20ar' isCanonicalOf: 'http://www.foo.org/b ar'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testPath
	self setUrlString: 'http://www.foo.org'.
	self assert: url path = '/'.
	self setUrlString: 'http://www.foo.org/'.
	self assert: url path = '/'.
	self setUrlString: 'http://www.foo.org/?q'.
	self assert: url path = '/'.
	self setUrlString: 'http://www.foo.org/bar?q'.
	self assert: url path = '/bar'.
	self setUrlString: 'http://www.foo.org/bar/?q'.
	self assert: url path = '/bar/'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteRelDotSlash
	self setUrlString: './x' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/a/b/x'.
	self setUrlString: './../x' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/a/x'.
	self setUrlString: './x/..' previousString: 'http://www.ics.uci.edu/Test/a/b/c'.
	self assert: url urlString = 'http://www.ics.uci.edu/Test/a/b/x'! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testIsDescendentOrSelf
	#(#('http://foo/bar/baz' 'http://foo/bar/') #('http://foo/bar/baz/foo' 'http://foo/bar/') #('/foo' '/') #('/foo/bar' '/') #('/foo/bar/' '/') #('http://foo/bar/' 'http://foo/bar/') #('/' '/')) 
		do: [:each | self assert: (SptHTTPUrl is: each first descendentOrSelfOf: each second)].
	#(#('http://foo/bar/' 'http://foo/baz/')) 
		do: [:each | self deny: (SptHTTPUrl is: each first descendentOrSelfOf: each second)]! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testHost
	self setUrlString: 'http://www.foo.org'.
	self assertIsFoo80.
	self assert: url path = '/'.
	self setUrlString: 'http://www.foo.org/'.
	self assertIsFoo80.
	self assert: url path = '/'.
	self setUrlString: 'http://www.foo.org/bar'.
	self assertIsFoo80.
	self assert: url path = '/bar'.
	self should: [self setUrlString: 'www.foo.org/bar'] raise: Error! !

!SptHTTPUrlTest methodsFor: 'asserting' stamp: ''!
assertIsFoo1234
	self assert: url host = 'www.foo.org'.
	self assert: url port = 1234! !

!SptHTTPUrlTest methodsFor: 'unit tests' stamp: ''!
testMakeAbsoluteLocal
	self setUrlString: '/bar2' previousString: 'http://www.foo.org/baz/bar'.
	self assertIsFoo80.
	self assert: url absolutePath = '/bar2'! !

SptHTTPTests subclass: #SptHTTPCookieJarTest
	instanceVariableNames: 'jar '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPCookieJarTest methodsFor: 'unit tests' stamp: ''!
testHeadersFor
	| cookieString |
	jar addCookie: self getFooRootCookie.
	jar addCookie: self getFooBarCookie.
	cookieString := jar headerFor: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: cookieString size = 'n1=v1;n2=v2' size.
	self assert: ((cookieString subStrings: $;) includes: 'n1=v1').
	self assert: ((cookieString subStrings: $;) includes: 'n2=v2').
	self assert: (jar headerFor: (SptHTTPUrl fromString: 'http://www.foo.org/')) = 'n1=v1'! !

!SptHTTPCookieJarTest methodsFor: 'unit tests' stamp: ''!
testReplaceSameNot
	jar addCookie: self getFooRootCookie.
	self assert: jar numberOfCookies = 1.
	jar addCookie: self getFooBarCookie.
	self assert: jar numberOfCookies = 2! !

!SptHTTPCookieJarTest methodsFor: 'unit tests' stamp: ''!
testReplaceSame
	jar addCookie: self getFooRootCookie.
	self assert: jar numberOfCookies = 1.
	jar addCookie: self getFooRootCookie.
	self assert: jar numberOfCookies = 1! !

!SptHTTPCookieJarTest methodsFor: 'unit tests' stamp: ''!
testHeadersForMultiples
	| cookieString |
	jar addCookie: self getFooRootCookie.
	jar addCookie: self getFooBarMultiplesCookie.
	cookieString := jar headerFor: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: ((cookieString subStrings: $;) includes: 'n1=v1').
	self assert: ((cookieString subStrings: $;) includes: 'n2=v2').
	self assert: ((cookieString subStrings: $;) includes: 'n3=v3')! !

!SptHTTPCookieJarTest methodsFor: 'set up' stamp: ''!
setUp
	super setUp.
	jar := SptHTTPCookieJar new! !

!SptHTTPCookieJarTest methodsFor: 'unit tests' stamp: ''!
testCookiesForExclusion
	| cookies |
	jar addCookie: self getFooBarCookie.
	cookies := jar cookiesFor: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: cookies size = 1.
	cookies := jar cookiesFor: (SptHTTPUrl fromString: 'http://www.foo.org/').
	self assert: cookies isEmpty.
	cookies := jar cookiesFor: (SptHTTPUrl fromString: 'http://www.foo.org/foo').
	self assert: cookies isEmpty! !

!SptHTTPCookieJarTest methodsFor: 'unit tests' stamp: ''!
testCookiesForMultiples
	| cookies |
	jar addCookie: self getFooRootCookie.
	jar addCookie: self getFooBarCookie.
	cookies := jar cookiesFor: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: cookies size = 2.
	cookies := jar cookiesFor: (SptHTTPUrl fromString: 'http://www.foo.org/').
	self assert: cookies size = 1.
	cookies := jar cookiesFor: (SptHTTPUrl fromString: 'http://www.foo.org/foo').
	self assert: cookies size = 1! !

!SptHTTPCookieJarTest methodsFor: 'unit tests' stamp: ''!
testCookiesFor
	| cookies |
	jar addCookie: self getFooRootCookie.
	cookies := jar cookiesFor: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	self assert: cookies size = 1.
	cookies := jar cookiesFor: (SptHTTPUrl fromString: 'http://www.foo.org/').
	self assert: cookies size = 1! !

!SptHTTPCookieJarTest methodsFor: 'accessing' stamp: ''!
getFooRootCookie
	^SptHTTPCookie fromCookieString: 'n1=v1; path=/'
		url: (SptHTTPUrl fromString: 'http://www.foo.org/bar')! !

!SptHTTPCookieJarTest methodsFor: 'accessing' stamp: ''!
getFooBarMultiplesCookie
	^SptHTTPCookie fromCookieString: 'n2=v2; n3=v3; path=/bar'
		url: (SptHTTPUrl fromString: 'http://www.foo.org/bar')! !

!SptHTTPCookieJarTest methodsFor: 'unit tests' stamp: ''!
testReplaceMatching
	jar addCookie: self getFooRootCookie.
	self assert: jar numberOfCookies = 1.
	jar addCookie: (SptHTTPCookie fromCookieString: 'n=v; path=/;domain=.foo.org'
				url: (SptHTTPUrl fromString: 'http://www.foo.org/bar')).
	self assert: jar numberOfCookies = 1.
	self assert: jar cookies asOrderedCollection first domain = '.foo.org'! !

SptHTTPTests subclass: #SptHTTPRequestTest
	instanceVariableNames: 'request sentClose '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendHeader
	self setConnectionReading: ''.
	request sendHeader: 'abc' -> 'def'.
	self assertSentString: 'abc: def
'.
	self setConnectionReading: ''.
	request sendHeader: 'abc' -> 123.
	self assertSentString: 'abc: 123
'! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
cancel
	sentClose := true! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
addResponseParsing: aString
	self setConnectionReading: aString.
	request readResponse! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
getSampleCookie
	^SptHTTPCookie fromCookieString: 'CUSTOMER=WILE_E_COYOTE; path=/; domain=www.foo.org'
		url: (SptHTTPUrl fromString: 'http://www.foo.org/bar')! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
progressMessages
	^request progress sentMessages! !

!SptHTTPRequestTest methodsFor: 'asserting' stamp: ''!
assertIsFoo1234
	self assert: request url host = 'www.foo.org'.
	self assert: request url port = 1234! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testUseProxy
	self deny: request useProxy.
	request proxySettings: SptHTTPProxySettings new.
	self deny: request useProxy.
	request proxySettings: (SptHTTPProxySettings new useProxy: true;yourself).
	self assert: request useProxy.! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
setNewRequest
	request := SptHTTPRequest new! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
addUserAladdin
	SptHTTPUser 
		addRealm: 'WallyWorld'
		host: 'www.foo.org'
		username: 'Aladdin'
		password: 'open sesame'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testHost
	request urlString: 'http://www.foo.org'.
	self assertIsFoo80.
	self assert: request url absolutePath = '/'.
	request urlString: 'http://www.foo.org/'.
	self assertIsFoo80.
	self assert: request url absolutePath = '/'.
	request urlString: 'http://www.foo.org/bar'.
	self assertIsFoo80.
	self assert: request url absolutePath = '/bar'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPortWithProxy
	self setProxy.
	request urlString: 'http://www.foo.org:1234'.
	self assert: request immediateHost = 'aHost'.
	self assert: request immediatePort = 4321! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
getSentString
	^request socketStream writeStream contents asString! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testProgressMessageBody
	self setNewProgressRequest.
	request setEntityOn: 'abc123'.
	request sendMessageBody.
	self assert: self progressMessages first selector = #onSendingEntity.
	self assert: self progressMessages second selector = #sent:of:.
	self assert: self progressMessages second arguments = #(6 6).
	self assert: self progressMessages third selector = #onSentEntity! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendMessageStart10
	self openGetFooBar.
	request beVersion10.
	self setConnectionReading: ''.
	request sendMessageStart.
	self assertSentString: 'GET /bar HTTP/1.0
'! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
setConnectionReading: aString
	request connection: (self getConnectionReading: aString)! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testCanRangeRetry
	self addResponseParsing: 'HTTP/1.1 200 OK
Accept-Ranges: bytes
Content-Length: 0

'.
	self assert: request canRangeRetry! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPrepareAuthorizeNoUser
	self setAuthenticateResponse.
	self should: [request prepareForAuthorize] raise: SptHTTPNotAuthorized! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testTimeoutLoopTerminate
	request startTimeoutLoop.
	self assert: request timeoutLoop notNil.
	request terminateTimeoutLoop.
	self assert: request timeoutLoop isNil.
	request terminateTimeoutLoop.
	request terminateTimeoutLoop! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPrepareRedirectNone
	| coll |
	coll := OrderedCollection new.
	self openGetFooBar.
	self addResponseParsing: 'HTTP/1.0 200

Hi'.
	self deny: (request prepareNextMessage: coll).
	self assert: request status = 200.
	self assert: request responseBody = 'Hi'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendToNonDefaultPort
	self openGetFoo1234Bar.
	self setConnectionReading: ''.
	request setEntityOn: 'abc'.
	request addHeader: 'User-Agent' value: 'Test'.
	request sendMessageStart.
	request sendMessageHeaders.
	self 
		assertSentString: 'GET /bar HTTP/1.1
Host: www.foo.org:1234
User-Agent: Test
Content-Length: 3

'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testWaitTimingEarly
	| conn |
	request connection: (conn := SptHTTPConnection new socket: self).
	request timeout: 200.
	conn timeoutState: #on.
	request deferredSend: [1] deferredValue.
	self assert: request waitOnReady = 1.
	self deny: sentClose! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendGetRequestEntity
	| contents |
	self openGetFooBar.
	self setConnectionReading: ''.
	request setEntityOn: 'abc123'.
	request sendMessageStart.
	request sendMessageHeaders.
	self assert: request hasEntity.
	request sendMessageBody.
	contents := self getSentString.
	self assert: (contents beginsWith: 'GET /bar HTTP/1.1').
	self assert: ('*abc123*' match: contents).
	self assert: ('*Content-Length: 6*' match: contents)! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPrepareForRetryRange
	self openGetFooBar.
	request addRequiredHeaders.
	request okBodyStream: ByteArray writeStream.
	"Ignore the error as it is part of the setup"
	
	[self addResponseParsing: 'HTTP/1.1 200
Content-Length: 10000
Accept-Ranges: bytes

' 
				, (String new: 5000 withAll: $a)] 
			on: Error
			do: [:e | ].
	request prepareForRetry.
	self assert: request isRangeRequest.
	"mmm this depends on SptHTTPResponse(class)>>readEntityChunkSize"
	self assert: (request headerAt: 'Range') = 'bytes=4096-9999'.
	self assert: request okBodyStream position = 0! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testCanRangeRetryNot2
	self addResponseParsing: 'HTTP/1.1 404 NF
Accept-Ranges: bytes

'.
	self deny: request canRangeRetry! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPrepareRedirectLoop
	| coll |
	coll := OrderedCollection new.
	self openGetFooBar.
	coll add: request urlString.
	self addResponseParsing: 'HTTP/1.0 301
Location: http://www.foo.org/bar

Hi Wrong'.
	"SW 20021212 ... changed so that same site can have two requests"
	request prepareNextMessage: coll.
	coll add: 'http://www.foo.org/bar'.
	request prepareNextMessage: coll.
	coll add: 'http://www.foo.org/bar'.
	self should: [request prepareNextMessage: coll] raise: Error! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
setAuthenticateResponse
	self openGetFooBar.
	request responses add: SptHTTPResponse new.
	request lastResponse statusLine: 'HTTP/1.1 401'.
	request lastResponse parseHeaderLine: 'WWW-Authenticate: Basic realm="WallyWorld"'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testTimeoutLoopSyncEx
	request startTimeoutLoop.
	self should: [request startTimeoutLoop] raise: Error! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
setNewProgressRequest
	request := SptHTTPRequest newFor: self getTestConfDeaf.
	self openGetFooBar.
	self setConnectionReading: ''! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testProgressConnect
	self setNewProgressRequest.
	request connect.
	self assert: self progressMessages first selector = #onConnecting.
	self assert: self progressMessages second selector = #onConnected! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendMessageStartGET
	self openGetFooBar.
	self setConnectionReading: ''.
	request sendMessageStart.
	self assertSentString: 'GET /bar HTTP/1.1
'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: 'avi 9/30/2003 20:42'!
testSendToProxy
	self setProxy.
	self openGetFooBar.
	self setConnectionReading: ''.
	request setEntityOn: 'abc'.
	request addHeader: 'User-Agent' value: 'Test'.
	request sendMessageStart.
	request sendMessageHeaders.
	self 
		assertSentString: 'GET http://www.foo.org/bar HTTP/1.1
host: www.foo.org
user-agent: Test
content-length: 3

'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendGetRequestEntityCT
	| contents |
	self openGetFooBar.
	self setConnectionReading: ''.
	request setEntityOn: 'abc123'.
	request entity contentType: 'text/html'.
	request sendMessageStart.
	request sendMessageHeaders.
	self assert: request hasEntity.
	request sendMessageBody.
	contents := self getSentString.
	self assert: (contents beginsWith: 'GET /bar HTTP/1.1').
	self assert: ('*abc123*' match: contents).
	self assert: ('*Content-Length: 6*' match: contents).
	self assert: ('*Content-Type: text/html*' match: contents)! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPrepareRedirectOne
	| coll |
	coll := OrderedCollection new.
	self openGetFooBar.
	coll add: request urlString.
	self addResponseParsing: 'HTTP/1.0 301
Location: http://www.foo.org/baz

Hi Wrong'.
	self assert: (request prepareNextMessage: coll).
	self assert: request status = 301
	"self assert: request responseBody isNil.
	self assert: request responseHeaders isEmpty"! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
openGetFoo1234Bar
	request openGetTo: 'http://www.foo.org:1234/bar'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPort
	request urlString: 'http://www.foo.org:1234'.
	self assertIsFoo1234.
	self assert: request url absolutePath = '/'.
	request urlString: 'http://www.foo.org:1234/'.
	self assertIsFoo1234.
	self assert: request url absolutePath = '/'.
	request urlString: 'http://www.foo.org:1234/bar'.
	self assertIsFoo1234.
	self assert: request url absolutePath = '/bar'.
	request urlString: 'http://www.foo.org:1234/bar?baz'.
	self assertIsFoo1234.
	self assert: request url absolutePath = '/bar?baz'! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
testAddCookieNot
	self openGetFooBar.
	request state addCookie: self getSampleCookie.
	request isUseCookies: false.
	request addRequiredHeaders.
	self deny: (request headers includesKey: 'Cookie')! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPrepareForRetryNoRange
	self openGetFooBar.
	request addRequiredHeaders.
	request okBodyStream: ByteArray writeStream.
	"Ignore the error as it is part of the setup"
	
	[self addResponseParsing: 'HTTP/1.1 200
Content-Length: 10000

' 
				, (String new: 5000 withAll: $a)] 
			on: Error
			do: [:e | ].
	request prepareForRetry.
	self deny: request isRangeRequest.
	self deny: (request headers includesKey: 'Range').
	self assert: request okBodyStream position = 0! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
testAddCookie
	self openGetFooBar.
	request state addCookie: self getSampleCookie.
	request addRequiredHeaders.
	self assert: (request headers at: 'Cookie') = 'CUSTOMER=WILE_E_COYOTE'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPrepareAuthorizeUser
	self setAuthenticateResponse.
	self addUserAladdin.
	request prepareForAuthorize.
	self assert: request state authorizationHeader = 'Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ=='! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
openGetFooBar
	request openGetTo: 'http://www.foo.org/bar'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testWaitTiming
	| conn |
	request connection: (conn := SptHTTPConnection new socket: self).
	request timeout: 200.
	conn timeoutState: #off.
	request deferredSend: 
			[(Delay forMilliseconds: 600) wait.
			1] deferredValue.
	self assert: request waitOnReady = 1.
	self deny: sentClose.
	conn timeoutState: #on.
	request deferredSend: 
			[(Delay forMilliseconds: 4000) wait.
			1] deferredValue.
	request waitOnReady.
	self assert: sentClose! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPrepareForRetryNoneRead
	self openGetFooBar.
	request addRequiredHeaders.
	request okBodyStream: ByteArray writeStream.
	"Ignore the error as it is part of the setup"
	[self addResponseParsing: 'HTTP/1.1 200
Content-Length: 10
Accept-Ranges: bytes

'] 
		on: Error
		do: [:e | ].
	request prepareForRetry.
	self deny: request isRangeRequest.
	self deny: (request headers includesKey: 'Range').
	self assert: request okBodyStream position = 0! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testIsIdempotent
	request := SptHTTPRequest new open: 'GET' to: 'http://www.foo.org/'.
	self assert: request isIdempotent.
	request := SptHTTPRequest new open: 'OPTIONS' to: 'http://www.foo.org/'.
	self assert: request isIdempotent.
	request := SptHTTPRequest new open: 'POST' to: 'http://www.foo.org/'.
	self deny: request isIdempotent! !

!SptHTTPRequestTest methodsFor: 'asserting' stamp: 'avi 9/30/2003 23:28'!
assertSentString: aString
	| sentString |
	self assert: (sentString := self getSentString) asLowercase = aString withInternetLineEndings asLowercase! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testWaitNoTiming
	request timeout: nil.
	request connection: (SptHTTPConnection new socket: self).
	request deferredSend: [100 factorial] deferredValue.
	self assert: request waitOnReady = 100 factorial.
	self deny: sentClose! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testCanRangeRetryNot
	self 
		addResponseParsing: 'HTTP/1.1 200 OK
Accept-Ranges: bytes
transfer-encoding: chunked

0

'.
	self deny: request canRangeRetry! !

!SptHTTPRequestTest methodsFor: 'set up' stamp: ''!
tearDown
	super tearDown.
	request notNil ifTrue: [request terminateTimeoutLoop]! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testCanRangeRetryNoResponse
	self deny: request canRangeRetry! !

!SptHTTPRequestTest methodsFor: 'asserting' stamp: ''!
assertIsFoo80
	self assert: request url host = 'www.foo.org'.
	self assert: request url port = 80! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendWithAuthenticate
	| contents |
	self openGetFooBar.
	self setConnectionReading: ''.
	request username: 'Aladdin' password: 'open sesame'.
	request sendMessageStart.
	request sendMessageHeaders.
	contents := self getSentString.
	self assert: ('*Authorization: Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ==*' match: contents)! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testImmediatesWithProxy
	request urlString: 'http://www.foo.org:1234'.
	self setProxy.
	self assert: request immediateHost = 'aHost'.
	self assert: request immediatePort = 4321.
	request proxySettings useProxy: false.
	self assert: request immediateHost = 'www.foo.org'.
	self assert: request immediatePort = 1234! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
close
	sentClose := true! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendPostRequestEntity
	| contents |
	request openPostTo: 'http://www.foo.org/bar'.
	self setConnectionReading: ''.
	request setEntityOn: 'abc123'.
	request sendMessageStart.
	request sendMessageHeaders.
	request sendMessageBody.
	contents := self getSentString.
	self assert: (contents beginsWith: 'POST /bar HTTP/1.1').
	self assert: ('*abc123*' match: contents).
	self assert: ('*Content-Length: 6*' match: contents)! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testWaitTimingMilliseconds
	| conn |
	request timeout: 20000.
	self assert: request waitTimingMilliseconds = 3000.
	request connection: (conn := SptHTTPConnection new socket: self).
	self assert: request waitTimingMilliseconds = 3000.
	conn timeoutState: #on.
	self assert: request waitTimingMilliseconds = 3000.
	conn checkTimeout.
	self assert: request waitTimingMilliseconds = 20000! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPrepareAuthorizeUserTwice
	self setAuthenticateResponse.
	request addHeader: 'Authorization' value: 'doesntmatter'.
	self addUserAladdin.
	self should: [request prepareForAuthorize] raise: SptHTTPNotAuthorized! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendAbsentHeader
	self setConnectionReading: ''.
	request addIfAbsentHeader: 'abc' value: 'def'.
	self assert: (request headers at: 'abc') = 'def'.
	self setConnectionReading: ''.
	request addHeader: 'abc' value: 123.
	request addIfAbsentHeader: 'abc' value: 'def'.
	self assert: (request headers at: 'abc') = 123! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
setProxy
	request proxySettings: (SptHTTPProxySettings new
		useProxy: true;
		proxyHost: 'aHost';
		proxyPort: 4321;
		yourself)! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testStatus
	self addResponseParsing: 'HTTP/1.1 500 NotGood

'.
	self assert: request status = 500.
	self assert: request statusText = 'NotGood'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendGetRequest
	| contents |
	self openGetFooBar.
	self setConnectionReading: ''.
	request sendMessageStart.
	request sendMessageHeaders.
	contents := self getSentString.
	self assert: (contents beginsWith: 'GET /bar HTTP/1.1').
	self deny: ('*Content-Length:*' match: contents)! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testSendWithEntity
	self openGetFooBar.
	self setConnectionReading: ''.
	request setEntityOn: 'abc'.
	request addHeader: 'User-Agent' value: 'Test'.
	request sendMessageStart.
	request sendMessageHeaders.
	self 
		assertSentString: 'GET /bar HTTP/1.1
Host: www.foo.org
User-Agent: Test
Content-Length: 3

'! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testUsernamePassword
	request urlString: 'http://www.foo.org/bar/index.html'.
	request username: 'Aladdin' password: 'open sesame'.
	self assert: request state credentials user username = 'Aladdin'.
	self assert: request state credentials user password = 'open sesame'.
	self assert: (request state shouldAuthenticate: 'http://www.foo.org/bar/index.html').
	self assert: (request state shouldAuthenticate: 'http://www.foo.org/bar/baz/index.html').
	self deny: (request state shouldAuthenticate: 'http://www.foo.org/baz').
	self assert: request state authorizationHeader = 'Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ=='! !

!SptHTTPRequestTest methodsFor: 'accessing' stamp: ''!
testEntityString
	request entityString: 'Hi'.
	self assert: request entity class = SptHTTPEntity.
	self assert: request entity stream contents asString = 'Hi'.
	self assert: request entity contentLength = 2.
	self deny: request entity hasContentType! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testProgressMessageHeaders
	self setNewProgressRequest.
	request sendMessageHeaders.
	self assert: self progressMessages first selector = #onSendingHeaders.
	self assert: self progressMessages second selector = #onSentHeaders! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testPrepareRedirectDiffHost
	self openGetFooBar.
	request username: 'Aladdin' password: 'open sesame'.
	request addRequiredHeaders.
	self assert: (request headers includesKey: 'Authorization').
	self addResponseParsing: 'HTTP/1.0 301
Location: http://www.Notfoo.org/baz

Hi Wrong'.
	request prepareForRedirect.
	self deny: (request headers includesKey: 'Authorization')! !

!SptHTTPRequestTest methodsFor: 'unit tests' stamp: ''!
testConfAccessing
	| conf |
	conf := SptHTTPConfiguration newDefault.
	conf sendEntityBufferSize: 1.
	conf readEntityBufferSize: 2.
	request := SptHTTPRequest newFor: conf.
	self assert: request sendEntityBufferSize = 1.
	self assert: request readEntityBufferSize = 2! !

!SptHTTPRequestTest methodsFor: 'set up' stamp: ''!
setUp
	super setUp.
	self setNewRequest.
	sentClose := false! !

SptHTTPTests subclass: #SptHTTPConnectionTest
	instanceVariableNames: 'connection sentClose '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPConnectionTest methodsFor: 'unit tests' stamp: ''!
testCheckTimeout
	self assert: connection timeoutState = #off.
	connection checkTimeout.
	self deny: sentClose.
	self assert: connection timeoutState = #off.
	connection beTimeoutOn.
	self assert: connection timeoutState = #on.
	connection checkTimeout.
	self deny: sentClose.
	self assert: connection timeoutState = #wait.
	connection checkTimeout.
	self assert: sentClose! !

!SptHTTPConnectionTest methodsFor: 'unit tests' stamp: ''!
testIsStale
	self assert: connection isStale.
	connection setKeepAliveTime.
	self deny: connection isStale! !

!SptHTTPConnectionTest methodsFor: 'accessing' stamp: ''!
cancel
	sentClose := true! !

!SptHTTPConnectionTest methodsFor: 'unit tests' stamp: ''!
testCheckTimeoutOff
	connection beTimeoutOn.
	connection checkTimeout.
	self deny: sentClose.
	self assert: connection timeoutState = #wait.
	connection beTimeoutOff.
	connection checkTimeout.
	self assert: connection timeoutState = #off.
	self deny: sentClose! !

!SptHTTPConnectionTest methodsFor: 'set up' stamp: ''!
setUp
	super setUp.
	connection := (SptHTTPConnection new)
				port: 80 host: 'www.foo.org';
				yourself.
	sentClose := false.
	connection socket: self! !

!SptHTTPConnectionTest methodsFor: 'unit tests' stamp: ''!
testHostPortString
	self assert: connection hostPortString = 'www.foo.org:80'! !

SptHTTPTests subclass: #SptHTTPResponseTest
	instanceVariableNames: 'response '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testIsConnectionCloseEx
	response statusLine: 'HT'.
	self assert: response isConnectionClose.
	response statusLine: nil.
	self assert: response isConnectionClose! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testRangeInterval
	response headers at: 'Content-Length' put: '10000'.
	""
	response headers at: 'Range' put: 'bytes=0-499'.
	self assert: response rangeInterval = (0 to: 499).
	""
	response headers at: 'Range' put: 'bytes=500-999'.
	self assert: response rangeInterval = (500 to: 999).
	""
	response headers at: 'Range' put: 'bytes=-500'.
	self assert: response rangeInterval = (9500 to: 9999).
	""
	response headers at: 'Range' put: 'bytes=9500-'.
	self assert: response rangeInterval = (9500 to: 9999)! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testIsUnauthorized
	response statusLine: 'HTTP/1.1 401 Unauthorized'.
	self assert: response isUnauthorized! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testBasicChallenge
	response statusLine: 'HTTP/1.1 401'.
	self assert: response basicChallenge isNil.
	response parseHeaderLine: 'WWW-Authenticate: Basic realm="WallyWorld"'.
	self assert: response basicChallenge realm = 'WallyWorld'.
	response 
		parseHeaderLine: 'WWW-Authenticate: Digest realm="WallyWorld", nonce="eb6c05fd7dd274e003a58dfc5092b305"'.
	self assert: response basicChallenge realm = 'WallyWorld'! !

!SptHTTPResponseTest methodsFor: 'asserting' stamp: ''!
assertHeader: aHeaderLine isKey: aKeyString value: aValueString
	response := SptHTTPResponse new.
	response parseHeaderLine: aHeaderLine.
	self assert: (response headerAt: aKeyString) = aValueString! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testReadContentLength
	self setResponseOn: 'HTTP/1.1 200
Content-Length: 3

123'.
	self assert: response body = '123'! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testContentRangeInterval
	response headers at: 'Content-Length' put: 10000.
	""
	response headers at: 'Content-Range' put: 'bytes 0-499/1234'.
	self assert: response contentRangeInterval = (0 to: 499).
	""
	response headers at: 'Content-Range' put: 'bytes 500-999/1234'.
	self assert: response contentRangeInterval = (500 to: 999).
	""
	response headers at: 'Content-Range' put: 'bytes 500-999/*'.
	self assert: response contentRangeInterval = (500 to: 999).
	""
	response headers at: 'Content-Range' put: 'bytes  500-999/*'.
	self assert: response contentRangeInterval = (500 to: 999)! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testResponseContentLength
	response parseHeaderLine: 'Content-Length: 456'.
	self assert: response contentLength = 456! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testProgressReadMessageHeaders
	self setTestProgress.
	response 
		readMessageHeaders: (self getConnectionReading: 'HTTP/1.1 200
Content-Length: 3

').
	self assert: self progressMessages first selector = #onReadingHeaders.
	self assert: self progressMessages last selector = #onReadHeaders! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testIsChunkedTransferCoding
	response statusLine: 'HTTP/1.1 200 OK'.
	self deny: response isChunkedTransferEncoding.
	response parseHeaderLine: 'transfer-encoding: chunked'.
	self assert: response isChunkedTransferEncoding! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
progressMessages
	^response progress sentMessages! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testIsRedirect
	response statusLine: 'HTTP/1.1 301 Moved Permanently'.
	self deny: response isRedirect.
	response parseHeaderLine: 'Location: http://www.foo.org/baz'.
	self assert: response isRedirect.
	response statusLine: 'HTTP/1.1 200 OK'.
	self deny: response isRedirect! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testIsInformational
	response statusLine: 'HTTP/1.1 100 Continue'.
	self assert: response isInformational.
	response statusLine: 'HTTP/1.1 200 OK'.
	self deny: response isInformational! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testCombiningHeaders
	self 
		setResponseOn: 'HTTP/1.1 200
WWW-Authenticate: BASIC realm="www.sharemation.com"
WWW-Authenticate: Digest realm="www.sharemation.com", nonce="eb6c05fd7dd274e003a58dfc5092b305"
Content-Length: 0

'.
	self assert: (response headerAt: 'WWW-Authenticate') 
				= 'BASIC realm="www.sharemation.com", Digest realm="www.sharemation.com", nonce="eb6c05fd7dd274e003a58dfc5092b305"'! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testReadContentRangeBodyAll
	| request stream |
	request := SptHTTPRequest new.
	"Set-up a request as if we had read nothing of previous request"
	request isRangeRequest: true.
	stream := ByteArray writeStream.
	request okBodyStream: stream.
	request responses add: response.
	response request: request.
	self 
		setResponseOn: 'HTTP/1.1 206 PC
Content-Length: 10
Content-Range: bytes 0-9/10

helloagain'.
	"Retry"
	self assert: stream contents asString = 'helloagain'! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: 'avi 9/30/2003 20:20'!
testNextLineFrom
	| stream |
	stream := SZReadWriteSocketStream testStreamReading: 'abc' , String crlf.
	self assert: stream upToCRLF = 'abc'.
	stream := SZReadWriteSocketStream testStreamReading: String crlf.
	self assert: stream upToCRLF isEmpty! !

!SptHTTPResponseTest methodsFor: 'asserting' stamp: ''!
setTestProgress
	| request |
	request := SptHTTPRequest newFor: self getTestConfDeaf.
	response request: request! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testIsRedirectCase
	response statusLine: 'HTTP/1.1 301 Moved Permanently'.
	response parseHeaderLine: 'loCatIon: http://www.foo.org/baz'.
	self assert: response isRedirect! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testDefaultIsConnectionClose
	response statusLine: 'HTTP/1.1 200 OK'.
	self deny: response defaultIsConnectionClose.
	response statusLine: 'HTTP/1.0 200 OK'.
	self assert: response defaultIsConnectionClose! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testProgressReadMessageStatus
	self setTestProgress.
	response readMessageStatus: (self getConnectionReading: 'HTTP/1.1 401' , self crlf).
	self assert: self progressMessages first selector = #onReadingStatus.
	self assert: self progressMessages last selector = #onReadStatus! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testReadToEnd
	self setResponseOn: 'HTTP/1.0 200

123'.
	self assert: response body = '123'! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testReadChunkedCoding
	self setResponseOn: 'HTTP/1.1 200
transfer-encoding: chunked

3
abc
2
de
0

'.
	self assert: response body = 'abcde'! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testProgressReadBodyToEnd
	| body |
	body := String new: 10000 withAll: $a.
	self setTestProgress.
	response readBodyToEnd: (self getConnectionReading: body).
	self assert: self progressMessages first selector = #onReadingBody.
	self assert: self progressMessages second selector = #read:of:.
	self assert: self progressMessages second arguments 
				= (Array with: response readEntityBufferSize with: nil).
	self assert: self progressMessages third selector = #read:of:.
	self assert: self progressMessages third arguments 
				= (Array with: response readEntityBufferSize * 2 with: nil).
	self assert: self progressMessages last selector = #onReadBody! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testIsAcceptRangesBytes
	response statusLine: 'HTTP/1.1 200 OK'.
	response parseHeaderLine: 'Accept-Ranges: bytes'.
	self assert: response isAcceptRangesBytes! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testProgressReadContentLength2
	| body |
	body := String new: 10000 withAll: $a.
	response parseHeaderLine: 'Content-Length: 10000' , self crlf.
	self setTestProgress.
	response readContentLengthBody: (self getConnectionReading: body).
	self assert: self progressMessages first selector = #onReadingBody.
	self assert: self progressMessages second selector = #read:of:.
	self assert: self progressMessages second arguments 
				= (Array with: response readEntityBufferSize with: 10000).
	self assert: self progressMessages third selector = #read:of:.
	self assert: self progressMessages third arguments 
				= (Array with: response readEntityBufferSize * 2 with: 10000).
	self assert: self progressMessages last selector = #onReadBody! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testProgressReadContentLength1
	response parseHeaderLine: 'Content-Length: 3' , self crlf.
	self setTestProgress.
	response readContentLengthBody: (self getConnectionReading: 'abc').
	self assert: self progressMessages first selector = #onReadingBody.
	self assert: self progressMessages second selector = #read:of:.
	self assert: self progressMessages second arguments = #(3 3).
	self assert: self progressMessages last selector = #onReadBody! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testHasContentLength
	self deny: response hasContentLength.
	response parseHeaderLine: 'Content-Length: 456'.
	self assert: response hasContentLength! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testBasicChallengeLast
	response statusLine: 'HTTP/1.1 401'.
	self assert: response basicChallenge isNil.
	response 
		parseHeaderLine: 'WWW-Authenticate: Digest realm="WallyWorld", nonce="eb6c05fd7dd274e003a58dfc5092b305"'.
	response parseHeaderLine: 'WWW-Authenticate: Basic realm="WallyWorld"'.
	self assert: response basicChallenge realm = 'WallyWorld'! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testAddResponseHeaderLine
	self 
		assertHeader: 'Content-Length: 456'
		isKey: 'Content-Length'
		value: '456'.
	self 
		assertHeader: 'Content-Length:  456'
		isKey: 'Content-Length'
		value: '456'.
	self 
		assertHeader: 'Content-Length: 456 '
		isKey: 'Content-Length'
		value: '456'.
	self 
		assertHeader: 'CONTENT-LENGTH: 456'
		isKey: 'Content-Length'
		value: '456'! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testReadContentRangeBody
	| request stream |
	request := SptHTTPRequest new.
	"Set-up a request as if we had a partial read error retry"
	request isRangeRequest: true.
	stream := ByteArray writeStream.
	stream nextPutAll: 'hello' asByteArray.
	stream reset.
	request okBodyStream: stream.
	request responses add: response.
	response request: request.
	self 
		setResponseOn: 'HTTP/1.1 206 PC
Content-Length: 5
Content-Range: bytes 5-9/10

again'.
	"Retry"
	self assert: stream contents asString = 'helloagain'! !

!SptHTTPResponseTest methodsFor: 'accessing' stamp: ''!
setResponseOn: aString
	response readResponse: (self getConnectionReading: aString)! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testSetCookieHeaders
	self setTestProgress.
	response request setUrl: (SptHTTPUrl fromString: 'http://www.foo.org/bar').
	response parseHeaderLine: 'Set-Cookie: CUSTOMER=WILE_E_COYOTE; path=/'.
	self assert: response request cookieJar numberOfCookies = 1.
	response parseHeaderLine: 'Set-Cookie: second=2; path=/bar'.
	self assert: response request cookieJar numberOfCookies = 2! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testIsConnectionCloseLC
	response statusLine: 'HTTP/1.1 200 OK'.
	response parseHeaderLine: 'Connection: Close'.
	self assert: response isConnectionClose! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testHasContentLengthCase
	self deny: response hasContentLength.
	response parseHeaderLine: 'CONTENT-LENGTH: 456'.
	self assert: response hasContentLength! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: 'avi 9/30/2003 20:38'!
testProgressReadChunkedCoding
	self setTestProgress.
	response readChunkedEncodedBody: (self getConnectionReading: '3
abc
2
de
0

').
	self assert: self progressMessages first selector = #onReadingBody.
	self assert: self progressMessages second selector = #read:of:.
	self assert: self progressMessages second arguments = {3. nil}.
	self assert: self progressMessages third selector = #read:of:.
	self assert: self progressMessages third arguments = {5. nil}.
	self assert: self progressMessages last selector = #onReadBody! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testBasicChallengeCase
	response statusLine: 'HTTP/1.1 401'.
	self assert: response basicChallenge isNil.
	response parseHeaderLine: 'WWW-Authenticate: BASIC realm="WallyWorld"'.
	self assert: response basicChallenge realm = 'WallyWorld'! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testIsConnectionClose
	response statusLine: 'HTTP/1.1 200 OK'.
	self deny: response isConnectionClose.
	response parseHeaderLine: 'Connection: close'.
	self assert: response isConnectionClose! !

!SptHTTPResponseTest methodsFor: 'asserting' stamp: ''!
setUp
	super setUp.
	response := SptHTTPResponse new! !

!SptHTTPResponseTest methodsFor: 'unit tests' stamp: ''!
testReadChunkedSizeFrom
	| stream |
	stream := SZReadWriteSocketStream testStreamReading: '5
abcde'.
	self assert: (response chunkSizeFrom: stream) = 5.
	self assert: stream next = $a.
	stream := SZReadWriteSocketStream testStreamReading: '5;foo
abcde'.
	self assert: (response chunkSizeFrom: stream) = 5.
	self assert: stream next = $a.
	stream := SZReadWriteSocketStream testStreamReading: 'A
abcde'.
	self assert: (response chunkSizeFrom: stream) = 10.
	self assert: stream next = $a.
	stream := SZReadWriteSocketStream testStreamReading: 'a
abcde'.
	self assert: (response chunkSizeFrom: stream) = 10.
	self assert: stream next = $a! !

!SptHTTPTests methodsFor: 'unclassified' stamp: ''!
getFooBarCookie
	^SptHTTPCookie fromCookieString: 'n2=v2; path=/bar'
		url: (SptHTTPUrl fromString: 'http://www.foo.org/bar')! !

SptHTTPTests subclass: #SptHTTPEntityTest
	instanceVariableNames: 'entity '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPEntityTest methodsFor: 'unit tests' stamp: ''!
testBeTextPlain
	entity beTextPlain.
	self assert: entity contentType = 'text/plain'! !

!SptHTTPEntityTest methodsFor: 'unit tests' stamp: ''!
testBeAppOctetStream
	entity beAppOctetStream.
	self assert: entity contentType = 'application/octet-stream'! !

!SptHTTPEntityTest methodsFor: 'unit tests' stamp: ''!
testHasContentType
	self deny: entity hasContentType.
	entity contentType: 'text/xml'.
	self assert: entity hasContentType! !

!SptHTTPEntityTest methodsFor: 'unit tests' stamp: ''!
testBeTextHtml
	entity beTextHtml.
	self assert: entity contentType = 'text/html'! !

!SptHTTPEntityTest methodsFor: 'unit tests' stamp: ''!
setUp
	super setUp.
	entity := SptHTTPEntity fromString: 'foo'! !

!SptHTTPEntityTest methodsFor: 'unit tests' stamp: ''!
testConstructionString
	self assert: entity stream contents = 'foo' asByteArray.
	self assert: entity contentLength = 3! !

!SptHTTPEntityTest methodsFor: 'unit tests' stamp: ''!
testBeTextXml
	entity beTextXml.
	self assert: entity contentType = 'text/xml'! !

!SptHTTPEntityTest methodsFor: 'unit tests' stamp: ''!
testConstructionBytes
	entity := SptHTTPEntity fromByteObject: 'foo'.
	self assert: entity stream contents = 'foo' asByteArray.
	self assert: entity contentLength = 3! !

SptHTTPTests subclass: #SptHTTPChallengeTest
	instanceVariableNames: 'challenges challenge '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Tests'!

!SptHTTPChallengeTest methodsFor: 'unit tests' stamp: ''!
testIsInProtectionSpace
	challenge := SptHTTPChallenge new.
	challenge rootUri: 'http://www.foo.org'.
	self assert: (challenge isInProtectionSpace: 'http://www.foo.org/').
	self assert: (challenge isInProtectionSpace: 'http://www.foo.org/bar').
	self assert: (challenge isInProtectionSpace: 'http://www.foo.org/bar/').
	self assert: (challenge isInProtectionSpace: 'http://www.foo.org/bar/foo').
	self deny: (challenge isInProtectionSpace: 'http://www.NOTfoo.org/bar')! !

!SptHTTPChallengeTest methodsFor: 'unit tests' stamp: ''!
testCredentialsFor
	| credentials |
	challenge := (SptHTTPChallenge readAllFrom: 'Basic realm="WallyWorld"' readStream) first.
	credentials := challenge 
				credentialsFor: (SptHTTPUser username: 'wally' password: 'world').
	self assertCredentials: credentials.
	credentials := challenge credentialsForUsername: 'wally' password: 'world'.
	self assertCredentials: credentials! !

!SptHTTPChallengeTest methodsFor: 'unit tests' stamp: ''!
testIsInProtectionSpaceWithDomain
	challenge := SptHTTPChallenge new.
	challenge rootUri: 'http://www.foo.org'.
	#('/' '/bar' 'bar' '/ /bar') do: 
			[:each | 
			challenge params at: 'domain' put: each.
			self assert: (challenge isInProtectionSpace: 'http://www.foo.org/bar').
			self assert: (challenge isInProtectionSpace: 'http://www.foo.org/baz').
			self deny: (challenge isInProtectionSpace: 'http://www.NOTfoo.org/bar')].
	#('/bar/' '/bar/foo') do: 
			[:each | 
			challenge params at: 'domain' put: each.
			self assert: (challenge isInProtectionSpace: 'http://www.foo.org/bar/foo').
			self assert: (challenge isInProtectionSpace: 'http://www.foo.org/bar/foo/').
			self assert: (challenge isInProtectionSpace: 'http://www.foo.org/bar/foo/foo').
			self deny: (challenge isInProtectionSpace: 'http://www.foo.org/baz').
			self deny: (challenge isInProtectionSpace: 'http://www.NOTfoo.org/bar')].
	challenge params at: 'domain' put: '/bar /baz'.
	self assert: (challenge isInProtectionSpace: 'http://www.foo.org/bar').
	self assert: (challenge isInProtectionSpace: 'http://www.foo.org/baz').
	self deny: (challenge isInProtectionSpace: 'http://www.NOTfoo.org/bar').
	challenge params at: 'domain' put: 'http://www.foo.org/bar/baz'.
	self assert: (challenge isInProtectionSpace: 'http://www.foo.org/bar/baz').
	self deny: (challenge isInProtectionSpace: 'http://www.foo.org/baz').
	self deny: (challenge isInProtectionSpace: 'http://www.NOTfoo.org/bar')! !

!SptHTTPChallengeTest methodsFor: 'unit tests' stamp: ''!
assertCredentials: credentials
	self assert: (credentials isKindOf: SptBasicCredentials).
	self assert: credentials challenge == challenge.
	self assert: credentials username = 'wally'.
	self assert: credentials password = 'world'! !

!SptHTTPChallengeTest methodsFor: 'unit tests' stamp: ''!
testAbsoluteUrl
	challenge := SptHTTPChallenge new.
	challenge absoluteUrl: 'http://www.foo.org/'.
	self assert: challenge rootUri = 'http://www.foo.org'.
	self assert: challenge domain = '/'.
	challenge := SptHTTPChallenge new.
	challenge absoluteUrl: 'http://www.foo.org/foo'.
	self assert: challenge rootUri = 'http://www.foo.org'.
	self assert: challenge domain = '/foo'! !

!SptHTTPChallengeTest methodsFor: 'unit tests' stamp: ''!
testAccessing
	| auth |
	challenge := SptHTTPChallenge new.
	challenge realm: 'foo'.
	challenge scheme: 'Basic'.
	auth := SptHTTPNotAuthorized new challenge: challenge.
	self assert: auth realm = 'foo'! !

!SptHTTPChallengeTest methodsFor: 'unit tests' stamp: ''!
testDigest
	challenges := SptHTTPChallenge 
				readAllFrom: 'Digest realm="testrealm@host.com", qop="auth,auth-int", nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093", opaque="5ccc069c403ebaf9f0171e9517f40e41"' 
						readStream.
	self assert: challenges size = 1.
	challenge := challenges first.
	self assert: challenge realm = 'testrealm@host.com'.
	self assert: challenge qop = 'auth,auth-int'.
	self assert: challenge nonce = 'dcd98b7102dd2f0e8b11d0f600bfb0c093'.
	self assert: challenge opaque = '5ccc069c403ebaf9f0171e9517f40e41'! !

!SptHTTPChallengeTest methodsFor: 'unit tests' stamp: ''!
testBasic
	challenges := SptHTTPChallenge readAllFrom: 'Basic realm="WallyWorld"' readStream.
	self assert: challenges size = 1.
	challenge := challenges first.
	self assert: challenge realm = 'WallyWorld'.
	self assert: challenge isBasic.
	self assert: challenge params isEmpty! !

!SptHTTPChallengeTest methodsFor: 'unit tests' stamp: ''!
testTwoChallenges
	challenges := SptHTTPChallenge 
				readAllFrom: 'Basic realm="WallyWorld",foo1="bar1",foo2="bar2",Digest realm="WallyLand",foo3="bar3",foo4="bar4"' 
						readStream.
	self assert: challenges size = 2.
	challenge := challenges first.
	self assert: challenge realm = 'WallyWorld'.
	self assert: challenge isBasic.
	self assert: (challenge params at: 'foo1') = 'bar1'.
	self assert: (challenge params at: 'foo2') = 'bar2'.
	challenge := challenges second.
	self assert: challenge realm = 'WallyLand'.
	self deny: challenge isBasic.
	self assert: (challenge params at: 'foo3') = 'bar3'.
	self assert: (challenge params at: 'foo4') = 'bar4'! !

!SptHTTPChallengeTest methodsFor: 'unit tests' stamp: ''!
testBasicParams
	challenges := SptHTTPChallenge 
				readAllFrom: 'Basic realm="WallyWorld",foo1="bar1",foo2="bar2" ' readStream.
	self assert: challenges size = 1.
	challenge := challenges first.
	self assert: challenge realm = 'WallyWorld'.
	self assert: challenge isBasic.
	self assert: (challenge params at: 'foo1') = 'bar1'.
	self assert: (challenge params at: 'foo2') = 'bar2'! !

Dictionary subclass: #LookupTable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Compatibility'!

LookupTable subclass: #SharedLookupTable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Compatibility'!

!LookupTable methodsFor: 'as yet unclassified' stamp: 'avi 9/29/2003 14:47'!
associationsDo: aBlock separatedBy: separationBlock
	| doBlock |
	doBlock _ [doBlock _ separationBlock].
	self associationsDo:
		[:assoc |
		doBlock value.
		aBlock value: assoc]! !

LookupTable subclass: #PluggableLookupTable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Compatibility'!

!PluggableLookupTable methodsFor: 'as yet unclassified' stamp: 'avi 9/29/2003 14:28'!
at: aString
	^ super at: aString asLowercase! !

!PluggableLookupTable methodsFor: 'as yet unclassified' stamp: 'avi 9/29/2003 13:25'!
searchPolicy: anObject! !

!PluggableLookupTable methodsFor: 'as yet unclassified' stamp: 'avi 9/29/2003 14:28'!
at: aString put: anObject
	^ super at: aString asLowercase put: anObject! !

!PluggableLookupTable methodsFor: 'as yet unclassified' stamp: 'avi 9/30/2003 01:57'!
at: aString ifAbsent: aBlock
	^ super at: aString asLowercase ifAbsent: aBlock! !

IdentitySet subclass: #SharedIdentitySet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Compatibility'!

!BlockContext methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 20:22'!
deferredValue
	^ self! !

!PositionableStream methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 13:18'!
nextAvailable: aNumber
	^ self next: aNumber! !

!PositionableStream methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 02:09'!
nextBytes: anInteger
	^self 
		next: anInteger
		into: (ByteArray new: anInteger)
		startingAt: 1! !

!PositionableStream methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 13:17'!
skipWhile: aBlock
	[(aBlock value: self peek) and: [self atEnd not]]
		whileTrue: [self next]! !

!Socket methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 23:07'!
cancel
	^ self closeAndDestroy ! !

!Socket methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 23:07'!
isOpen
	^ self isConnected! !

!SmallInteger methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 02:05'!
mask: aNumber set: aBoolean
	^ aBoolean
		ifTrue: [self bitOr: aNumber]
		ifFalse: [self bitAnd: aNumber bitInvert]! !

!SmallInteger methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:27'!
| aSmallInteger
	^ self bitOr: aSmallInteger! !

!Number methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 20:36'!
% aNumber
	^ self \\ aNumber! !

!Number methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:36'!
printStringRadix: aNumber showRadix: aBoolean
	| str |
	str _ self printStringBase: aNumber.
	^ (aBoolean or: [aNumber = 10])
		ifTrue: [str]
		ifFalse: [str copyAfter: $r]! !

!WriteStream methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:48'!
display: anObject
	self nextPutAll: anObject displayString! !

!Interval methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 02:10'!
start
	^ self first! !

!Interval methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 02:10'!
stop
	^ self last! !

ProtoObject subclass: #DeafObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWHTTPClient-Compatibility'!

!DeafObject class methodsFor: 'as yet unclassified' stamp: 'avi 9/30/2003 01:33'!
current
	^ self new! !

!DeafObject methodsFor: 'as yet unclassified' stamp: 'avi 9/30/2003 01:33'!
doesNotUnderstand: aMessage
	^ self! !

!ReadStream class methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 02:08'!
endOfStreamSignal
	^ nil! !

!Character class methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 20:08'!
codePoint: aNumber
	^ aNumber ifNotNil: [self value: aNumber]! !

!String methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:43'!
match: aString ignoreCase: aBoolean
	aBoolean ifFalse: [self notYetImplemented].
	^ self match: aString! !

!String methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:22'!
byteArrayMap
	^ (CharacterSet newFrom: self) byteArrayMap! !

!String methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 20:30'!
displayString
	^ self! !

!String methodsFor: '*swhttpclient' stamp: ''!
trimBlanks
	| first last |
	self isEmpty ifTrue: [^self].
	first := 1.
	last := self size.
	[(self at: first) isSeparator and: [first < last]] 
		whileTrue: [first := first + 1].
	[(self at: last) isSeparator and: [last >= first]] 
		whileTrue: [last := last - 1].
	^self copyFrom: first to: last! !

!String methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:23'!
indexOfSubCollection: aString
	^ self findString: aString! !

!String methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 13:22'!
subStrings: aCharacter
	^ self findTokens: aCharacter asString! !

!String methodsFor: '*swhttpclient' stamp: 'avi 9/29/2003 14:23'!
startsWith: aString
	^ self beginsWith: aString! !

!String class methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 20:37'!
lineDelimiter 
	^ self crlf! !

!ProcessorScheduler methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 20:23'!
forkMainIfMain! !

!Integer class methodsFor: '*swhttpclient' stamp: 'avi 9/30/2003 02:07'!
readFrom: aStream radix: aNumber
	^ self readFrom: aStream base: aNumber! !
SptHTTPConnection initialize!
SptHTTPRequest initialize!
SptHTTPSConnection initialize!
SptHTTPUrl initialize!
SptHTTPUser initialize!
